#!/bin/bash

### SIDfm VM Server ###

### Defines ###

# load profile
[ -f /etc/profile.d/lang.sh ] && source /etc/profile.d/lang.sh
[ -f /etc/profile.d/railsenv.sh ] && source /etc/profile.d/railsenv.sh
[ -f /etc/profile.d/rh-ruby23.sh ] && source /etc/profile.d/rh-ruby23.sh
[ -f /etc/profile.d/proxy.sh ] && source /etc/profile.d/proxy.sh
[ -f /etc/profile.d/timezone.sh ] && source /etc/profile.d/timezone.sh

# Paths
BASE_PATH=/var/lib/redmine
PLUGIN_PATH=${BASE_PATH}/plugins/sidfm-vm-server
LOGFILE=${BASE_PATH}/log/vm_server.log

# polling interval (sec.)
INTERVAL=5

# issue update interval (* INTERVAL .sec)
ISSUE_UPDATE_INTERVAL=120

# status flags
FLG_PREFIX="/flag."
FLG_E_APACHE="enable-apache"
FLG_E_ISSUE_UPDATE="enable-issue-update"

# lock path
L_VM_DAEMON=/var/run/vm_daemon.lock

# cache path
CACHE_PATH=/tmp/sidfmcache

### functions ###
# logger
function log() {
    mes="$*"
    echo -n `date +'%Y/%m/%d %H:%M:%S'` >> $LOGFILE
    echo " [vm_server] ${mes}" >> $LOGFILE
}
# sweep cache
function sweep_cache() {
    find ${CACHE_PATH} -type f -mtime +3 -exec rm -rf '{}' \; 2>&1 >/dev/null
}

### initial ###
# counter init
ISSUE_UPDATE_COUNTER=$ISSUE_UPDATE_INTERVAL
# for Apache httpd
rm -rf /var/run/httpd
mkdir /var/run/httpd
chown apache:apache /var/run/httpd
# for Rails (file_cache)
rm -rf ${CACHE_PATH}
mkdir ${CACHE_PATH}
chown apache:apache ${CACHE_PATH}

### main loop ###
while true; do
    sleep $INTERVAL

    # debug
    #log "${ISSUE_UPDATE_COUNTER}"

    sweep_cache

    # enable-apache
    if [ -f "${FLG_PREFIX}${FLG_E_APACHE}" ]; then
	pgrep -F /var/run/httpd/httpd.pid || httpd -k start
    else
	pgrep -F /var/run/httpd/httpd.pid && kill `cat /var/run/httpd/httpd.pid`
    fi

    # enable-issue-update
    if [ -f "${FLG_PREFIX}${FLG_E_ISSUE_UPDATE}" ]; then
	ISSUE_UPDATE_COUNTER=$((ISSUE_UPDATE_COUNTER-1))
	#log "counter_decrement: ${ISSUE_UPDATE_COUNTER}"
	if [ "${ISSUE_UPDATE_COUNTER}" == "0" ]; then
	    cd ${BASE_PATH}
	    bundle exec rake db:sessions:trim 2>&1 >/dev/null
	    log "vm_daemon - invoke"
	    cd ${BASE_PATH}
	    flock ${L_VM_DAEMON} ${PLUGIN_PATH}/ROR/bin/vm_daemon_invoke >> $LOGFILE 2>&1
	    ISSUE_UPDATE_COUNTER=$ISSUE_UPDATE_INTERVAL
	fi
    fi

done
