# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles

  before_action :get_view_state
  after_action :put_view_state

  # 権限チェック
  def check_roles
    @role_admin = @sidfm_roles.include?(:admin)
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? 'dashboard/all' : "dashboard/categories/#{User.current.id}"
  end

  def create_hosts_all_content
    @data[:hosts] = {
      :all_content => SidfmdHost.create_all_content(category_id: 0, role_admin: @role_admin, user_host_ids: @user_host_ids, user_category_ids: @user_category_ids, corresp_required: @corresp_required)
    }
    @data[:hosts][:all_content].sort! {|x, y|
      (x[:status_ids].size > 0 ? 0 : 100) <=> (y[:status_ids].size > 0 ? 0 : 100) &&
      x[:hostname] <=> y[:hostname]
    }
  end

  def create_vuls_all_content
    @data[:vuls] = {
      :all_content => SidfmdIssue.create_all_content(category_id: 0, role_admin: @role_admin, user_host_ids: @user_host_ids, user_category_ids: @user_category_ids, corresp_required: @corresp_required),
    }
    @enable_host_ids = SidfmdHost.enable_hosts.collect {|x| x.id }
    @category_host_ids =
      if @user_category_ids then
        SidfmdCategoriesHost
          .where(:sidfmd_category_id => @user_category_ids)
          .select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      else
        SidfmdCategoriesHost.select(:sidfmd_host_id).distinct.collect {|x| x.sidfmd_host_id }
      end
    @data[:vuls][:all_content].sort! {|x, y|
      SidfmdIssue::SORT_PRIORITIES[x[:min_status_id]] <=> SidfmdIssue::SORT_PRIORITIES[y[:min_status_id]] ||
      y[:min_created_on][:value] <=> x[:min_created_on][:value]
    }
  end

  def parse_params
    create_hosts_all_content
    create_vuls_all_content
  end

  def create_data
    create_hosts_data
    create_vuls_data
  end

  def create_hosts_data
    @data[:hosts][:all_host_size] = @data[:hosts][:all_content].size

    @data[:hosts][:corresp_required_host_size] = {}
    @data[:hosts][:fixed_corresp_required_host_size] = {}
    @data[:hosts][:not_fixed_corresp_required_host_size] = {}
    [ :sri, :cvss ].each do |i|
      @data[:hosts][:corresp_required_host_size][i] =
        @data[:hosts][:all_content].inject(0) {|r, x|
        r += (x[:warning][@data[:idx]] ? 1 : 0)
      }
      @data[:hosts][:fixed_corresp_required_host_size][i] =
        @data[:hosts][:all_content].sum {|x| x[:fixed_warning][@data[:idx]] ? 1 : 0 }
      @data[:hosts][:not_fixed_corresp_required_host_size][i] =
        @data[:hosts][:all_content].sum {|x| x[:not_fixed_warning][@data[:idx]] ? 1 : 0 }
    end
  end

  def create_vuls_data
    @data[:vuls][:all_vul_size] = @data[:vuls][:all_content].size

    @data[:vuls][:corresp_required_vul_size] = {}
    @data[:vuls][:fixed_corresp_required_vul_size] = {}
    @data[:vuls][:not_fixed_corresp_required_vul_size] = {}

    [ :sri, :cvss ].each do |i|
      @data[:vuls][:corresp_required_vul_size][i] =
        @data[:vuls][:all_content].inject(0) {|r, x|
        r += (x[:warning][@data[:idx]] ? 1 : 0)
      }
      @data[:vuls][:fixed_corresp_required_vul_size][i] =
        @data[:vuls][:all_content].sum {|x| x[:fixed_warning][@data[:idx]] ? 1 : 0 }
      @data[:vuls][:not_fixed_corresp_required_vul_size][i] =
        @data[:vuls][:all_content].sum {|x| x[:not_fixed_warning][@data[:idx]] ? 1 : 0 }
    end
  end

  public

  def index
    pages_params = params.fetch(:pages, {}).permit(:idx)
    @idx = (pages_params[:idx] || @view_state[:idx].to_s)
    @view_state[:idx] = @idx
  end

  def index_r
    pages_params = params.fetch(:pages, {}).permit(:idx)
    @data = {
      :sidfm_user => @sidfm_user,
      :sidfm_roles => @sidfm_roles,
    }
    
    @alerts = view_context.prepare_alerts
    @corresp_required = Sidfm.corresp_required
    @data[:idx] = (pages_params[:idx] || @view_state[:idx].to_s).to_sym
    @view_state[:idx] = @data[:idx]

    parse_params
    create_data

    @data[:dist_cat_hosts] = {}
    SidfmdCategory::LABELS.each do |label|
      @data[:dist_cat_hosts][label] = { :count => 0, :not_fixed_count => 0, :max_sri => 0, :max_cvss => { :label => :notreg, :score => nil }, :elapsed_days => 0 }
    end
    @data[:hosts][:all_content].each do |host|
      syscat = host[:system_category]
      if syscat then
        label = SidfmdCategory.id2label(syscat[:id])
        if label then
          @data[:dist_cat_hosts][label][:count] += 1
          if host[:warning][@data[:idx]] and SidfmdIssue::STATUS_UNFINISHED.include?(host[:min_status_id]) then
            @data[:dist_cat_hosts][label][:not_fixed_count] += 1
            if @data[:dist_cat_hosts][label][:elapsed_days] < host[:elapsed_days] then
              @data[:dist_cat_hosts][label][:elapsed_days] = host[:elapsed_days]
            end
          end
          case @data[:idx]
          when :sri then
            if @data[:dist_cat_hosts][label][:max_sri] < host[:max_sri] then
              @data[:dist_cat_hosts][label][:max_sri] = host[:max_sri]
            end
          when :cvss then
            if host[:max_cvss][:score] then
              if (@data[:dist_cat_hosts][label][:max_cvss][:score] and @data[:dist_cat_hosts][label][:max_cvss][:score] < host[:max_cvss][:score]) or @data[:dist_cat_hosts][label][:max_cvss][:score].nil? then
                @data[:dist_cat_hosts][label][:max_cvss] = host[:max_cvss]
              end
            end
          end
        end
      end
    end

    temp_r5cc = {}
    @data[:hosts][:all_content].each do |host|
      host[:no_system_categories].each do |cat|
        catid = cat[:id]
        if temp_r5cc.has_key?(catid) then
          temp_r5cc[catid][:host_count] += 1
          if host[:warning][@data[:idx]] and host[:min_status_id] == SidfmdIssue::STATUS_NOTCONFIRMED then
            temp_r5cc[catid][:not_fixed_count] += 1
          end
          if (host[:max_sri] and temp_r5cc[catid][:max_sri] and temp_r5cc[catid][:max_sri] < host[:max_sri]) or
              temp_r5cc[catid][:max_sri].nil? then
            temp_r5cc[catid][:max_sri] = host[:max_sri]
          end
          if (host[:max_cvss][:score] and temp_r5cc[catid][:max_cvss][:score] and temp_r5cc[catid][:max_cvss][:score] < host[:max_cvss][:score]) or
              temp_r5cc[catid][:max_cvss][:score].nil? then
            temp_r5cc[catid][:max_cvss] = host[:max_cvss]
          end
        else
          temp_r5cc[catid] = {
            :id => catid,
            :name => cat[:name],
            :host_count => 1,
            :not_fixed_count => (host[:warning][@data[:idx]] and host[:min_status_id] == SidfmdIssue::STATUS_NOTCONFIRMED) ? 1 : 0,
            :max_sri => host[:max_sri],
            :max_cvss => host[:max_cvss],
          }
        end
      end
    end
    @data[:recently_5_custom_categories] = temp_r5cc.keys.collect {|x| temp_r5cc[x] }
    @data[:recently_5_custom_categories].sort! {|x, y|
      temp_ev = proc {|x|
        [ x[:not_fixed_count], x[:max_cvss][:score] ? x[:max_cvss][:score] : 0.0 , x[:max_sri] ]
      }
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = yy[2] <=> xx[2]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[0] <=> xx[0]
        end
      end
      r
    }
    @data[:recently_5_custom_categories] = @data[:recently_5_custom_categories][0, 5]
    @data[:recently_5_custom_categories].each do |cat|
      min_created_on =
        SidfmdIssue
        .enable_hosts
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => cat[:id])
                 .select(:sidfmd_host_id))
        .joins(:issue)
        .where(:issues => { :status_id => SidfmdIssue::STATUS_UNFINISHED })
        .select(:created_on).min
      cat[:elapsed_days] = min_created_on ? ((Time.current - min_created_on.created_on) / 24.0 / 60.0 / 60.0).to_i : -1
    end

    temp_ev = proc {|x|
      [ SidfmdIssue::STATUS_UNFINISHED.include?(x[:min_status_id]) ? 100 : 1000,
        x[:max_sri] ? x[:max_sri] : 0,
        x[:max_cvss][:score] ? x[:max_cvss][:score] : 0.0,
        x[:elapsed_days], ]
    }
    @data[:recently_5_hosts] = @data[:hosts][:all_content].sort {|x, y|
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = xx[0] <=> yy[0]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[2] <=> xx[2]
          if r == 0 then
            r = yy[3] <=> xx[3]
          end
        end
      end
      r
    }[0, 5]

    @data[:recently_5_vuls] = @data[:vuls][:all_content].sort {|x, y|
      temp_ev = proc {|x|
        [ x[:warning][@data[:idx]] ? 100 : 1000, x[:max_sri] ? x[:max_sri] : 0, x[:cvss][:score] ? x[:cvss][:score] : 0.0, x[:not_fixed_hosts], x[:min_created_on][:display] ]
      }
      xx = temp_ev.call(x)
      yy = temp_ev.call(y)
      r = xx[0] <=> yy[0]
      if r == 0 then
        r = yy[1] <=> xx[1]
        if r == 0 then
          r = yy[2] <=> xx[2]
          if r == 0 then
            r = yy[3] <=> xx[3]
            if r == 0 then
              r = yy[4] <=> xx[4]
            end
          end
        end
      end
      r
    }[0, 5]

    @data[:recently_5_journals] = Journal
      .joins(:issue)
      .joins(:details)
      .joins(:issue => :sidfmd_issue)
      .joins(:issue => { :sidfmd_issue => :sidfmd_host })
      .where(:private_notes => false)
      .where(:sidfmd_hosts => { :flg_delete => false })
    if @user_category_ids then
        @data[:recently_5_journals] = @data[:recently_5_journals].where(:sidfmd_issues => {:sidfmd_host_id => SidfmdCategoriesHost.all.where(:sidfmd_category_id => @user_category_ids).select(:sidfmd_host_id).distinct})
    end
    @data[:recently_5_journals] = @data[:recently_5_journals]
      .where(:journal_details => { :prop_key => [ 'status_id', 'due_date' ] })
      .order(:created_on => :desc)
      .limit(5)
  end

end
