# -*- coding: utf-8 -*-
require 'date'
require 'sidfm_controller_support'

# 内部的に使用するAPIをまとめたもの
# 基本的にviewのJavaScriptから利用する。
class SidfmInternalApiController < ApplicationController
  unloadable

  layout false

  self.main_menu = false

  before_action :get_sidfm_roles
  before_action :check_roles
  protect_from_forgery with: :exception

  private

  include SidfmControllerSupport

  # 権限チェック
  def check_roles
    @role_admin = @sidfm_roles.include?(:admin)
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    unless @sidfm_roles.include?(:view) then
      redirect_to user_path(User.current), :notice => af_log_user('アクセス権限がありません。') and return
    end
    @user_category_ids = if @role_admin then
                           nil
                         else
                           SidfmdCategoriesUser.where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
                         end
    @user_host_ids = if @role_admin then
                       nil
                     else
                       SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => @user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
                     end
    @cache_prefix = @role_admin ? 'internalapi/all' : "internalapi/categories/#{User.current.id}"
  end

  def get_hosts(keyword)
    if keyword and keyword.size >= 2 then
      hosts =
        SidfmdHost
        .enable_hosts
        .select(:id, :hostname, :created_at, :updated_at, :is_agent)
        .where('LOWER(hostname) LIKE ?', '%' + keyword.gsub(/\%/, '').downcase + '%')
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        hosts = hosts
          .where(:id => user_host_ids)
      end
      hosts = hosts
        .order(:hostname)
        .limit(100)
    end
  end

  public

  # JavaScriptでのエラーをアプリケーションログに記録する
  def api_logger
    f_params = params.fetch(:params, {}).permit(:msg, :file, :line, :column, :err)
    msg = f_params[:msg]
    file = f_params[:file]
    line = f_params[:line]
    column = f_params[:column]
    err = f_params[:err]
    logger.info('---JavaScript error - begin---')
    logger.info('msg: ' + msg)
    logger.info('file: ' + file)
    logger.info('line: ' + line)
    logger.info('column: ' + column)
    logger.info('err: ' + err.inspect)
    logger.info('---JavaScript error - end---')
    render :layout => false, :json => {}.to_json
  end

  # ホスト名のキーワード検索
  # suggestで使うことを想定。
  def api_hosts
    f_params = params.fetch(:params, {}).permit(:keyword)
=begin    ret = []
    if f_params[:keyword] and f_params[:keyword].size >= 2 then
      hosts =
        SidfmdHost
        .enable_hosts
        .select(:id, :hostname, :created_at, :updated_at, :is_agent)
        .where('LOWER(hostname) LIKE ?', '%' + f_params[:keyword].gsub(/\%/, '').downcase + '%')
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        hosts = hosts
          .where(:id => user_host_ids)
      end
      hosts = hosts
        .order(:hostname)
        .limit(100)
    end
=end
    hosts = get_hosts(f_params[:keyword])
    render :layout => false, :json => hosts.to_json
  end

  # 脆弱性のキーワード検索
  # suggestで使うことを想定。
  def api_vulnerabilities
    f_params = params.fetch(:params, {}).permit(:keyword)
    ret = nil
    if f_params[:keyword] and (f_params[:keyword].size >= 2 or /\A\d\z/ =~ f_params[:keyword]) then
      keyword = f_params[:keyword].gsub(/[\'"\\%]/, '')
      cve_keyword = nil
      # CVE IDっぽい文字列を見つけたら、整形してcve_idの検索に入れる。
      if /\Acve(\D)?(\d{1,4})(\D)?(\d+)?\z/i =~ keyword then
        year = Regexp.last_match(2)
        num = Regexp.last_match(4)
        if year and num then
          cve_keyword = "cve-#{year}-#{num}"
        end
      end
      sidfmid_keyword = nil
      # 数値ならSIDfmIDとして検索
      if /\A(\d+)\z/ =~ keyword then
        sidfmd_keyword = Regexp.last_match(1)
      end
      # KB数値ならMicrosoftKBとして検索
      kb_keyword = nil
      if /\AKB(\d+)\z/i =~ keyword then
        kb_keyword = Regexp.last_match(1).to_i
      end
      part_sql = [ 'LOWER(sidfmd_issues.title) LIKE ?' ]
      part_arg = [ '%' + keyword.downcase + '%' ]
      if cve_keyword then
        part_sql << 'LOWER(sidfmd_issue_cves.cve_id) LIKE ?'
        part_arg << '%' + cve_keyword + '%'
      end
      if sidfmd_keyword then
        part_sql << 'sidfmd_issues.html_id = ?'
        part_arg << sidfmd_keyword.to_i
      end
      if kb_keyword then
        part_sql << 'sidfmd_issues.html_id = ?'
        part_arg << -kb_keyword
      end
      logger.info(part_sql.inspect)
      logger.info(part_arg.inspect)
      issues = SidfmdIssue
        .enable_hosts
        .categorized
      if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        issues = issues
          .where(:sidfmd_host_id => user_host_ids)
      end
      issues = issues
        .limit(100)
        .joins("LEFT OUTER JOIN sidfmd_issue_cves ON sidfmd_issue_cves.sidfmd_issue_id=sidfmd_issues.id")
        .where(part_sql.join(' OR '), *part_arg)
        .group(:html_id, :title)
      logger.info(issues.to_sql)
      issues = issues.count
      ret = issues.keys.collect {|x| { :html_id => x[0], :title => x[1] } }
    end
    render :layout => false, :json => ret.to_json
  end

  # 指定された期間に登録された脆弱性のIDとタイトル、日付、関連するホスト数を返す。
  def api_vulnerabilities_period
    f_params = params.fetch(:params, {}).permit(:unit, :num)
    ret = []
    unit = f_params[:unit]
    unless [ 'days', 'weeks', 'months' ].include?(unit) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = f_params[:num]
    unless num then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = num.to_i
    unless num > 0 then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end

    issues = SidfmdIssue
      .enable_hosts
      .categorized
    if !@sidfm_roles.include?(:admin) then
        user_category_ids = SidfmdCategoriesUser
          .where(:user_id => User.current.id).select(:sidfmd_category_id).order(:sidfmd_category_id).distinct.collect {|x| x[:sidfmd_category_id] }
        user_host_ids = SidfmdCategoriesHost.joins(:sidfmd_host).where(:sidfmd_category_id => user_category_ids).where(:sidfmd_hosts => { :flg_delete => false }).select(:sidfmd_host_id).order(:sidfmd_host_id).distinct.collect {|x| x[:sidfmd_host_id] }
        issues = issues
          .where(:sidfmd_host_id => user_host_ids)
    end
    issues = issues
      .joins(:issue)
      .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
      .group(:html_id, :title)
      .count
      .keys

    issues.each do |issue|
      item = {
        :html_id => issue[0],
        :title => issue[1],
        :created_on => '',
        :hosts => [],
      }

      times_hosts = SidfmdIssue
        .joins(:issue)
        .where(:html_id => item[:html_id])
        .group(:created_on, :sidfmd_host_id)
        .count
        .keys

      created_on = Time.current
      times_hosts.each do |th|
        t = th[0]
        sidfmd_host_id = th[1]
        sidfmd_host = SidfmdHost.find(sidfmd_host_id)
        created_on = t if created_on > t
        item[:hosts] << {
          :id => sidfmd_host_id,
          :hostname => sidfmd_host.hostname,
        }
      end
      item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
      item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
      ret << item
    end

    render :layout => false, :json => ret.to_json
  end

  # 指定された期間に登録され、かつ、指定の危険度の脆弱性を抽出し、
  # html_idとタイトル、日付、関連するホスト数を返す。
  def api_vulnerabilities_risk_period
    f_params = params.fetch(:params, {}).permit(:risk_type, :risk_level, :unit, :num)
    ret = []
    risk_type = f_params[:risk_type]
    unless [ 'sri', 'cvss' ].include?(risk_type) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    risk_level = f_params[:risk_level]
    if
        !(risk_type == 'sri' and
          SidfmdIssueSri::VALUES.include?(risk_level.to_i)) and
        !(risk_type == 'cvss' and
          SidfmdIssueCve::LABELS.include?(risk_level.to_sym))
    then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    unit = f_params[:unit]
    unless [ 'days', 'weeks', 'months' ].include?(unit) then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = f_params[:num]
    unless num then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end
    num = num.to_i
    unless num > 0 then
      render :layout => false, :json => ret.to_json, :status => 400 and return
    end

    case risk_type
    when 'sri' then
      issues = SidfmdIssue
        .enable_hosts
        .categorized
        .joins(:issue)
        .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
        .group(:html_id, :title)
        .count
        .keys
      issues.each do |issue|
        item = {
          :html_id => issue[0],
          :title => issue[1],
          :created_on => '',
          :hosts => [],
        }
        times_hosts = SidfmdIssue
          .joins(:issue)
          .where(:html_id => item[:html_id])
          .group(:created_on, :sidfmd_host_id)
          .count
          .keys
        created_on = Time.current
        times_hosts.each do |th|
          t = th[0]
          sidfmd_host_id = th[1]
          sidfmd_host = SidfmdHost.find(sidfmd_host_id)
          created_on = t if created_on > t
          sri = SidfmdIssueSri
            .joins(:sidfmd_issue)
            .where(:sidfmd_issues => { :sidfmd_host_id => sidfmd_host_id })
            .where(:category_id => sidfmd_host.system_category.id)
            .first
          risk_id = if sri then
                      sri.risk_id
                    else
                      0
                    end
          item[:hosts] << {
            :id => sidfmd_host_id,
            :hostname => sidfmd_host.hostname,
            :created_on => t,
            :risk_id => risk_id,
          }
        end
        item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
        item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
        ret << item
      end
      risk_level_i = risk_level.to_i
      logger.info("before #{ret.size}")
      logger.info(ret.inspect)
      ret.delete_if {|x|
        x[:hosts].delete_if {|y| y[:risk_id] != risk_level_i }
        x[:hosts].size == 0
      }
      logger.info("after #{ret.size}")
      logger.info(ret.inspect)
    when 'cvss' then
      issues = SidfmdIssue
        .enable_hosts
        .categorized
        .joins(:issue)
        .where('issues.created_on >= ?', Time.current.ago(num.send(unit.to_sym)))
        .group(:html_id, :title)
        .count
        .keys
      issues.each do |issue|
        item = {
          :html_id => issue[0],
          :title => issue[1],
          :created_on => '',
          :hosts => [],
          :temp_score => 0.0,
        }

        cves = SidfmdIssue
          .where(:html_id => item[:html_id])
          .first
          .sidfmd_issue_cves
          .select('MAX(COALESCE(c3_base_score,c2_base_score)) AS max_score')
        item[:temp_score] = cves[0].max_score

        times_hosts = SidfmdIssue
          .joins(:issue)
          .where(:html_id => item[:html_id])
          .group(:created_on, :sidfmd_host_id)
          .count
          .keys

        created_on = Time.current
        times_hosts.each do |th|
          t = th[0]
          sidfmd_host_id = th[1]
          sidfmd_host = SidfmdHost.find(sidfmd_host_id)
          created_on = t if created_on > t
          item[:hosts] << {
            :id => sidfmd_host_id,
            :hostname => sidfmd_host.hostname,
          }
        end
        item[:created_on] = created_on.localtime.strftime('%Y/%m/%d')
        item[:hosts].sort! {|x, y| x[:id] <=> y[:id] }
        ret << item
      end
      case risk_level
      when 'notreg' then
        ret.delete_if {|x| !x[:temp_score].nil? }
      when 'd4' then
        ret.delete_if {|x| x[:temp_score].nil? or x[:temp_score] < 9.0 }
      when 'd3' then
        ret.delete_if {|x| x[:temp_score].nil? or
          (x[:temp_score] >= 9.0 or x[:temp_score] < 7.0 ) }
      when 'd2' then
        ret.delete_if {|x| x[:temp_score].nil? or
          (x[:temp_score] >= 7.0 or x[:temp_score] < 4.0 ) }
      when 'd1' then
        ret.delete_if {|x| x[:temp_score].nil? or x[:temp_score] >= 4.0 }
      end
      ret.each do |i|
        i.delete(:temp_score)
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_hosts.idの配列から、対象ホスト、関連するカテゴリ、
  # カテゴリに関連づけられたユーザ、ユーザに関連づけられたメール
  # アドレスの情報を返す。
  def api_hosts_categories_users_emails
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])

    hosts = SidfmdHost.enable_hosts.where(:id => hostids).select(:id, :hostname, :flg_delete, :is_agent)

    categories_hosts = SidfmdCategoriesHost.where(:sidfmd_host_id => hosts.collect {|x| x.id })

    categories = SidfmdCategory.joins(:sidfmd_categories_hosts).where(:sidfmd_categories_hosts => { :sidfmd_host_id => hosts.collect {|x| x.id } }).select(:id, :name, :desc, :user_id, :is_system)

    categories_users = SidfmdCategoriesUser.where(:sidfmd_category_id => categories.collect {|x| x.id })

    users = User.where(:id => categories_users.collect {|x| x.user_id }).select(:id, :login, :firstname, :lastname, :admin, :status, :language, :type, :identity_url, :mail_notification)


    emails = EmailAddress.where(:user_id => users.collect {|x| x.id }).select(:id, :user_id, :address, :is_default, :notify)

    r = {
      :hosts => hosts,
      :categories => categories.uniq,
      :categories_users => categories_users,
      :categories_hosts => categories_hosts,
      :users => users,
      :emails => emails,
    }

    render :layout => false, :json => r.to_json
  end

  # sidfmd_hosts.idの配列から、対象ホストとそれに関連づけられたissuesの
  # due_dateの一覧を返す。
  def api_hosts_due_dates
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])

    hosts = SidfmdHost.enable_hosts.where(:id => hostids).select(:id, :hostname, :flg_delete, :is_agent)
    issues = Issue.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => hostids }).where(:status_id => [ 1, 2 ]).select(:sidfmd_host_id, :due_date).distinct

    r = {
      :hosts => hosts,
      :issues => issues,
    }
    
    render :layout => false, :json => r.to_json
  end


  # sidfmd_hosts.idの配列から、対象ホストとそれに関連づけられたissuesの
  # status_idの一覧を返す。
  def api_hosts_statuses
    f_params = params.fetch(:params, {}).permit(:hostids)
    hostids = JSON.parse(f_params[:hostids])

    hosts = SidfmdHost.enable_hosts.where(:id => hostids).order(:hostname).select(:id, :hostname, :flg_delete, :is_agent)
    issues = Issue.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => hostids }).select(:sidfmd_host_id, :status_id).distinct
    statuses = {}
    hosts.each do |host|
      statuses[host.id] = host.status_count
    end

    r = {
      :hosts => hosts,
      :issues => issues,
      :statuses => statuses,
    }
    
    render :layout => false, :json => r.to_json
  end

  # html_idの配列から、関連づけられた脆弱性とissuesのstatus_idの一覧を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_statuses
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])
    ret = {}

    issues = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT
 CAST(issues.status_id AS TEXT) || ':' || CAST(COUNT(issues.status_id) AS TEXT)
FROM sidfmd_issues AS t_sidfmd_issues
INNER JOIN issues
 ON t_sidfmd_issues.issue_id=issues.id
WHERE t_sidfmd_issues.html_id=sidfmd_issues.html_id
      #{@role_admin ? '' : "AND t_sidfmd_issues.sidfmd_host_id IN (" + @user_host_ids.join(',') + ")" }
GROUP BY issues.status_id
ORDER BY issues.status_id
),',') AS status_ids")
      .where(:html_id => htmlids)
      .where(:sidfmd_hosts => { :flg_delete => false })
    unless @role_admin then
      issues = issues
        .where(:sidfmd_host_id => @user_host_ids)
    end
    issues = issues
      .group(:html_id, :title)
    status2color = [ '', :danger, :warning, :info, :success ]
    issues.each do |issue|
      unless ret.has_key?(issue[:html_id]) then
        ret[issue[:html_id]] = {
          :title => issue[:title],
          :status => [],
        }
        status_ids = issue[:status_ids] ? issue[:status_ids].split(/,/) : []
        status_ids.each do |status_id|
          item = status_id.split(/:/)
          id = item.first.to_i
          cnt = item.last.to_i
          ret[issue[:html_id]][:status] << {
            :id => issue[:html_id],
            :value => cnt,
            :name => SidfmdIssue::STATUS_NAMES[id],
            :color => ('status_' + SidfmdIssue::STATUS_LABELS[id].to_s).to_sym,
            :placement => 'top',
          }
        end

      end
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idの配列から、関連づけられた脆弱性とissuesのdue_datesの一覧を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_due_dates
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .joins(:sidfmd_host)
      .joins(:issue)
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:html_id => htmlids)
    unless @role_admin then
      issues = issues
        .where(:sidfmd_host_id => @user_host_ids)
    end
    issues = issues
      .group(:html_id, :title, :due_date)
      .count
      .keys

    ret = {}

    issues.each do |issue|
      html_id = issue[0]
      title = issue[1]
      due_date = issue[2] ? issue[2].strftime('%Y/%m/%d') : nil
      if ret.has_key?(html_id) then
        ret[html_id][:due_dates] << due_date
      else
        ret[html_id] = {
          :title => title,
          :due_dates => [ due_date ],
        }
      end
    end
    ret.keys.each do |html_id|
      ret[html_id][:due_dates].uniq!
      ret[html_id][:due_dates].sort! {|x, y| (x.nil? ? '' : x) <=> (y.nil? ? '' : y) }
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_host_idとhtml_idの配列から、関連付けられたissuesのdue_dateを返す。
  def api_host_htmlids_due_dates
    f_params = params.fetch(:params, {}).permit(:hostid, :htmlids)
    unless f_params[:hostid] or f_params[:htmlids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostid = f_params[:hostid].to_i
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .enable_hosts
      .categorized
      .joins(:issue)
      .where(:sidfmd_host_id => hostid)
      .where(:html_id => htmlids)
      .group(:html_id, :title, :due_date)
      .count
      .keys

    ret = []

    issues.each do |issue|
      ret << {
        :html_id => issue[0],
        :title => issue[1],
        :due_date => issue[2] ? issue[2].strftime('%Y/%m/%d') : issue[2],
      }
    end

    render :layout => false, :json => ret.to_json
  end

  # sidfmd_host_idとhtml_idの配列から、関連付けられたissuesのstatus_idの一覧を返す。
  def api_host_htmlids_statuses
    f_params = params.fetch(:params, {}).permit(:hostid, :htmlids)
    unless f_params[:hostid] or f_params[:htmlids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostid = f_params[:hostid].to_i
    htmlids = JSON.parse(f_params[:htmlids])

    issues = SidfmdIssue
      .enable_hosts
      .categorized
      .joins(:issue)
      .where(:sidfmd_host_id => hostid)
      .where(:html_id => htmlids)
      .group(:html_id, :title, :status_id)
      .count
      .keys

    ret = []

    issues.each do |issue|
      ret << {
        :html_id => issue[0],
        :title => issue[1],
        :status_id => issue[2],
      }
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idとsidfmd_hosts.idの配列から、対象ホストとそれに関連づけられ
  # たissuesのstatus_idの一覧を返す。
  def api_htmlid_hosts_statuses
    f_params = params.fetch(:params, {}).permit(:htmlid, :hostids)
    unless f_params[:htmlid] or f_params[:hostids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    htmlid = f_params[:htmlid].to_i
    if htmlid == 0 then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostids = nil
    begin
      hostids = JSON.parse(f_params[:hostids])
    rescue JSON::ParserError => e
      log_exception(e)
    end
    unless hostids then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end

    hosts = SidfmdHost
      .enable_hosts
      .categorized
      .joins(:sidfmd_issues)
      .joins(:sidfmd_issues => [ :issue ])
      .where(:id => hostids)
      .where(:sidfmd_issues => { :html_id => htmlid })
      .select(:id, :hostname, :status_id)

    ret = []
    hosts.each do |host|
      ret << {
        :id => host[:id],
        :hostname => host[:hostname],
        :status_id => host[:status_id],
      }
    end
    render :layout => false, :json => ret.to_json
  end

  # htmlidとsidfmd_hosts.idの配列から、対象ホストとそれに関連づけられ
  # たissuesのdue_dateの一覧を返す。
  def api_htmlid_hosts_due_dates
    f_params = params.fetch(:params, {}).permit(:htmlid, :hostids)
    unless f_params[:htmlid] or f_params[:hostids] then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    htmlid = f_params[:htmlid].to_i
    if htmlid == 0 then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end
    hostids = nil
    begin
      hostids = JSON.parse(f_params[:hostids])
    rescue JSON::ParserError => e
      log_exception(e)
    end
    unless hostids then
      render :layout => false, :status => 400, :json => {}.to_json and return
    end

    hosts = SidfmdHost
      .enable_hosts
      .categorized
      .joins(:sidfmd_issues)
      .joins(:sidfmd_issues => [ :issue ])
      .where(:id => hostids)
      .where(:sidfmd_issues => { :html_id => htmlid })
      .select(:id, :hostname, :due_date)

    ret = []
    hosts.each do |host|
      ret << {
        :id => host[:id],
        :hostname => host[:hostname],
        :due_date => host[:due_date] ? host[:due_date].strftime('%Y/%m/%d') : host[:due_date],
      }
    end

    render :layout => false, :json => ret.to_json
  end


  # html_idの配列から、関連づけられた脆弱性と脆弱性に関連したホストの内、
  # ユーザが定義されたカテゴリに属するユーザの情報を返す。
  # 利用:
  #   sidfm_vulnerbilities
  def api_htmlids_emails
    f_params = params.fetch(:params, {}).permit(:htmlids)
    htmlids = JSON.parse(f_params[:htmlids])
    ret = { :users => {}, :htmls => [] }
    htmls = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT DISTINCT(t_sidfmd_hosts.id)
FROM sidfmd_hosts AS t_sidfmd_hosts
INNER JOIN sidfmd_issues AS t_sidfmd_issues
 ON t_sidfmd_issues.sidfmd_host_id=t_sidfmd_hosts.id
WHERE sidfmd_issues.html_id=t_sidfmd_issues.html_id AND
     t_sidfmd_hosts.flg_delete IS FALSE
     #{@role_admin ? '' : 'AND t_sidfmd_hosts.id IN(' + @user_host_ids.join(',') + ')'}
ORDER BY t_sidfmd_hosts.id
),',') AS hostids")
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:sidfmd_host_id => SidfmdCategoriesHost.joins(:sidfmd_category).where(:sidfmd_categories => { :is_system => false }).select(:sidfmd_host_id).distinct)
    unless @role_admin then
      htmls = htmls.where(:sidfmd_host_id => @user_host_ids)
    end
    htmls = htmls
      .where(:html_id => htmlids)
      .group(:html_id, :title)
    hostids = []
    htmls.each do |html|
      ret[:htmls] << {
        :id => html[:html_id],
        :title => html[:title],
      }
      hostids += html[:hostids].split(/,/).collect {|x| x.to_i }
    end
    hostids.uniq!
    hostids.compact!
    hostids &= @user_host_ids unless @role_admin

    categories = SidfmdCategoriesHost
      .joins(:sidfmd_category)
      .where(:sidfmd_host_id => hostids)
      .where(:sidfmd_categories => { :is_system => false })
    unless @role_admin then
      categories = categories
        .where(:sidfmd_category_id => @user_category_ids)
    end
    categories = categories
      .select(:sidfmd_category_id)
      .distinct

    users = SidfmdCategoriesUser
      .joins(:sidfmd_category)
      .joins(:user)
      .select(:user_id)
      .select(:login)
      .select(:firstname)
      .select(:lastname)
      .select(:sidfmd_category_id)
      .select(:name)
      .where(:sidfmd_categories => { :is_system => false })
      .where(:sidfmd_category_id => categories)
    unless @role_admin then
      users = users
        .where(:sidfmd_category_id => @user_category_ids)
    end
    users.each do |user|
      unless ret[:users].has_key?(user[:user_id]) then
        ret[:users][user[:user_id]] = {
          :login => user[:login],
          :firstname => user[:firstname],
          :lastname => user[:lastname],
          :emails => user.user.email_addresses.collect {|x|
            { :id => x.id, :address => x.address, }
          } ,
          :categories => [],
          :hosts => [],
        }
      end
      if @role_admin or @user_category_ids.include?(user[:sidfmd_category_id]) then
        ret[:users][user[:user_id]][:categories] << { :id => user[:sidfmd_category_id], :name => user[:name] }
        ret[:users][user[:user_id]][:categories].uniq!
        cat_hosts = SidfmdCategoriesHost
          .joins(:sidfmd_host)
          .select(:sidfmd_host_id)
          .select(:hostname)
          .where(:sidfmd_hosts => { :flg_delete => false })
          .where(:sidfmd_host_id => hostids)
          .where(:sidfmd_category_id => user[:sidfmd_category_id])
          .where(:sidfmd_category_id => categories)
          .group(:sidfmd_host_id, :hostname)
        cat_hosts.each do |host|
          ret[:users][user[:user_id]][:hosts] << { :id => host[:sidfmd_host_id], :hostname => host[:hostname] }
        end
        ret[:users][user[:user_id]][:categories].uniq!
        ret[:users][user[:user_id]][:hosts].uniq!
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # html_idの配列から、関連づけられた脆弱性と脆弱性に関連したホストの内、
  # ユーザが定義されたカテゴリに属するユーザの情報を返す。
  # api_htmlids_emailsとほぼ同じ処理だが、ホスト詳細画面専用で、ホストが
  # 固定されているため、カテゴリもホストに付与されたものが採用される。
  # 利用:
  #   sidfm_host
  def api_host_htmlids_emails
    f_params = params.fetch(:params, {}).permit(:htmlids, :hostid)
    htmlids = JSON.parse(f_params[:htmlids])
    hostid = f_params[:hostid].to_i
    host = SidfmdHost.find(hostid)
    unless host then
      render :layout => false, :json => '{}'.to_json, :status => 500
      return
    end

    ret = { :users => {}, :htmls => [] }
    htmls = SidfmdIssue
      .joins(:sidfmd_host)
      .select(:html_id, :title)
      .select("ARRAY_TO_STRING(ARRAY(
SELECT DISTINCT(t_sidfmd_hosts.id)
FROM sidfmd_hosts AS t_sidfmd_hosts
INNER JOIN sidfmd_issues AS t_sidfmd_issues
 ON t_sidfmd_issues.sidfmd_host_id=t_sidfmd_hosts.id
WHERE sidfmd_issues.html_id=t_sidfmd_issues.html_id AND
     t_sidfmd_hosts.flg_delete IS FALSE
     #{@role_admin ? '' : 'AND t_sidfmd_hosts.id IN(' + @user_host_ids.join(',') + ')'}
ORDER BY t_sidfmd_hosts.id
),',') AS hostids")
      .where(:sidfmd_hosts => { :flg_delete => false })
      .where(:sidfmd_host_id => SidfmdCategoriesHost.joins(:sidfmd_category).where(:sidfmd_categories => { :is_system => false }).select(:sidfmd_host_id).distinct)
    unless @role_admin then
      htmls = htmls.where(:sidfmd_host_id => @user_host_ids)
    end
    htmls = htmls
      .where(:html_id => htmlids)
      .group(:html_id, :title)
    
    hostids = [ hostid ]
    htmls.each do |html|
      ret[:htmls] << {
        :id => html[:html_id],
        :title => html[:title],
      }
    end
    hostids &= @user_host_ids unless @role_admin

    categories = SidfmdCategoriesHost
      .joins(:sidfmd_category)
      .where(:sidfmd_host_id => hostids)
      .where(:sidfmd_categories => { :is_system => false })
    unless @role_admin then
      categories = categories
        .where(:sidfmd_category_id => @user_category_ids)
    end
    categories = categories
      .select(:sidfmd_category_id)
      .distinct

    users = SidfmdCategoriesUser
      .joins(:sidfmd_category)
      .joins(:user)
      .select(:user_id)
      .select(:login)
      .select(:firstname)
      .select(:lastname)
      .select(:sidfmd_category_id)
      .select(:name)
      .where(:sidfmd_categories => { :is_system => false })
      .where(:sidfmd_category_id => categories)
    unless @role_admin then
      users = users
        .where(:sidfmd_category_id => @user_category_ids)
    end
    users.each do |user|
      unless ret[:users].has_key?(user[:user_id]) then
        ret[:users][user[:user_id]] = {
          :login => user[:login],
          :firstname => user[:firstname],
          :lastname => user[:lastname],
          :emails => user.user.email_addresses.collect {|x|
            { :id => x.id, :address => x.address, }
          } ,
          :categories => [],
          :hosts => [],
        }
      end
      if @role_admin or @user_category_ids.include?(user[:sidfmd_category_id]) then
        ret[:users][user[:user_id]][:categories] << { :id => user[:sidfmd_category_id], :name => user[:name] }
        ret[:users][user[:user_id]][:categories].uniq!
        cat_hosts = SidfmdCategoriesHost
          .joins(:sidfmd_host)
          .select(:sidfmd_host_id)
          .select(:hostname)
          .where(:sidfmd_hosts => { :flg_delete => false })
          .where(:sidfmd_host_id => hostids)
          .where(:sidfmd_category_id => user[:sidfmd_category_id])
          .where(:sidfmd_category_id => categories)
          .group(:sidfmd_host_id, :hostname)
        cat_hosts.each do |host|
          ret[:users][user[:user_id]][:hosts] << { :id => host[:sidfmd_host_id], :hostname => host[:hostname] }
        end
        ret[:users][user[:user_id]][:categories].uniq!
        ret[:users][user[:user_id]][:hosts].uniq!
      end
    end

    render :layout => false, :json => ret.to_json
  end

  # セキュリティニュース情報（SIDfm情報）で使用するAPI
  def api_info_find_news_topics
    f_params = params.permit(:begin_date, :end_date, :search_text, :limit, :offset, kinds: [])
    ret = { 'topics' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_news_topics(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_find_products
    f_params = params.permit(:search_text)
    ret = { 'products' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_products(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_recent_product_eols
    ret = { 'product_eols' => [] }
    Sidfm.sidapi do |o|
      ret = o.recent_product_eols
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報（SIDfm情報）で使用するAPI
  def api_info_products
    # FIXME: datetime_typeはobsolete
    f_params = params.permit(:datetime_type, :time_field, :begin_datetime, :end_datetime)
    ret = { 'products' => [] }
    Sidfm.sidapi do |o|
      ret = o.products2(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product
    f_params = params.permit(:id)
    ret = { 'product' => [] }
    Sidfm.sidapi do |o|
      ret = o.product(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product_eols
    f_params = params.permit(:product_id)
    ret = { 'product_eols' => [] }
    Sidfm.sidapi do |o|
      ret = o.product_eols(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_risks
    ret = { 'risks' => [] }
    Sidfm.sidapi do |o|
      ret = o.risks
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_areas
    ret = { 'areas' => [] }
    Sidfm.sidapi do |o|
      ret = o.areas
    end
    render :layout => false, :json => ret.to_json
  end

  # リリース・EOL情報詳細（SIDfm情報）で使用するAPI
  def api_info_product_vulns
    f_params = params.permit(:product_id, :begin_datetime, :end_datetime)
    ret = { 'product_vulns' => [] }
    Sidfm.sidapi do |o|
      ret = o.product_vulns(params)
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報（SIDfm情報）で使用するAPI
  def api_info_find_vulns
    f_params = params.permit(:begin_date, :end_date, :cvss_base_score, :search_text, :limit, :offset)
    ret = { 'vulns' => [] }
    Sidfm.sidapi do |o|
      ret = o.find_vulns(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報詳細（SIDfm情報）で使用するAPI
  def api_info_sri_categories
    ret = { 'sri_categories' => [] }
    Sidfm.sidapi do |o|
      ret = o.sri_categories
    end
    render :layout => false, :json => ret.to_json
  end

  # セキュリティホール情報詳細（SIDfm情報）で使用するAPI
  def api_info_vuln
    f_params = params.permit(:id)
    ret = { 'vuln' => {} }
    Sidfm.sidapi do |o|
      ret = o.vuln2(f_params)
    end
    render :layout => false, :json => ret.to_json
  end

  # KBでセキュリティホール情報を検索し、その結果を返す。
  def api_find_vulns_by_mskb
    ret = []
    f_params = params.fetch(:params, {}).permit(:kb)
    kb = (f_params[:kb] || '0').to_i
    Sidfm.sidapi do |o|
      ret = o.find_vulns_by_mskb(kb)
    end
    render :layout => false, :json => ret.to_json
  end

  # ホスト名から、アクセス可能なホストか否かを返す。
  # ホスト管理画面で使用
  def api_check_hostname
    ret = { :exist => false, :accessable => false }
    f_params = params.fetch(:params, {}).permit(:hostname)
    begin
      host =
        SidfmdHost
        .where(:flg_delete => false)
        .where(:hostname => (f_params[:hostname] || '')).first
      ret[:exist] = !host.nil?
      if @sidfm_roles.include?(:admin) then
        ret[:accessable] = true
      elsif @sidfm_roles.include?(:use) then
        if ret[:exist] then
          host =
            SidfmdHost
            .joins(:sidfmd_categories_hosts)
            .where(:flg_delete => false)
            .where(:sidfmd_categories_hosts =>
                   { :sidfmd_category_id =>
                     SidfmdCategoriesUser
                       .where(:user_id => User.current.id)
                       .select(:sidfmd_category_id)
                       .distinct.collect {|x| x[:sidfmd_category_id] } })
            .where(:hostname => (f_params[:hostname] || '')).first
          ret[:accessable] = !host.nil?
        else
          ret[:accessable] = true
        end
      end
    rescue Exception => e
      log_exception(e)
    end

    render :layout => false, :json => ret.to_json
  end
  
  #履歴表示のためのデータ取得
  def api_get_journals
    f_params = params.permit(:html_id, :host_id)
    data = []
    html_id = f_params[:html_id].to_i
    host_id = f_params[:host_id].to_i
    issue = SidfmdIssue.where(:html_id => html_id).where(:sidfmd_host_id => host_id)
    if issue.length > 0 then
      issue_id = issue.map{|i| i.issue_id}
      journal = Journal.where(:journalized_id => issue_id).map{|j| [j.id, j.user_id, j.notes, j.created_on.strftime('%Y/%m/%d %H:%M:%S')]}
      users = {}
      User.where(:id => journal.map{|j| j[1]}).each do |user|
        users[user.id] = [user.mail, user.lastname, user.firstname]
      end
      journal.each  do |j|
        j_details = []
        get_j_details = JournalDetail.where(:journal_id => j[0])
        if get_j_details.length > 0 then
          get_j_details.order(:id).each do |detail|
            case detail[:prop_key] 
            when 'status_id' then
              j_detail = ['status', SidfmdIssue.status_id_to_s(detail[:old_value].to_i), SidfmdIssue.status_id_to_s(detail[:value].to_i)]
            when 'due_date' then
              j_detail = ['due_date', SidfmdIssue.due_date_to_s(detail[:old_value]), SidfmdIssue.due_date_to_s(detail[:value])]
            else
              j_detail = ['other']
            end
            j_details.push(j_detail)
          end
        end
        j.push(users[j[1]])
        j.push(j_details)
        data.push(j)
      end
    end
    render :layout => false, :json => data.to_json
  end

  #履歴のコメント投稿時に履歴作成
  def api_post_journal_comment
    f_params =  params.permit(:html_id, :host_id, :comment, :user_id)
    html_id = f_params[:html_id].to_i 
    host_id = f_params[:host_id].to_i
    comment = f_params[:comment]
    user_id = f_params[:user_id].to_i
    pass_flag = 0
    if @sidfm_roles.include?(:admin) then
      pass_flag = 1
    else
      if @sidfm_roles.include?(:use) then
        host_categories = SidfmdHost.find(host_id).sidfmd_categories.map{|c|c.id}
        user_categories = User.current.sidfmd_categories.map{|c|c.id}
        user_categories.each do |c|
          if host_categories.include?(c) then
            pass_flag = 1
            break
          end
        end
      end
    end
    if pass_flag == 1 then
      begin 
        issue_id = SidfmdIssue.where(:html_id => html_id).where(:sidfmd_host_id => host_id).first
        if issue_id then
          Journal.transaction do
            j_comment = Journal.new(journalized_id: issue_id.issue_id, journalized_type: "Issue", user_id: user_id, notes: comment) 
            j_comment.save
          end
        end
          created_journal = Journal.last
          journal_content = [created_journal.created_on.strftime('%Y/%m/%d %H:%M:%S'), created_journal.notes]
          user = User.find(user_id)
          user_email = user.mail
          user_lastname = user.lastname
          user_firstname = user.firstname
          journal_content.push([user_email, user_lastname, user_firstname])
      rescue Exception => e
        log_exception(e)
      end
      render :layout => false, :json => journal_content
    else
      render status: 401, json: { status: 401, message: 'Unauthorized' }
    end
  end

  #ページロード時の履歴作成
  def api_get_all_jounals
    data = {}
    if @sidfm_roles.include?(:admin) then 
      category_ids = SidfmdCategory.all.map{|c|c.id}
    else
      category_ids = @user_category_ids
    end
    if !session['first_history_time'] then
      session['first_history_time'] = Time.current.to_time
    end
    host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => category_ids).map{|s| s.sidfmd_host_id}.uniq
    issues = SidfmdIssue.where(:sidfmd_host_id => host_ids)
    issue_ids =issues.map{|s|s.issue_id}.uniq
    recent_journals = Journal.where(:journalized_id => issue_ids)
                             .where(:created_on => NOTIFY_HISTORY_DATE..Time.current.to_time) 
    journalized_ids = recent_journals.map{|j|j.journalized_id}
    issues.where(:issue_id => journalized_ids).each do |issue|
      host_id = issue.sidfmd_host_id.to_i
      host_name = SidfmdHost.find(host_id).hostname
      if(data[host_id]) then
        data[host_id].push([issue.html_id.to_i, issue.title, host_name])
      else
        data[host_id] = [[issue.html_id.to_i, issue.title, host_name]]
      end
    end
    render :layout => false, :json => data.to_json
  end

  #アクセス後に投稿された新しい履歴を返す
  def api_get_new_histories
    data = {}
    if @sidfm_roles.include?(:admin) then
      category_ids = SidfmdCategory.all.map{|c|c.id}
    else
      category_ids = @user_category_ids
    end
    host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => category_ids).map{|s| s.sidfmd_host_id}.uniq
    issues = SidfmdIssue.where(:sidfmd_host_id => host_ids)
    issue_ids =issues.map{|s|s.issue_id}.uniq

    if session['delete_histories'] then
      hostid = session['delete_histories'][0].to_i
      htmlid = session['delete_histories'][1].to_i
      delete_issue = issues.where(:sidfmd_host_id => hostid).where(:html_id => htmlid).first
      if delete_issue then
        delete_issue_id = delete_issue.issue_id
        delete_journals = Journal.where(:journalized_id => delete_issue_id)
                                 .where(:created_on => session['first_history_time']..Time.current.to_time)
                                 .map{|j|j.id.to_i}
        if session['read_journals'] then
          session['read_journals'].push(*delete_journals)
        else
          session['read_journals'] = delete_journals
        end
      end
      session['delete_histories'] = nil
    end
    journals = Journal.where(:journalized_id => issue_ids)
                      .where(:created_on => session['first_history_time']..Time.current.to_time)                           
    journalized_ids = []
    journals.each do |j|
      id = j.id.to_i
      pass_flag = 1
      if session['read_journals'] then
        if session['read_journals'].include?(id) then
          pass_flag = 0
        end
      end
      if pass_flag == 1 then 
        j_id = j.journalized_id
        journalized_ids.push(j_id)
        issue = issues.find_by(:issue_id => j_id)
        host_id = issue.sidfmd_host_id.to_s
        if !data[host_id] then
          data[host_id] = {}
        end
        if data[host_id]['journals']
          data[host_id]['journals'] = data[host_id]['journals'] + 1
        else 
          data[host_id]['journals'] = 1
        end
      end 
    end
    issues.where(:issue_id => journalized_ids).each do |issue|  
      host_id = issue.sidfmd_host_id.to_s
      host_name = SidfmdHost.find(host_id.to_i).hostname
      issue_htmlid = issue.html_id.to_i
      title = issue.title

      if data[host_id]['htmlids'] then
        data[host_id]['htmlids'].push([issue_htmlid, title])
      else
        data[host_id]['htmlids'] = [[issue_htmlid, title]]
      end
      
      if !data[host_id]['hostname'] then
        data[host_id]['hostname'] = host_name
      end
    end

    data.each{|id, val|
      if host_ids.include?(id.to_i) then
        data[id]['flag'] = 'true'
      else
        data[id]['flag'] = 'false'
      end
    }
    render :layout => false, :json => data.to_json  
  end

  #操作したサマリー表のtrに色を付ける機能で使用
  def api_store_selected_id
    data = params.permit(:hostid_with_htmlid, :selected_id, :page_name)
    page_name = data[:page_name]
    if data[:hostid_with_htmlid] then
      data = JSON.parse(data[:hostid_with_htmlid])
      hostid = data[0].to_s
      htmlid = data[1].to_s
      if !session['hostid_with_htmlid'] then
        session['hostid_with_htmlid'] = {}
      end
      if !session['hostid_with_htmlid'][page_name]
        session['hostid_with_htmlid'][page_name] = {}
      end
      if page_name == 'host' then
        session['hostid_with_htmlid'][page_name][hostid] = [htmlid, false]
        render :layout => false, :json => {}.to_json
      elsif page_name == 'vul' then
        session['hostid_with_htmlid'][page_name][htmlid] = [hostid, false]
        render :layout => false, :json => {}.to_json
      end
    elsif data[:selected_id] then
      if page_name == 'host' then
        host_id = data[:selected_id].to_s
        selected_htmlid = []
        if session['hostid_with_htmlid'] && session['hostid_with_htmlid'][page_name] then
          if session['hostid_with_htmlid'][page_name][host_id] then
            object = session['hostid_with_htmlid'][page_name][host_id]
            if object[1] then
              selected_htmlid.push(object[0])
              session['hostid_with_htmlid'][page_name].delete(host_id)
            end
          end
        end
        render :layout => false, :json => selected_htmlid.to_json
      elsif page_name == 'vul' then
        html_id = data[:selected_id].to_s
        selected_hostid = []
        if session['hostid_with_htmlid'] && session['hostid_with_htmlid'][page_name] then
          if session['hostid_with_htmlid'][page_name][html_id] then
            object = session['hostid_with_htmlid'][page_name][html_id]
            if object[1] then
              selected_hostid.push(object[0])
              session['hostid_with_htmlid'][page_name].delete(html_id)
            end
          end
        end
        render :layout => false, :json => selected_hostid.to_json
      end
    end
  end

  def api_check_browser_back
    data = params.permit(:page_name)
    page_name = data[:page_name]
    if !session['chkbok_ids'][page_name]['is_browser_back'] then
      render :layout => false, :text => 'normal_load'
    else
      render :layout => false, :text => 'browser_back'
    end
    session['chkbok_ids'][page_name]['is_browser_back'] = true
  end

  #ルール設定のメール設定で必要なデータ(カテゴリ名・ユーザ名)を取得
  def api_get_data_for_mail_setting
    data = params.permit(:info_name, :ids)
    info_name = data[:info_name]
    ids = JSON.parse(data[:ids])
    ccids = User.current.sidfmd_categories.where(:is_system => false).map{|c| c.id}
    pass = true
    names = []
    if info_name == 'get_users_by_category_id' then
      if ids[0] == 'all'
        ids = SidfmdCategory.where(:is_system => false).map{|c| c.id}
      end
      if !@sidfm_roles.include?(:admin) then
        ids.each do |id|
          if !ccids.include?(id.to_i) then
            pass = false
          end
        end
      end
      userids = SidfmdCategoriesUser.where(:sidfmd_category_id => ids)
                                    .map{|c| c.user_id}
      User.where(:id => userids).order(:login => :asc).each do |user|
        fullname = user.lastname + user.firstname
        address = user.mail
        id = user.id
        names.push([id, fullname, address])
      end
    elsif info_name.include?('user') then
      if !@sidfm_roles.include?(:admin) then
        userids = SidfmdCategoriesUser.where(:sidfmd_category_id => ccids).map{|c| c.user_id}
        accessible_user_ids = []
        none_accessible_user_ids = []
        ids.each do |id|
          if userids.include?(id.to_i) then
            accessible_user_ids << id.to_i
          else
            none_accessible_user_ids << id.to_i
          end  
        end
        if accessible_user_ids.size > 0 then
          User.where(:id => accessible_user_ids).each do |user|
            fullname = user.lastname + user.firstname
            names << fullname
          end
          if none_accessible_user_ids.size > 0 then
            names << "他#{none_accessible_user_ids.size}名"
          end
        else
          names << "表示できないユーザです"
        end
      else
        User.where(:id => ids).each do |user|
          fullname = user.lastname + user.firstname
          names << fullname
        end 
      end
    elsif info_name.include?('category') then
      if !@sidfm_roles.include?(:admin) then
        accessible_category_ids = []
        none_accessible_category_ids = []
        ids.each do |id|
          if ccids.include?(id.to_i) then
            accessible_category_ids << id.to_i
          else
            none_accessible_category_ids << id.to_i
          end
        end
        if accessible_category_ids.size > 0 then
          SidfmdCategory.where(:id => accessible_category_ids).where(:is_system => false)
          .each do |cat|
            names << cat.name
          end
          if none_accessible_category_ids.size > 0 then
            names << "他#{none_accessible_category_ids.size}カテゴリ"
          end
        else
          names << "表示できないカテゴリです"
        end
      else
        if ids[0] == 'all' then
          cats = SidfmdCategory.where(:is_system => false)
        else
          cats = SidfmdCategory.where(:id => ids).where(:is_system => false)
        end
        cats.each do |cat|
          names.push(cat.name)
        end 
      end
    end
    if pass then
      render :layput => false, :json => names.to_json
    else
      render status: 400, json: { status: 400, message: 'Wrong Parameter' } 
    end 
  end

  def api_get_hosts_by_category_id 
    data = params.permit(:keyword, :category_id)
    keyword = data[:keyword]
    category_ids = JSON.parse(data[:category_id])
    if category_ids[0] == 'all' then
      category_ids = SidfmdCategory.where(:is_system => false)
                                  .map{|c| c.id}
    end
    if keyword and keyword.size >= 2 then
      hosts = get_hosts(keyword)
      host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => category_ids).map{|c| c.sidfmd_host_id}
      hosts = hosts.where(:id => host_ids)
    end
    render :layout => false, :json => hosts.to_json
  end

  #同名ルールがあるかどうかを返す
  def api_is_same_rule_name
    data = params.permit(:rule_name)
    rule_name = data[:rule_name]
    rule = SidfmdRule.find_by(:name => rule_name)
    rule_data = {}
    if rule then
      rule_data["id"] = rule.id
      rule_data["name"] = rule.name
    end
    render :layout => false, :json => rule_data.to_json
  end
 
  #全プロダクトを返す。ホストに登録されているものにはホスト名を付ける
  def api_get_products_with_hostname
    data = params.permit(:keyword)
    keyword = data[:keyword]
    ret = {}
    if keyword and keyword.size >= 2 then
      hosts = SidfmdHost.where(:flg_delete => false)
      hosts_versions = {}
      hosts.each do |host|
        hostname = host.hostname
        host.sidfmd_host_apps.each do |app|
          v_id = app.version_id
          hosts_versions[v_id] = [] unless hosts_versions[v_id]
          hosts_versions[v_id].push(hostname)
        end
        host.get_agent_products.each do |prod|
          v_id = prod["version_id"]
          hosts_versions[v_id] = [] unless hosts_versions[v_id]
          hosts_versions[v_id].push(hostname) if v_id
        end  
      end
      products = SidfmdProduct
        .where('LOWER(name) LIKE ? OR LOWER(name) LIKE ?',
               keyword.downcase + '%',
               '% ' + keyword.downcase + '%')
        .order(:category)
      products.each do |product|
        ret[product.pid] = { 'product' => product }
      end
      versions = SidfmdVersion
        .where.not(:version => 'ALL')
        .where('LOWER(fullname) LIKE ? OR LOWER(fullname) LIKE ?',
               keyword.downcase + '%',
               '% ' + keyword.downcase + '%')
        .order(:position => :desc)
      versions.each do |version|
        product = version.sidfmd_product
        if !ret.has_key?(product.pid) then
          ret[product.pid] = { 'product' => product }
        end
        ret[product.pid]['versions'] = [] unless ret[product.pid].has_key?('versions')
        ret[product.pid]['versions'] << {
                                          "data" => version, 
                                          "related-host" => hosts_versions[version.vid]
                                        }
      end
    end
    render :layout => false, :json => ret.to_json
  end

  #idからversionもしくはproductの名前を返す
  def api_get_products_and_versions_by_ids
    f_param = params.permit(:product_ids, :version_ids)
    prod_ids = JSON.parse(f_param[:product_ids]) if f_param[:product_ids]
    ver_ids = JSON.parse(f_param[:version_ids]) if f_param[:version_ids]
    data = {}
    products = []
    versions = []
    SidfmdProduct.where(:id => prod_ids)
                 .each do |p|
                   products.push(
                     { "id" => p.id, "name" =>  p.name }
                   )
                 end
    SidfmdVersion.where(:id => ver_ids)
                 .each do |v| 
                   versions.push(
                     { "id" => v.id, "name" => v.fullname }
                   )
                 end
    data[:products] = products
    data[:versions] = versions
    render :layout => false, :json => data.to_json
  end
   
  #ルール設定画面のメール送信画面上にあるキーワードを変換ボタンを押した時にキーワードを変換した文字列を返す
  #出力されるデータは全てサンプル
  def api_replace_mail_keywords
    f_param = params.permit(:title,:message)
    opts = {}
    mail_title = f_param[:title]
    message = f_param[:message]
    vulsync_info_titles = {
      11287 => "GNU Wget の HTTP リダイレクトの処理に任意のファイルを上書きされる問題",
      15766 => "FreeType の BDF フォントの処理にサービスを妨害される問題",
      19285 => "GNU Bash の環境変数の処理に任意のコマンドを実行される問題",
      26981 => "Linux Kernel の Bluetooth の処理に任意のコードを実行される問題",
      15602 => "CUPS に特権を奪われる問題"
    }
    risk_rate_description = ->(c_percent, h_percent, m_percent, l_percent, n_percent, f_percent, risk_index){
      return ret = if risk_index == "sri" then
        "重大:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
        "高:#{h_percent >= 10 ? h_percent.round(1) : (h_percent >= 1 ? h_percent.round(2) : h_percent.round(3))}%," +
        "中:#{m_percent >= 10 ? m_percent.round(1) : (m_percent >= 1 ? m_percent.round(2) : m_percent.round(3))}%," +
        "低:#{l_percent >= 10 ? l_percent.round(1) : (l_percent >= 1 ? l_percent.round(2) : l_percent.round(3))}%," +
        "無:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%," +
        "済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%"
      else
        "未登録:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%," +
        "10.0～9.0:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
        "8.9～7.0:#{h_percent >= 10 ? h_percent.round(1) : (h_percent >= 1 ? h_percent.round(2) : h_percent.round(3))}%," +
        "6.9～4.0:#{m_percent >= 10 ? m_percent.round(1) : (m_percent >= 1 ? m_percent.round(2) : m_percent.round(3))}%," +
        "3.9～0.0#{l_percent >= 10 ? l_percent.round(1) : (l_percent >= 1 ? l_percent.round(2) : l_percent.round(3))}%," +
        "済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%"
      end
    }
    handling_rate_description = ->(c_percent, g_percent, f_percent, n_percent){
      "未対策:#{c_percent >= 10 ? c_percent.round(1) : (c_percent >= 1 ? c_percent.round(2) : c_percent.round(3))}%," +
      "対策中:#{g_percent >= 10 ? g_percent.round(1) : (g_percent >= 1 ? g_percent.round(2) : g_percent.round(3))}%," +
      "対策済:#{f_percent >= 10 ? f_percent.round(1) : (f_percent >= 1 ? f_percent.round(2) : f_percent.round(3))}%," +
      "影響無:#{n_percent >= 10 ? n_percent.round(1) : (n_percent >= 1 ? n_percent.round(2) : n_percent.round(3))}%"
    }
    filter_list_data = ->(type, data_num, content, content_size, no_data_message, join_type, limit){
      return ret = if type == "default" then
        content_size > 0 ? content.slice(0..limit - 1).join(join_type) : no_data_message
      elsif type == "case"
        data_num.nil? ? content_size : (content_size + data_num > 0 ? content_size + data_num : 0)
      elsif type == "top" then
        content_size > 0 ? content.slice(0..data_num - 1).join(join_type) : no_data_message
      else
        content_size > 0 ? content.reverse.slice(0..data_num - 1).join(join_type) : no_data_message
      end
    }
    messages_keywords = SidfmdMailer.get_keywords(message)
    titles_keywords = SidfmdMailer.get_keywords(mail_title)
    limit_num = SidfmdMailer::MAIL_LIMIT_DATA_NUM
    keywords = messages_keywords.merge(titles_keywords)
    keywords.each do |filtered_keyword, data|
      keyword = data[:keyword] 
      type = data[:type]
      data_num = data[:necessary_data_num] 
      case keyword 
      when "%RISK_RATE_SRI_ALL%" then
        opts[filtered_keyword] = risk_rate_description.(4.08, 29.3, 41.0, 24.1, 1.2, 0.298, "sri")
      when "%UNHANDLED_RATE_SRI_ALL%" then
        opts[filtered_keyword] = "38%"
      when "%RISK_RATE_SRI_MATCH%" then
        opts[filtered_keyword] = risk_rate_description.(5.08, 19.3, 41.0, 24.1, 2.2, 10.298, "sri")
      when "%UNHANDLED_RATE_SRI_MATCH%" then
        opts[filtered_keyword] = "28%"
      when "%RISK_RATE_CVSS_ALL%" then
        opts[filtered_keyword] = risk_rate_description.(15.08, 19.3, 32.0, 23.1, 3.2, 8.298, "cvss")
      when "%UNHANDLED_RATE_CVSS_ALL%"
        opts[filtered_keyword] = "20%"
      when "%RISK_RATE_CVSS_MATCH%"
        opts[filtered_keyword] = risk_rate_description.(25.08, 29.3, 52.0, 3.1, 1.2, 10.298, "cvss")
      when "%UNHANDLED_RATE_CVSS_MATCH%" then
        opts[filtered_keyword] = "30%"
      when "%HANDLING_RATE_ALL%" then
        opts[filtered_keyword] = handling_rate_description.(38.33, 32.67, 15.33, 14.77)
      when "%HANDLING_RATE_MATCH%" then
        opts[filtered_keyword] = handling_rate_description.(58.33, 22.67, 5.33, 14.77)
      when "%URL%" then
        opts[filtered_keyword] = sidfm_rule_setting_index_url
      when "%TITLE%" then
        opts[filtered_keyword] = SidfmdRule::PAGE_TITLE
      when "%OS_RATE_ALL%" then
        opts[filtered_keyword] = "Ubuntu 18.04 LTS: 33.3%, CentOS 7: 33.3%, （不明）: 33.3%"
      when "%OS_RATE_MATCH%" then
        opts[filtered_keyword] = "Ubuntu 18.04 LTS: 50.0%, CentOS 7: 50.0%"
      when "%OS_ALL%" then
        all_os_names = ["Ubuntu 18.04 LTS", "CentOS 7"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_os_names, all_os_names.size, "対象のOSはありません。", ', ', limit_num)
      when "%OS_MATCH%" then
        match_all_os_names = ["Ubuntu 18.04 LTS", "CentOS 7"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_os_names, match_all_os_names.size, "対象のOSはありません。", ', ', limit_num)
      when "%HOST_ALL%" then
        hostnames = ["host-a", "host-b", "host-c"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hostnames, hostnames.size, "対象のホストはありません。", ', ', limit_num)
      when "%HOST_MATCH%" then
        match_hostnames = ["host-a", "host-c"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hostnames, match_hostnames.size, "対象のホストはありません。", ', ', limit_num)
      when "%HOSTINFO_ALL%" then
        hosts_info = [
          "host-a\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}\n" + 
          "OS:Ubuntu 18.04 LTS\n" +
          "システムカテゴリ:内部サーバ   カスタムカテゴリ:サンプルA, サンプルB\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:9.8   影響範囲:リモート   攻撃コード:あり\n" +
          "対応状況: 未対策101件  対策中60件 対策済20件  影響無1件\n" +
          "対策予定日設定済:0.0%   対処率:77.3%\n" +
          "検出パッケージ数:100   検出アプリケーション数:5   手動登録プロダクト数:1\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/05/11 20:38:54\n" +
          "メモ:サンプル1",
          "host-b\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10001)}\n" +   
          "OS:---\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値: 5.8   影響範囲:リモート   攻撃コード:なし\n" +
          "対応状況: 未対策11件  対策中50件 対策済120件  影響無10件\n" +
          "対策予定日設定済:10.0%   対処率:90.0%\n" +
          "検出パッケージ数:0   検出アプリケーション数:0   手動登録プロダクト数:1\n" +
          "同期状態:非同期   エージェントの使用状態:不使用\n" +
          "最終確認日時:2020/03/11 12:31:50\n" +
          "メモ:サンプル2",
          "host-c\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}\n" +   
          "OS:CentOS 7\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA, サンプルB, サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:重大   最大CVSS値:9.7   影響範囲:クライアント   攻撃コード:あり\n" +
          "対応状況: 未対策211件  対策中10件 対策済10件  影響無10件\n" +
          "対策予定日設定済:20.0%   対処率:80.0%\n" +
          "検出パッケージ数:200   検出アプリケーション数:10   手動登録プロダクト数:3\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" + 
          "最終確認日時:2020/01/11 11:11:00\n" +
          "メモ:サンプル3"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hosts_info, hosts_info.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%HOSTINFO_MATCH%" then
        match_hosts_info = [
          "host-a\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}\n" +
          "OS:Ubuntu 18.04 LTS\n" +
          "システムカテゴリ:内部サーバ   カスタムカテゴリ:サンプルA, サンプルB\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:9.8   影響範囲:リモート   攻撃コード:あり\n" +
          "対応状況: 未対策101件  対策中60件 対策済20件  影響無1件\n" +
          "対策予定日設定済:0.0%   対処率:77.3%\n" +
          "検出パッケージ数:100   検出アプリケーション数:5   手動登録プロダクト数:1\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/05/11 20:38:54\n" +
          "メモ:サンプル1",
          "host-c\n" +
          "#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}\n" +
          "OS:CentOS 7\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA, サンプルB, サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:重大   最大CVSS値:9.7   影響範囲:クライアント   攻撃コード:あり\n" +
          "対応状況: 未対策211件  対策中10件 対策済10件  影響無10件\n" +
          "対策予定日設定済:20.0%   対処率:80.0%\n" +
          "検出パッケージ数:200   検出アプリケーション数:10   手動登録プロダクト数:3\n" +
          "同期状態:同期中   エージェントの使用状態:使用\n" +
          "最終確認日時:2020/01/11 11:11:00\n" +
          "メモ:サンプル3"
        ] 
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hosts_info, match_hosts_info.size, "対象のホストはありません。", "\n\n", limit_num) 
      when "%S_CATEGORY_ALL%" then
        s_categories = ["公開サーバ", "内部サーバ"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, s_categories, s_categories.size, "対象のシステムカテゴリはありません。", ', ', limit_num)
      when "%S_CATEGORY_MATCH%" then
        match_s_categories = ["公開サーバ", "内部サーバ"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_s_categories, match_s_categories.size, "対象のシステムカテゴリはありません。", ', ', limit_num)
      when "%C_CATEGORY_ALL%" then
        c_categories = ["サンプルA", "サンプルB", "サンプルC"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, c_categories, c_categories.size, "対象のカスタムカテゴリはありません。", ', ', limit_num)
      when "%C_CATEGORY_MATCH%" then
        match_c_categories = ["サンプルA", "サンプルB", "サンプルC"]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_c_categories, match_c_categories.size, "対象のカスタムカテゴリはありません。", ', ', limit_num)
      when "%HOST_LINKS_ALL%" then
        hosts_links = [
          "host-a\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}",
          "host-b\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10001)}",
          "host-c\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, hosts_links, hosts_links.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%HOST_LINKS_MATCH%" then
        match_hosts_links = [
          "host-a\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10000)}",
          "host-c\n#{url_for(:controller => :sidfm_host, :action => :index, :id => 10002)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_hosts_links, match_hosts_links.size, "対象のホストはありません。", "\n\n", limit_num)
      when "%VULINFO_ALL%" then
        vuls_info = [
          "[23355]Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 23355)}\n" +
          "関連するホスト:host-a, host-b\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値:7.8   影響範囲:ローカル   攻撃コード:なし\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/18   検出後最長経過日数:116   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "SIDfmID:23355\n" +
          "脆弱性情報の詳細ページへのリンク:\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23355)}",
          "[KB2984943]Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => -2984943)}\n" +
          "関連するホスト:hostb. host-c\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:n/a   影響範囲:未定義   攻撃コード:n/a\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/14   検出後最長経過日数:119   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "KB番号:KB2984943\n" +
          "Microsoft Update カタログリンク:\n" +
          "https://www.catalog.update.microsoft.com/Search.aspx?q=KB2984943\n" +
          "SIDfm情報:\n" +
          "23348 Microsoft Office の Office ファイルの処理に任意のコードを実行されるなど複数の問題 (MS16-054)\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23348)}"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, vuls_info, vuls_info.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VULINFO_MATCH%" then
        match_vuls_info = [
          "[23355]Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => 23355)}\n" +
          "関連するホスト:host-a, host-b\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルA\n" +
          "対応:CVSS値を元にした場合、不要。SRI値を元にした場合、不要。\n" +
          "最大SRI値:中   最大CVSS値:7.8   影響範囲:ローカル   攻撃コード:なし\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/18   検出後最長経過日数:116   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "SIDfmID:23355\n" +
          "脆弱性情報の詳細ページへのリンク:\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23355)}",
          "[KB2984943]Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム\n" +
          "#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => -2984943)}\n" +
          "関連するホスト:hostb. host-c\n" +
          "システムカテゴリ:公開サーバ   カスタムカテゴリ:サンプルC\n" +
          "対応:CVSS値を元にした場合、要。SRI値を元にした場合、要。\n" +
          "最大SRI値:高   最大CVSS値:n/a   影響範囲:未定義   攻撃コード:n/a\n" +
          "対応状況: 未対策2件  対策中0件 対策済0件   影響無0件\n" +
          "検出日:2020/01/14   検出後最長経過日数:119   対策予定日設定済:50.0%   対処率:0.0%\n" +
          "KB番号:KB2984943\n" +
          "Microsoft Update カタログリンク:\n" +
          "https://www.catalog.update.microsoft.com/Search.aspx?q=KB2984943\n" +
          "SIDfm情報:\n" +
          "23348 Microsoft Office の Office ファイルの処理に任意のコードを実行されるなど複数の問題 (MS16-054)\n" +
          "#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => 23348)}" 
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vuls_info, match_vuls_info.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VULTITLE_ALL%" then
        vul_titles = [
          "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          "Adobe Flash Player に任意のコードを実行される問題",
          "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          "CentOS 6 の samba に複数のセキュリティホール",
          "Microsoft Windows の Adobe フォントドライバの処理に任意のコードを実行される複数の問題 (MS15-021)",
          "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          "CentOS の firefox に複数のセキュリティホール"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, vul_titles, vul_titles.size, "対象の脆弱性はありません。", ",\n", limit_num)
      when "%VULTITLE_MATCH%" then
        match_vul_titles = [
          "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          "Adobe Flash Player に任意のコードを実行される問題",
          "CentOS 6 の samba に複数のセキュリティホール",
          "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vul_titles, match_vul_titles.size, "対象の脆弱性はありません。", ",\n", limit_num)
      when "%VUL_LINKS_ALL%" then
        vulns = {
          23355 => "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          24346 => "Microsoft Windows に Kernel API の処理に権限を昇格されるなど複数の問題 (MS16-111)",
          22373 => "CentOS 6 の samba に複数のセキュリティホール",
          20294 => "Microsoft Windows の Adobe フォントドライバの処理に任意のコードを実行される複数の問題 (MS15-021)",
          23396 => "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          21520 => "CentOS の firefox に複数のセキュリティホール", 
        }
        vuls_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}   
        opts[filtered_keyword] = filter_list_data.(type, data_num, vuls_links, vuls_links.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%VUL_LINKS_MATCH%" then
        vulns = {
          23355 => "Microsoft Windows のカーネルの処理に特権を奪われる問題 (MS16-060)",
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
        }
        match_vuls_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_vuls_links, match_vuls_links.size, "対象の脆弱性はありません。", "\n\n", limit_num)
      when "%SIDfmID_ALL%" then
        sidfm_ids = [15685, 22373, 23396, 19767, 21520]
        opts[filtered_keyword] = filter_list_data.(type, data_num, sidfm_ids, sidfm_ids.size, "関連するSIDfmIDはありません。", ", ", limit_num)
      when "%SIDfmID_MATCH%" then
        match_sidfm_ids = [15685, 19767]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_sidfm_ids, match_sidfm_ids.size, "関連するSIDfmIDはありません。", ", ", limit_num)
      when "%SIDfm_LINKS_ALL%" then
        vulns = {
          15685 => "Adobe Flash Player に任意のコードを実行される問題",
          22373 => "CentOS 6 の samba に複数のセキュリティホール",
          23396 => "Red Hat Enterprise Linux 6 の ntp に複数のセキュリティホール",
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
          21520 => "CentOS の firefox に複数のセキュリティホール",
        }
        sidfm_info_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => id)}"} 
        opts[filtered_keyword] = filter_list_data.(type, data_num, sidfm_info_links, sidfm_info_links.size, "関連する「脆弱性情報の詳細」ページはありません。", "\n\n", limit_num)
      when "%SIDfm_LINKS_MATCH%" then
        vulns = {
          15685 => "Adobe Flash Player に任意のコードを実行される問題", 
          19767 => "Adobe Flash Player に任意のコードを実行されるなど複数の問題",
        }
        match_sidfm_info_links = vulns.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_informations, :action => :vuln, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_sidfm_info_links, match_sidfm_info_links.size, "関連する「脆弱性情報の詳細」ページはありません。", "\n\n", limit_num)
      when "%MS_LINKS_ALL%" then
        kbs = {
          2984943 => "Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム",
          2825645 => "Microsoft Office 2007 suites (KB2825645)   のセキュリティ更新プログラム",
          2899473 => "Microsoft Office 2007 suites (KB2899473)   のセキュリティ更新プログラム"
        }
        ms_links = kbs.map{|kb, title| "#{title}\nhttps://www.catalog.update.microsoft.com/Search.aspx?q=KB#{kb}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, ms_links, ms_links.size, "関連する「Microsoft Update カタログ」はありません。", "\n\n", limit_num)
      when "%MS_LINKS_MATCH%" then
        kbs = {
          2984943 => "Microsoft Office 2007 suites (KB2984943)   のセキュリティ更新プログラム", 
          2825645 => "Microsoft Office 2007 suites (KB2825645)   のセキュリティ更新プログラム",
        }
        match_ms_links = kbs.map{|kb, title| "#{title}\nhttps://www.catalog.update.microsoft.com/Search.aspx?q=KB#{kb}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_ms_links, match_ms_links.size, "関連する「Microsoft Update カタログ」はありません。", "\n\n", limit_num)
      when "%KB_ALL%" then
        kb_nums = [2984943, 2825645, 2899473]
        opts[filtered_keyword] = filter_list_data.(type, data_num, kb_nums, kb_nums.size, "関連するKB番号無し", ", ", limit_num)
      when "%KB_MATCH%" then
        match_kb_nums = [2984943, 2825645]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_kb_nums, match_kb_nums.size, "関連するKB番号無し", ", ", limit_num)
      when "%APP_ALL%" then
        all_apps = [
          "Apache Struts 1.3.9\n/home/softek/agenttools/dummy/struts/struts-1.3.9/struts-core-1.3.9.jar",
          "Apache Struts 2.5.16\n/home/softek/agenttools/dummy/struts/struts-2.5.16/struts2-core-2.5.16.jar",
          "Apache Tomcat 7.0.85\n/home/softek/agenttools/dummy/tomcat7/apache-tomcat-7.0.85/catalina.jar"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_apps, all_apps.size, "検出アプリケーションはありません。", "\n\n", limit_num)
      when "%APP_MATCH%" then
        match_all_apps = [
          "Apache Struts 1.3.9\n/home/softek/agenttools/dummy/struts/struts-1.3.9/struts-core-1.3.9.jar",
          "Apache Struts 2.5.16\n/home/softek/agenttools/dummy/struts/struts-2.5.16/struts2-core-2.5.16.jar",
          "Apache Tomcat 7.0.85\n/home/softek/agenttools/dummy/tomcat7/apache-tomcat-7.0.85/catalina.jar"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_apps, match_all_apps.size, "検出アプリケーションはありません。", "\n\n", limit_num)
      when "%PRODUCT_ALL%" then
        all_lapps = [
          "CentOS 7", "Microsoft Windows 8", "Microsoft Windows Media Player 12"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, all_lapps, all_lapps.size, "手動登録プロダクトはありません。", ", ", limit_num)
      when "%PRODUCT_MATCH%" then
        match_all_lapps = [
          "CentOS 7", "Microsoft Windows 8", "Microsoft Windows Media Player 12"
        ]
        opts[filtered_keyword] = filter_list_data.(type, data_num, match_all_lapps, match_all_lapps.size, "手動登録プロダクトはありません。", ", ", limit_num)
      when "%VULSYNC_REGI_ALL%" then
        new_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, new_vuls, new_vuls.size, "追加された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_RENEW_ALL%" then
        mod_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, mod_vuls, mod_vuls.size, "更新された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_DEL_ALL%" then
        del_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, del_vuls, del_vuls.size, "削除(ステータスが「影響無」に変更)された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_REGI_MATCH%" then
        new_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, new_vuls, new_vuls.size, "追加された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_RENEW_MATCH%" then
        mod_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, mod_vuls, mod_vuls.size, "更新された脆弱性はありません。", "\n\n", limit_num)
      when "%VULSYNC_DEL_MATCH%" then
        del_vuls = vulsync_info_titles.map{|id, title| "#{title}\n#{url_for(:controller => :sidfm_vulnerability, :action => :index, :id => id)}"}
        opts[filtered_keyword] = filter_list_data.(type, data_num, del_vuls, del_vuls.size, "削除(ステータスが「影響無」に変更)された脆弱性はありません。", "\n\n", limit_num)
      when "%RULE_NAME%" then
        opts[filtered_keyword] = "要対応の脆弱性を検知(サンプル)"
      when "%RULE_CATEGORY%" then
        opts[filtered_keyword] = "サンプルA"
      when "%RULE_TIMING%" then
        opts[filtered_keyword] = "毎日実行-時間指定(実行時間: 1時)"
      when "%RULE_COND%" then
        opts[filtered_keyword] = "脆弱性同期時(登録):脆弱性の登録が10件以上,\n" +
                                 "SRI指定値以上:対象の脆弱性の危険度が高以上,\n" +
                                 "カスタムカテゴリ限定: サンプルA"
      when "%RULE_ACTION%" then
        opts[filtered_keyword] = "メール送信(ユーザ),\n" +
                                 "対策予定日設定:対策予定日を7日後に設定する(上書き設定: 上書き)"
      when "%RULE_DATE%" then
        opts[filtered_keyword] = Date.today.strftime("%Y年%m月%d日")
      end
    end
    mail_body = SidfmdMailer.replace_keywords(message, opts)
    mail_title = SidfmdMailer.replace_keywords(mail_title, opts)
    data = {}
    data[:title] = mail_title
    data[:message] = mail_body
    render :layout => false, :json => data
  end

  #ルールのカテゴリ変更時にアラートを表示する
  def check_change_rule_category
    rule_params = params.permit(:new_category_name, :new_category_id, :trigger, :conds, :actions)
    new_category_name = rule_params[:new_category_name]
    new_category_id = rule_params[:new_category_id].to_s
    trigger = JSON.parse(rule_params[:trigger])
    conds = JSON.parse(rule_params[:conds])
    actions = JSON.parse(rule_params[:actions])
    all_cond_names = SidfmdRule::RULE_COND_NAMES
    all_action_names = SidfmdRule::RULE_ACTION_NAMES
    is_trigger_vulsync = (trigger["period"] == "vulsync")
    new_param = {}
    message = ""
    conf_cond_names = {}
    conf_action_names = {}  
    remove_users = []  
    if is_trigger_vulsync then
      if trigger["hostname"] != "" then
        message += "タイミング設定に設定されているホストが削除されます。"
        trigger["hostname"] = ""
      end
      trigger["category"] = new_category_id
    end
    conds.each_key do |param_name|
      if(param_name == "category-match" || param_name == "category-unmatch" || param_name == "host-match" || param_name == "host-unmatch")&&
        new_category_id != "all"
      then
        conf_cond_names[param_name] = "「" + all_cond_names[param_name] + "」"
        conds.delete(param_name)
      end
    end
    if conf_cond_names.size > 0 then
      message += "また、" if message != ""
      message += "条件に設定されている " + conf_cond_names.values.join(", ") + " が削除されます。"
    end
    actions.each_key do |param_name|
      if param_name == "send-mail-category" || param_name == "send-mail-user" then
        mail_data = actions[param_name]
        delete_flg = false 
        if mail_data["sender"]["userid"] && new_category_id != "all" then
          user_cats = User.find(mail_data["sender"]["userid"]).sidfmd_categories.map(&:id)
          if !user_cats.include?(new_category_id.to_i) then 
            conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
            actions.delete(param_name)
            delete_flg = true
          end
        end
        if param_name == "send-mail-user" && !delete_flg && new_category_id != "all" && mail_data["sendby"]["ids"] then
          ids = mail_data["sendby"]["ids"]
          if ids.size == 1 then
            user_cats = User.find(ids[0]).sidfmd_categories.map(&:id)
            if !user_cats.include?(new_category_id.to_i) then
              conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
              actions.delete(param_name)
            end
          elsif ids.size > 1 then
            new_category_id = new_category_id.to_i
            User.where(:id => ids).each do |user|
              user_cats = user.sidfmd_categories.map{ |c| c.id }
              if !user_cats.include?(new_category_id) then
                remove_users << user.id 
              end
            end
            if remove_users.size == ids.size then
              conf_action_names[param_name] = "「" + all_action_names[param_name] + "」"
              actions.delete(param_name)
              remove_users = []
            elsif remove_users.size > 0 then
              actions[param_name]["sendby"]["ids"] = ids - remove_users
            end
          end 
        end
      end
    end
    if conf_action_names.size > 0 then
      message += "加えて、" if message != ""
      message += "操作に設定されている" + conf_action_names.values.join(", ") + "が削除されます。"
    end
    if remove_users.size > 0 then
      message += " #{all_action_names['send-mail-user']}の操作の送信先に設定されている#{remove_users.size}名のユーザは、変更するカテゴリに属さないユーザのため、送信先から削除されます。"
    end
    if message != "" then
      message = "このカテゴリの選択に関連する項目が設定されています。変更に進むと、" + message + "よろしいですか?"
      new_param[:message] = message
      new_param[:trigger] = trigger
      new_param[:cond] = conds
      new_param[:action] = actions
      new_param[:del_conds] = conf_cond_names.keys
      new_param[:del_actions] = conf_action_names.keys
      new_param[:is_alert] = true
      new_param[:remove_users] = remove_users if remove_users.size > 0
    else
      new_param[:is_alert] = false
      new_param[:trigger] = trigger
    end
    render :layout => false, :json => new_param
  end

  #カテゴリ設定・ホスト設定・ユーザ管理画面で、カテゴリ・ホスト・ユーザ削除時にルールをチェックし、アラート表示+ルールの内容を変更
  def api_check_related_rules 
    f_params = params.permit(:page_name, :delete_id, :action_idx, :user_ids, :host_ids, :hostnames, :category_ids)
    page_name = f_params[:page_name]
    delete_id = (f_params[:delete_id] || 0).to_i
    trigger_names = SidfmdRule::RULE_TRIGGER_NAMES
    cond_names = SidfmdRule::RULE_COND_NAMES
    action_names = SidfmdRule::RULE_ACTION_NAMES
    delete_rule_ids = []
    delete_cond_ids = []
    delete_action_ids = []
    if page_name == "hostadmin" then #ホスト設定ページでホスト削除時のチェック
      action_idx = f_params[:action_idx]
      if action_idx == "category_change" || action_idx == "bulk_category_change" then #ホストのカテゴリ変更時
        if action_idx == "category_change" then 
          message_prefix = "この"
          host_ids = JSON.parse(f_params[:host_ids]) 
        else 
          message_prefix = "選択中の"
          hostnames = JSON.parse(f_params[:hostnames])
          host_ids = SidfmdHost.where(:hostname => hostnames).map{|h| h.id}
        end
        category_ids = JSON.parse(f_params[:category_ids])
        host_cond_ids = []
        #ホストのカテゴリがなくなるならルールを削除
        if category_ids.empty? then
          delete_rule_ids = SidfmdRuleTVulsync.where(:sidfmd_host_id => host_ids).map{|t| t.sidfmd_rule_trigger.sidfmd_rule.id}
        end
        #triggerにホストが設定されているかチェック
        SidfmdRuleTVulsync.where(:sidfmd_host_id => host_ids).where.not(:sidfmd_category_id => nil).each do |vulsync_trigger|
          #変更するカテゴリが設定されているカテゴリに含まれていなければルール削除
          if !category_ids.include?(vulsync_trigger.sidfmd_category_id) then
            rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
            delete_rule_ids << rule.id unless delete_rule_ids.include?(rule.id)
          end
        end
        #ホスト限定・除外の条件にホストが設定されているかチェック
        SidfmdRuleCmHostSub.where(:sidfmd_host_id => host_ids).each do |host|
          sub_cond = host.sidfmd_rule_c_match_host
          cond = sub_cond.sidfmd_rule_cond
          rule = cond.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            #ルールのtriggerがvulsyncで且つカテゴリが全体でない、またはカスタムカテゴリ限定の条件があり、
            #変更するカテゴリが設定されているカテゴリに含まれていなければホストを条件から外す
            rule_conds = rule.sidfmd_rule_conds
            rule_match_cat_conds_cat_ids = []
            is_match_category_cond = false
            rule_conds.each do |c|
              detail = c.detail
              if detail.class.table_name == "sidfmd_rule_c_match_categories" &&
                detail.inv == false
              then
                is_match_category_cond = true
                sub_cat_ids = detail.sidfmd_rule_cm_category_subs.map{|sub_c|
                                sub_c.sidfmd_category_id
                              } 
                rule_match_cat_conds_cat_ids = sub_cat_ids.select{|c_id| category_ids.include?(c_id)}
              end
            end
            if (rule.sidfmd_category_id && !category_ids.include?(rule.sidfmd_category_id))||
               (!rule.sidfmd_category_id && is_match_category_cond && rule_match_cat_conds_cat_ids.size == 0)||
               (category_ids.size == 0)
            then
              sub_cond_ids = SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).map{|sub_c| sub_c.id}
              #ホストが１つだけだったら条件を削除の対象にする  
              if sub_cond_ids.count == 1 then
                #この条件を削除すると条件がなくなるならルールを削除
                if rule_conds.size == 1 then
                  delete_rule_ids << rule.id
                else
                  delete_cond_ids << cond.id
                end
              else
                host_cond_ids << host.id
                if (sub_cond_ids - host_cond_ids).size == 0 then
                  #全てのホストが条件から外れるなら条件を削除※カテゴリ一括更新の時のみこのscopeに入る
                  delete_cond_ids << cond.id
                  host_cond_ids = host_cond_ids - sub_cond_ids
                end 
              end
            end
          end
        end
        #カテゴリ一括変更時にチェック。条件が全てなくなるならルール削除
        if action_idx == "bulk_category_change" then
          checked_rule_ids = []
          delete_cond_ids.each do |cond_id|
            rule = SidfmdRuleCond.find(cond_id).sidfmd_rule
            rule_id = rule.id
            if !checked_rule_ids.include?(rule_id) then
              checked_rule_ids << rule_id
              cond_ids = rule.sidfmd_rule_conds.map{|c| c.id}
              if (cond_ids - delete_cond_ids).size == 0 then
                delete_rule_ids << rule_id
                delete_cond_ids = delete_cond_ids - cond_ids
              end
            end
          end
        end
        message = ""
        if delete_rule_ids.size > 0 then
          message += "#{message_prefix}ホストが設定されているルールが#{delete_rule_ids.size}件あり、\nカテゴリを変更すると設定されているルールも共に削除されます。"
        end
        if delete_cond_ids.size > 0 || host_cond_ids.size > 0 then
          message += "\nまた、" if message != ""
          message += "登録されているルールに「#{cond_names['host-match']}」または「#{cond_names['host-unmatch']}」の条件が設定されていて、\nカテゴリを変更することによってそのルールのカテゴリに#{message_prefix}ホストが属さなくなる場合、\nその条件の対象ホストが#{message_prefix}ホストのみの場合は条件が削除され、そうでない場合はその条件の対象ホストから#{message_prefix}ホストが削除されます。"
        end
        message += "\n削除してもよろしいですか?" if message != ""
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_cond_ids => delete_cond_ids,
                 :delete_host_cond_ids => host_cond_ids,
                 :message => message
               } 
      else
        #脆弱性同期時実行のタイミングで、対象ホストに削除するホストが設定されていれば、そのルールを削除の対象にする
        SidfmdRuleTVulsync.where(:sidfmd_host_id => delete_id).each do |vulsync_trigger|
          rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
          delete_rule_ids << rule.id
        end
        #ホスト限定・除外に削除するホストが設定されていればそのホストを外す
        host_cond_ids = []
        SidfmdRuleCmHostSub.where(:sidfmd_host_id => delete_id).each do |host|
          sub_cond = host.sidfmd_rule_c_match_host
          cond = sub_cond.sidfmd_rule_cond
          rule = cond.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            #ホストが１つだけだったら条件を削除の対象にする
            if SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).count == 1 then
              #この条件を削除すると条件がなくなるならルールを削除
              if rule.sidfmd_rule_conds.size == 1 then
                delete_rule_ids << rule.id
              else
                delete_cond_ids << cond.id
              end
            else
            #ホストが２つ以上あればホスト限定・削除から対象のホストを外す
              host_cond_ids << host.id
            end
          end
        end
        message = ""
        if delete_rule_ids.size > 0 then
          message += "このホストが設定されているルールが#{delete_rule_ids.size}件あり、\nホストを削除すると設定されているルールも共に削除されます。"
        end
        if delete_cond_ids.size > 0 || host_cond_ids.size > 0 then
          message += "\nまた、" if message != ""
          message += "登録されているルールに「#{cond_names['host-match']}」または「#{cond_names['host-unmatch']}」の条件が設定されている場合、\nその条件の対象ホストがこのホストのみの場合は条件が削除され、そうでない場合はその条件の対象ホストからこのホストが削除されます。"
        end
        message += "\n削除してもよろしいですか?" if message != ""
        render :layout => false, 
               :json => { 
                 :delete_rule_ids => delete_rule_ids, 
                 :delete_cond_ids => delete_cond_ids, 
                 :delete_host_cond_ids => host_cond_ids,
                 :message => message
               }
      end
    elsif page_name == "category" then #カテゴリ設定ページでカテゴリ削除時のチェック
      #ルールの「ルールを適用するカスタムカテゴリ」にカテゴリが設定されていれば、ルールを削除の対象にする
      SidfmdRule.where(:sidfmd_category_id => delete_id).each do |rule|
        delete_rule_ids << rule.id
      end
      #脆弱性同期時実行のタイミングで、対象カテゴリに削除するカテゴリが設定されていれば、そのルールを削除の対象にする
      SidfmdRuleTVulsync.where(:sidfmd_category_id => delete_id).each do |vulsync_trigger|
        rule = vulsync_trigger.sidfmd_rule_trigger.sidfmd_rule
        delete_rule_ids << rule.id unless delete_rule_ids.include?(rule.id)
      end 
      #カテゴリ限定・除外に削除するカテゴリが設定されていればそのカテゴリを外す
      category_cond_ids = []
      SidfmdRuleCmCategorySub.where(:sidfmd_category_id => delete_id).each do |category|
        sub_cond = category.sidfmd_rule_c_match_category
        cond = sub_cond.sidfmd_rule_cond
        rule = cond.sidfmd_rule
        if !delete_rule_ids.include?(rule.id) then
          #カテゴリが１つだったら条件を削除の対象にする
          if SidfmdRuleCmCategorySub.where(:sidfmd_rule_c_match_category_id => sub_cond.id).count == 1 then
            #この条件を削除すると条件がなくなるならルールを削除
            if rule.sidfmd_rule_conds.size == 1 then
              delete_rule_ids << rule.id
            else
              delete_cond_ids << cond.id
            end
          else
            #カテゴリが２つ以上あればカテゴリ限定・削除から対象のカテゴリを外す
            category_cond_ids << category.id
          end
        end
      end
      #メール送信の操作に関連するカテゴリがあれば操作を削除の対象にする
      SidfmdRuleAMailCategory.where(:sidfmd_category_id => delete_id).each do |mail_action|
        action = mail_action.sidfmd_rule_action
        rule = action.sidfmd_rule
        if !delete_rule_ids.include?(rule.id) then
          #この操作を削除すると操作がなくなるならルールを削除
          if rule.sidfmd_rule_actions.size == 1 then
            delete_rule_ids << rule.id
          else
            delete_action_ids << action.id
          end
        end
      end
      #カテゴリを削除するとカスタムカテゴリが無くなるホストを取得して、そのホストに関連するルールを削除・変更の対象にする
      target_host_ids = []
      related_host_ids = SidfmdCategoriesHost.where(:sidfmd_category_id => delete_id).map{|a| a.sidfmd_host_id}
      related_host_ids.each do |h_id|
        h_cat_ids = SidfmdCategoriesHost.where(:sidfmd_host_id => h_id).map{|c| c.sidfmd_category_id}.uniq
        h_cat_ids.delete(delete_id)
        if h_cat_ids.size == 1 then
          target_host_ids << h_id
        end
      end
      host_cond_ids = []
      if target_host_ids.size > 0 then
        r_ids = SidfmdRuleTVulsync.where(:sidfmd_host_id => target_host_ids).map{|t| t.sidfmd_rule_trigger.sidfmd_rule.id}
        delete_rule_ids.push(*r_ids)
        SidfmdRuleCmHostSub.where(:sidfmd_host_id => target_host_ids).each do |host|
          sub_cond = host.sidfmd_rule_c_match_host
          cond = sub_cond.sidfmd_rule_cond
          rule = cond.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            #このホストを削除するとホストが無くなるなら条件削除
            sub_cond_ids = SidfmdRuleCmHostSub.where(:sidfmd_rule_c_match_host_id => sub_cond.id).map{|sub_c| sub_c.id}
            if sub_cond_ids.count == 1 then
              #条件を削除すると条件が１つもなくなるならルールを削除
              if rule.sidfmd_rule_conds.size == 1 then
                delete_rule_ids << rule.id
              else
                delete_cond_ids << cond.id
              end
            else
              #ホスト限定・削除条件からホストを抜く
              host_cond_ids << host.id
              #全てのホストが条件から外れるなら条件を削除
              if (sub_cond_ids - host_cond_ids).size == 0 then
                delete_cond_ids << cond.id
                host_cond_ids = host_cond_ids - sub_cond_ids
              end
            end
          end
        end
      end
      delete_rule_ids = delete_rule_ids.uniq
      #カテゴリを無くすとカスタムカテゴリがなくなるホストがいる時のみチェック
      if target_host_ids.size > 0 then
        #条件を削除するとルールの条件がなくなるならルールを削除
        checked_rule_ids = []
        delete_cond_ids.each do |cond_id|
          rule = SidfmdRuleCond.find(cond_id).sidfmd_rule
          rule_id = rule.id
          if !checked_rule_ids.include?(rule_id) then
            checked_rule_ids << rule_id
            cond_ids = rule.sidfmd_rule_conds.map{|c| c.id}
            if (cond_ids - delete_cond_ids).size == 0 then
              delete_rule_ids << rule_id
              delete_cond_ids = delete_cond_ids - cond_ids
            end
          end
        end
      end
      
      message = ""
      if delete_rule_ids.size > 0 then
        message += "このカテゴリが設定されているルールが#{delete_rule_ids.size}件あり、\nカテゴリを削除すると設定されているルールも共に削除されます。"
      end
      if delete_cond_ids.size > 0 || category_cond_ids.size > 0 then
        message += "\nまた、" if message != ""
        message += "登録されているルールに「#{cond_names['category-match']}」または「#{cond_names['category-unmatch']}」の条件が設定されている場合、\nその条件の対象カテゴリがこのカテゴリのみの場合は条件が削除され、そうでない場合はその条件の対象カテゴリからこのカテゴリが削除されます。"
      end
      if delete_action_ids.size > 0 then
        message += "\n加えて、" if message != ""
        message += "登録されているルールの「#{action_names['send-mail-category']}」の操作の送信先にこのカテゴリが設定されている場合、その操作が削除されます。"
      end
      if target_host_ids.size > 0 then
        message += "\n尚、" if message != ""
        message += "登録されているルールに、カスタムカテゴリがこのカスタムカテゴリのみのホストが設定されている場合、「#{cond_names['host-match']}」または「#{cond_names['host-unmatch']}」の条件からそのホストが削除されます。ホストを削除することによって「#{cond_names['host-match']}」または「#{cond_names['host-unmatch']}」に設定されているホストの数が0になる場合は、その条件が削除されます。"
      end
      message += "\n削除してもよろしいですか?" if message != ""
      render :layout => false,
             :json => {
               :delete_rule_ids => delete_rule_ids,
               :delete_cond_ids => delete_cond_ids,
               :delete_action_ids => delete_action_ids,
               :delete_category_cond_ids => category_cond_ids,
               :delete_host_cond_ids => host_cond_ids,
               :message => message
             }
    elsif page_name == "user" then
      action_idx = f_params[:action_idx]
      #send_mail_user_action_ids = []
      if action_idx == "delete" then #ユーザ削除の場合
        #メール送信の操作の送信元が削除するユーザだったら、操作を削除の対象にする
        SidfmdRuleAMailCategory.where(:from_user_id => delete_id).each do |mail_action|
          action = mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          #この操作を削除すると操作が無くなるならルールを削除
          if rule.sidfmd_rule_actions.size === 1 then
            delete_rule_ids << rule.id
          else
            delete_action_ids << action.id
          end
        end
        SidfmdRuleAMailUser.where("from_user_id = ? OR user_id = ?", delete_id, delete_id).each do |mail_action|
          action = mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          if !delete_rule_ids.include?(rule.id) then
            rule_actions = rule.sidfmd_rule_actions
            rule_actions_except_mail_users = rule_actions.select{|a| a.detail.class.table_name != "sidfmd_rule_a_mail_users"}
            #メール送信の操作の送信元が削除するユーザだったら、操作を削除の対象にする
            #この操作を削除すると操作がなくなるならルールを削除
            if mail_action.from_user_id == delete_id then
              if rule_actions_except_mail_users.size == 0 ||
                (rule_actions_except_mail_users.size == 1 && delete_action_ids.include?(rule_actions_except_mail_users[0].id))
              then
                delete_rule_ids << rule.id
                if rule_actions_except_mail_users.size == 1 then
                  delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                end
              else
                delete_action_ids << action.id
              end
            else
              #送信先に削除するユーザが設定されていれば、送信先から外す
              #送信先ユーザが削除するユーザのみだったら操作削除
              rule_actions = rule.sidfmd_rule_actions
              if rule_actions.select{|a| a.detail.class.table_name == "sidfmd_rule_a_mail_users"}.size == 1 then
                #この操作を削除すると操作が無くなるならルール削除
                if rule_actions.size == 1 then
                  delete_rule_ids << rule.id
                elsif rule_actions_except_mail_users.size == 1 && delete_action_ids.include?(rule_actions_except_mail_users[0].id) then
                  delete_rule_ids << rule.id
                  delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                else
                  delete_action_ids << action.id
                end
              else
                delete_action_ids << action.id
              end
            end
          end
        end
        message = ""
        if delete_rule_ids.size > 0 then
          message += "このユーザが設定されているルールが#{delete_rule_ids.size}件あり、\nユーザを削除すると設定されているルールも共に削除されます。"
        end
        if delete_action_ids.size > 0 then
          message += "\nまた、" if message != ""
          message += "登録されているルールの「#{action_names['send-mail-category']}」または「#{action_names['send-mail-user']}」の操作の送信元にこのユーザが設定されているか、「#{action_names['send-mail-user']}」の操作の送信先ユーザがこのユーザのみの場合、その操作が削除されます。"
        end
        message += "\n削除してもよろしいですか?" if message != ""
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_action_ids => delete_action_ids,
                 :message => message
               }
      elsif action_idx == "category_change" || action_idx == "bulk_category_change" then #ユーザのカテゴリ変更の場合
        message_prefix = (action_idx == "category_change" ? "この" : "選択中の")
        user_ids = JSON.parse(f_params[:user_ids])
        category_ids = JSON.parse(f_params[:category_ids])
        is_user_deleted_form_address = false #送信先ユーザから外すユーザがいるかどうか
        #メール送信の操作の送信元に変更するユーザが設定されていて、
        #ユーザのカテゴリがそのルールの「ルールを適用するカスタムカテゴリ」のカテゴリで無くなる場合、ルールを削除
        SidfmdRuleAMailCategory.where(:from_user_id => user_ids).each do |mail_action|
          action =  mail_action.sidfmd_rule_action
          rule = action.sidfmd_rule
          rule_category = rule.sidfmd_category_id
          if (rule_category && !category_ids.include?(rule_category)) || (category_ids.empty?) then
            #この操作を削除すると操作がなくなるならルールを削除
            if rule.sidfmd_rule_actions.size == 1 then
              delete_rule_ids << rule.id
            else
              delete_action_ids << action.id
            end
          end
        end
        user_ids.each do |user_id|
          SidfmdRuleAMailUser.where("from_user_id = ? OR user_id = ?", user_id, user_id).each do |mail_action|
            action =  mail_action.sidfmd_rule_action
            rule = action.sidfmd_rule
            if !delete_rule_ids.include?(rule.id) then
              rule_actions = rule.sidfmd_rule_actions
              rule_category = rule.sidfmd_category_id
              rule_actions_except_mail_users = rule_actions.select{|a| a.detail.class.table_name != "sidfmd_rule_a_mail_users"}
              if (rule_category && !category_ids.include?(rule_category)) || (category_ids.empty?) then
                #メール送信(ユーザ)の送信元ユーザをチェック
                if mail_action.from_user_id == user_id then
                  #この操作を削除すると操作がなくなるならルールを削除
                  if rule_actions_except_mail_users.size == 0 then
                    delete_rule_ids << rule.id
                  else
                    if rule_actions_except_mail_users.size == 1 && 
                      delete_action_ids.include?(rule_actions_except_mail_users[0].id)
                    then 
                      delete_rule_ids << rule.id
                      delete_action_ids.delete(rule_actions_except_mail_users[0].id)
                    else
                      delete_action_ids << action.id
                    end
                  end
                else
                  #送信先チェック
                  #送信先ユーザがこのユーザのみなら操作削除
                  #この操作を削除すると操作がなくなるならルールを削除
                  if rule_actions_except_mail_users.size == 0 &&
                    rule_actions.size == 1 
                  then
                    delete_rule_ids << rule.id
                  elsif rule_actions.select{|a| a.detail.class.table_name == "sidfmd_rule_a_mail_users"}.size == 1 then
                    if rule_actions_except_mail_users.size == 1 &&
                      delete_action_ids.include?(rule_actions_except_mail_users[0].id) 
                    then
                      delete_rule_ids << rule.id
                    else
                      delete_action_ids << action.id
                    end
                  else
                    delete_action_ids << action.id
                    is_user_deleted_form_address = true
                  end
                end
              end
            end
          end
        end
        #カテゴリ一括変更の時にチェック。操作を削除するとルールの操作が無くなるならルールを削除
        if action_idx == "bulk_category_change" then
          checked_rule_ids = []
          SidfmdRuleAction.where(:id => delete_action_ids).each do |action|
            rule = action.sidfmd_rule
            rule_id = rule.id
            if !checked_rule_ids.include?(rule_id) then
              checked_rule_ids << rule_id
              rule_action_ids = rule.sidfmd_rule_actions
                                    .map{|a| a.id}
              if (rule_action_ids - delete_action_ids).size == 0 then
                delete_rule_ids << rule.id
                delete_action_ids = (delete_action_ids - rule_action_ids)
              end
            end
          end
          is_user_deleted_form_address = false if delete_action_ids.size == 0 
        end
        message = ""
        if delete_rule_ids.size > 0 then
            message += "#{message_prefix}ユーザが設定されているルールが#{delete_rule_ids.size}件あり、\nユーザのカテゴリを変更すると設定されているルールも共に削除されます。"
        end
        if delete_action_ids.size > 0 then
          message += "\nまた、" if message != ""
          message += "登録されているルールの「#{action_names['send-mail-category']}」または「#{action_names['send-mail-user']}」の操作の送信元に#{message_prefix}ユーザが設定されているか、「#{action_names['send-mail-user']}」の操作の送信先ユーザが#{message_prefix}ユーザのみの場合、\nそのルールの「ルールを適用するカスタムカテゴリ」が変更するカテゴリに含まれていなければ、その操作が削除されます。"
        end
        if is_user_deleted_form_address then
          message += "\n加えて、" if message != ""
          message += "登録されているルールの「#{action_names['send-mail-user']}」の操作の送信先ユーザが複数で且つ、「ルールを適用するカスタムカテゴリ」が変更するカテゴリに含まれていない場合、このユーザを送信先ユーザから削除します。"
        end
        message += "\n削除してもよろしいですか?" if message != ""
        render :layout => false,
               :json => {
                 :delete_rule_ids => delete_rule_ids,
                 :delete_action_ids => delete_action_ids,
                 :message => message
               }
      end 
    end
  end

  def api_get_rule_selectable_users 
    f_params = params.permit(:category_id)
    category_id = f_params[:category_id]
    users_info = []
    admin = @sidfm_roles.include?(:admin) 
    if admin then
      if category_id == "all" then
        userids = SidfmdCategoriesUser.all.map(&:user_id)
      else
        userids = SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).map(&:user_id)
      end
    else
      if category_id == "all" || @user_category_ids.include?(category_id.to_i) then
        userids = []
      end
      userids = SidfmdCategoriesUser.where(:sidfmd_category_id => category_id).map(&:user_id)
    end
    users = User.where(:id => userids).order(:login => :asc)
    users.each do |user|
      users_info.push({
          :user_id => user.id,
          :login => user.login,
          :lastname => user.lastname,
          :firstname => user.firstname,
          :address => user.mail,
          :categories => if admin then
                           user.sidfmd_categories.map{|c| c.name}.sort
                         else
                           user.sidfmd_categories.select{|c| @user_category_ids.include?(c.id)}.map{|c| c.name}.sort
                         end
      })
    end
    render :layout => false, :json => users_info
  end

end
