#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHelper

  RISKS = [ 'Critical', 'High', 'Medium', 'Low' ].map(&:freeze).freeze
  TARGETS = [ 'リモート', 'ローカル', 'クライアント' ].map(&:freeze).freeze
  FORM_PARAMS = 'queries'.freeze

  ### FIXME: RedmineのIssueに合わせて要修正
  STATUSES = [ '未読', '影響有り', '保留', '影響無し', '対策済み' ].map(&:freeze).freeze

  ### FIXME: SIDfmのデータに合わせて要修正
  CATEGORIES = [ '公開サーバ', '内部サーバ', 'モバイル', 'クライアント', '共用サーバ' ].map(&:freeze).freeze


  # 危険度を数値から表示用の文字列に変換
  def risk_to_string(v)
    number_to_string(RISKS, v)
  end

  # 対象を数値から表示用の文字列に変換
  def target_to_string(v)
    number_to_string(TARGETS, v)
  end

  # ステータスを数値から表示用の文字列に変換
  def status_to_string(v)
    number_to_string(STATUSES, v)
  end

  # カテゴリを数値から表示用の文字列に変換
  def category_to_string(v)
    number_to_string(CATEGORIES, v)
  end

  # 表の列名のタイトル出力
  #   sortnameとsort,orderでソート状態とリンクへのソートパラメータ挿入を行う。
  def table_title(name, sortname, sort, order, target = {}, options = {}, sort_param_name: 'sort', order_param_name: 'order')
    # ソートオプションのチェックと設定
    sort = '' unless sort
    if sort == sortname then
      # sortが自分のsortnameと一致している場合
      target[sort_param_name] = sortname
      target[order_param_name] = order == 'asc' ? 'desc' : 'asc'
      options[:class] = '' unless options.has_key?(:class)
      options[:class] << ' ' if options[:class].size > 0 
      options[:class] << 'sort ' << order
    else
      # その他全ての場合
      target[sort_param_name] = sortname
      target[order_param_name] = 'asc'
    end
    content_tag(:th, title: "並べ替え \"#{name}\"") do
      concat link_to name, target, options
    end
  end

  # 現在のURLにパラメータを追加したURLを返す。
  def url_for_with_new_param(k, v)
    url_for(params.merge(k => v))
  end

  # 現在のURLからパラメータを削除したURLを返す。
  def url_for_without_param(k)
    url_for(params.merge(k => nil))
  end

  # ブロックの返り値が"true"ならtrue、それ以外は全てdefault_valueで返す。
  # ブロックの処理で例外が出た場合はrescue_valueを返す。
  # viewでのパラメータ値の判定用。エラーにならずに値を判定できる。
  def query_true?(default_value = false, rescue_value = false)
    raise ArgumentError, 'must have a block' unless block_given?
    ret = default_value
    begin
      ret = ("true" == yield)
    rescue Exception
      ret = rescue_value
    end
    ret
  end

  # ブロックの返り値がnilでなければその値を文字列にして返す。
  # それ以外は全て(例外含む)defaut_valueで返す。
  # viewでのパラメータ値の取得用。エラーにならずに値を取得できる。
  def query_value(default_value = '')
    raise ArgumentError, 'must have a block' unless block_given?
    ret = default_value
    begin
      ret = yield
    rescue Exception
    end
    ret
  end

  # buttonと対応するhiddenを同時に生成する
  #  idはbuttonは{id}_btn, hiddenは{id}になる。
  def button_hidden_tag(id, btn_name, hdn_name, hdn_value, btn_class: '')
    concat(button_tag(btn_name, :id => id + '_btn', :class => btn_class))
    concat(hidden_field_tag(hdn_name, hdn_value, :id => id))
  end

  # SIDfmDigitalのmenuを出力する
  def sid_main_menu(current_menu_item)
    concat(content_tag(:div, render_menu(:sidfm_menu), :id => 'sidfm-menu', :class => 'main-menu tabs'))
    concat(content_tag(:div, render_menu(:sidfm_menu), :id => 'sidfm-menu-mobile', :class => 'flyout-menu'))
    concat(hidden_field_tag('current_menu_item', current_menu_item.to_s.tr('_', '-'), :id => 'current_menu_item'))
    # 選択されているメニューを選択状態にする処理と、PC/モバイルでメニューの有無を切り換える処理を出力する。
    concat(javascript_tag('$(function(){$("#sidfm-menu ."+$("#current_menu_item").val()).addClass("selected");(function(){var sidfm_menu_resize={timer:false,action:function(){if(this.timer!==false){clearTimeout(this.timer);};this.timer=setTimeout(function(){if(isMobile()){$("#sidfm-menu").css("display", "none");}else{$("#sidfm-menu").css("display", "block");}},200);}};$(window).resize(sidfm_menu_resize.action);})();});', :type => 'text/javascript'));
  end

  # 左向き三角を出力する。
  # sizeの単位はpt。
  def right_triangle(size = 10)
    content_tag(:svg, width: "#{size}px", height: "#{size}px") do
      content_tag(:path, nil, d: "M0 0 L#{size} #{size/2} L0 #{size} Z")
    end
  end

  # FIXME: under construction.
  def sidfm_menu(opts = {})
    concat(content_tag(:div, :id => 'sidfm-menu') do
             content_tag(:ul, :class => 'top') do
               concat(content_tag(:li, link_to('ダッシュボード', :controller => :sidfm, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('ダッシュボード', :controller => :sidfm, :action => :index)))
                        concat(content_tag(:li, link_to('レイアウト・表示内容変更', '#')))
                      end)
               concat(content_tag(:li, link_to('ホスト情報', :controller => :sidfm_hosts, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('対応が必要なホスト', '#')))
                        concat(content_tag(:li, link_to('#') do
                                             concat('カテゴリ')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('ホスト')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('統計・一覧', '#')))
                        concat(content_tag(:li, link_to('カテゴリ管理', '#')))
                      end)
               concat(content_tag(:li, link_to('脆弱性情報', :controller => :sidfm_vulnerabilities, :action => :index)))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('対応が必要な脆弱性', '#')))
                        concat(content_tag(:li, link_to('対応が必要な脆弱性 条件設定', '#')))
                        concat(content_tag(:li, link_to('#') do
                                             concat('指標')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('検出時期')
                                             concat(right_triangle)
                                           end))
                        concat(content_tag(:li, link_to('#') do
                                             concat('脆弱性詳細')
                                             concat(right_triangle)
                                           end))
                      end)
               concat(content_tag(:li, link_to('SIDfm脆弱性情報', '#')))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('セキュリティホール情報')))
                        concat(content_tag(:li, link_to('アドバイザリ情報')))
                        concat(content_tag(:li, link_to('リリース・EOL情報')))
                        concat(content_tag(:li, link_to('セキュリティニュース情報')))
                      end)
               concat(content_tag(:li, link_to('test(development)', '#')))
               concat(content_tag(:ul, :class => 'layer-2') do
                        concat(content_tag(:li, link_to('settings', :controller => :sidfm_settings, :action => :index)))
                      end)
             end
           end)
  end

  # FIXME
  #   ここより下が新しいシステム用。上は削除予定。

  COLOR = {
    # Bootstrapより
    :success => '#5cb85c'.freeze,
    :info => '#5bc0de'.freeze,
    :warning => '#f0ad4e'.freeze,
    :danger => '#d9534f'.freeze,


    :sri_none => 'hsl(200, 80%, 48%)'.freeze,
    :sri_low => 'hsl( 55, 80%, 48%)'.freeze, 
    :sri_medium => 'hsl( 30, 80%, 48%)'.freeze,
    :sri_high => 'hsl(  5, 80%, 48%)'.freeze,
    :sri_critical => 'hsl(340, 65%, 48%)'.freeze,
    :sri_fixed => '#4dcc33'.freeze,

    :cvss_notreg => '#f7d6b6'.freeze,
    :cvss_d4 => '#ca2b60'.freeze,
    :cvss_d3 => '#dc2918'.freeze,
    :cvss_d2 => '#dc7a18'.freeze,
    :cvss_d1 => '#dccc18'.freeze,
    :cvss_fixed => '#4dcc33'.freeze,

    :risk_low => '#c5c958'.freeze,
    :risk_medium => '#f8c036'.freeze,
    :risk_high => '#fa2121'.freeze,
    :risk_critical => '#c42161'.freeze,
    :risk_fixed => '#4dcc33'.freeze,

    :status_notaffect => '#3399cc'.freeze,
    :status_fixed => '#4dcc33'.freeze,
    :status_ongoing => '#e6c319'.freeze,
    :status_notconfirmed => '#e62a19'.freeze,

    :status_gradient_start => '#d9534f'.freeze,
    :status_gradient_end => '#5cb85c'.freeze,
  }.freeze

  # viewからCOLORにアクセスするために使用する。
  def colors
    COLOR
  end

  # colorの値が :success のようなシンボルだった場合、そのシンボルに
  # 対応する色を示す文字列を返す。
  # それ以外はそのま（シンボルは文字列化）ま返す。
  def symbol_color(color)
    COLOR[color] || color.to_s
  end

  # status barを生成する。width, heightから大きさを自動調節する。
  # width: px
  # height: px
  # data = [
  #  { :value => 値, :name => '名称', :color => '#aaa', :placement => 'top' },
  #  { :value => 値, :name => '名称', :color => 'blue', :placement => 'bottom' }, ...
  # ]
  #  省略可能(:placement(bottom))
  # opts = { :id, ... }
  def status_bar(width, height, data, opts = {})
    unit = opts[:unit] || ''
    opts.delete(:unit)
    content_tag(:div, :class => 'text-nowrap') do
      concat(content_tag(:svg,
                         opts.merge(:width => "#{width}px",
                                    :height => "#{height}px",
                                    :xmlns => 'http://www.w3.org/2000/svg',
                                    :version => '1.1')) do
               #data_size = data.size
               value_sum = data.inject(0) {|r,x| r += x[:value] }
               return if value_sum == 0
               unit_width = width.to_f / value_sum.to_f
               current_x = 0
               data.each_with_index do |item, i|
                 w = (unit_width * item[:value].to_f).to_i
                 concat(content_tag(:a,
                                    :'data-toggle' => 'tooltip',
                                    :'data-container'=> 'body',
                                    :'data-placement' => item[:placement] || 'bottom', 
                                    :'data-html' => 'true',
                                    :title => "<svg xmlns='http://www.w3.org/2000/svg' version='1.1' width='12px' height='12px'><rect x='0' y='0' width='10' height='12' fill='#{symbol_color(item[:color])}' stroke='#{symbol_color(item[:stroke])}' /></svg>#{item[:name]}:#{item[:value]}#{unit}")  do
                          rect_opt = {
                            :x => "#{current_x}",
                            :y => '0',
                            :width => "#{w}",
                            :height => "#{height}",
                            :'fill-opacity' => "1.0",
                            :fill => "#{symbol_color(item[:color] || 'white')}",
                          }
                          rect_opt[:stroke] = "#{symbol_color(item[:stroke])}" if item[:stroke]
                          concat(content_tag(:rect, '', rect_opt))
                        end)
                 current_x += w
               end
             end)
      concat(content_tag(:div, :class => 'row statusbar') do
               data.each do |i|
                 concat(content_tag(:div, :class => 'col-6') do
                          concat(content_tag(:span, '■', :style => "color:#{COLOR[i[:color]]};padding-right:2px;"))
                          concat(i[:name])
                        end)
                 concat(content_tag(:div, i[:value], :class => 'col-6 num_align'))
               end
             end)
    end
  end

  # パラメータnoticeに値が設定されていれば、アラートとして出力する。
  def notice_output(params, level: :warning, dismissible: true)
    if params[:notice] then
      concat(content_tag(:div, :class => 'row') do
               k = [ 'alert ']
               case level
               when :success then k << 'alert-success'
               when :info    then k << 'alert-info'
               when :warning then k << 'alert-warning'
               when :danger  then k << 'alert-danger'
               else               k << 'alert-info'
               end
               k << 'alert-dismissible' if dismissible
               content_tag(:div, :class => k.join(' ')) do
                 if dismissible then
                   concat(content_tag(:button, :type => 'button', :class => 'close', :'data-dismiss' => 'alert', :role => 'alert') do
                            content_tag(:span, '&times;'.html_safe, :'aria-hidden' => 'true')
                          end)
                 end
                 concat(h(params[:notice]))
               end
             end)
    end
  end

  # 各種アラートを出力する。
  # アラートの内容は、コントローラで@alertsに設定しておく。
  #   :no_categorized_hosts - カテゴリを設定していないホストが存在。
  def alerts_output
    if @alerts then
      if @alerts[:no_categorized_hosts] then
        concat(content_tag(:div, :class => 'alert alert-info mb-0 s-shadow-v6 font-weight-bold text-gray-555', :role => 'alert') do
                 concat(content_tag(:span, '', :class => 'fas fa-exclamation-circle g-font-size-1_14x text-info', :'aria-hidden' => 'true'))
                 concat("\n" + 'カテゴリを設定していないホストが存在しています。カテゴリを設定していない場合、対象ホストの情報が出力されないため、少なくとも「システムカテゴリ」の設定は行ってください。')
                 if controller.controller_name != 'sidfm_hostcategories' or controller.action_name != 'index' then
                   concat(link_to('ホストカテゴリ設定', { :controller => :sidfm_hostcategories, :action => :index}, {:class => 'btn btn-default btn-xs', :role => 'button' }))
                 end
                 
               end)
      end
    end
  end

  # alerts_output用のデータを返す。controllerではこの値を@alertに設定しておく。
  # FIXME: 本当はconcern辺りに入れるべき処理だが、alerts_outputと近い場所に
  #   あった方がよいという判断でここに設置。controllerからview_context経由で
  #   アクセスする。
  def prepare_alerts
    ret = {}
    ret[:no_categorized_hosts] = SidfmdHost.enable_hosts.not_categorized.count > 0
    ret
  end

  # paginator
  def paginator2(base: 'pages', content: nil, method: :get, anchor: 'SummaryTable')
    raise ArgumentError, 'need "content"' unless content
    concat(content_tag(:div, :class => 'mr-2') do
             concat(content_tag(:label, '表示行数:', :class => 'g-font-size-13'))
             concat(content_tag(:select, :class => base + '_change_per_page g-font-size-14', :style => 'border:1px solid #bbb;') do
                      [ 10, 25, 50, 100, 200 ].each do |i|
                        if content[:per_page] == i then
                          concat(content_tag(:option, i, :selected => 'selected'))
                        else
                          concat(content_tag(:option, i))
                        end
                      end
                    end)
           end)

    c_start = content[:offset] + 1 
    f_start_edge = (content[:offset] == 0)
    c_end = content[:offset] + content[:per_page]
    c_end = content[:total_size] if c_end > content[:total_size]
    f_end_edge = (content[:total_size] == c_end)
    offset_prev = content[:offset] - content[:per_page]
    offset_prev = 0 if offset_prev < 0
    offset_next = content[:offset] + content[:per_page]
    offset_next = content[:total_size] - 1 if content[:total_size] <= offset_next

    concat(content_tag(:div, :class => 'mr-2') do
             concat(content_tag(:label, '移動:', :class => 'g-font-size-13'))
             concat(text_field_tag('dummy', c_start, :maxlength => '5', :class => base + '_change_offset_direct g-font-size-14 pagination-input-box', :autocomplete => 'off'))
           end)

    concat(content_tag(:div, :class => 'mx-1') do
             concat(content_tag(:span, "#{c_start}-#{c_end}/#{content[:total_size]}", :class => 'g-font-size-14'))
           end)
    concat(content_tag(:nav, :class => 'ml-1', :'aria-label' => 'Page navigation') do
             concat(content_tag(:ul, :class => 'pagination pagination-sm') do
                      concat(content_tag(:li, :class => 'page-item' + ( f_start_edge ? ' disabled' : '')) do
                               concat(link_to('#', :class => base + '_change_offset_paging page-link', :data => { :offset => offset_prev.to_s }) do
                                        concat(content_tag(:span, :'aria-hidden' => 'true') do
                                                 concat(content_tag(:i, '', :class => 'fa fa-angle-double-left'))
                                               end)
                                      end)
                             end)
                      concat(content_tag(:li, :class => 'page-item' + (f_end_edge ? ' disabled' : '')) do
                               concat(link_to('#', :class => base + '_change_offset_paging page-link', :data => { :offset => offset_next.to_s }, :'aria-label' => '次のページへ') do
                                        concat(content_tag(:span, :'aria-hidden' => 'true') do
                                                 concat(content_tag(:i, '', :class => 'fa fa-angle-double-right'))
                                               end)
                                      end)
                             end)
                    end)
           end)
    target_action = controller.action_name
    if 'index_r' == target_action then
      target_action = 'index'
    end
    concat(form_tag(url_for(:controller => controller.controller_name, :action => target_action, :anchor => anchor), :method => method, :id => base + '_form') do
             concat(hidden_field_tag base + '[offset]', content[:offset])
             concat(hidden_field_tag base + '[per_page]', content[:per_page])
             concat(yield base, content) if block_given?
           end)
  end

  # 例外を発生しないJSON.parse
  # 例外になるような不正な文字列を入力した場合、nilを返す。
  def force_json_parse(s)
    ret = nil
    begin
      ret = JSON.parse(s)
    rescue Exception => e
      logger.error(e.message)
      if e.backtrace then
        e.backtrace.each do |i|
          logger.error(i)
        end
      end
    end
    ret
  end

  # テーブルの列ソートの状態表示
  # 使用しなくなったため、削除予定。
  def table_sort_th(order: nil, title: nil, target: nil, type: 'attribute')
    content_tag(:th,
                :class => 'sort',
                :data => {
                  :target => target,
                  :order => order,
                  :type => type,
                  :status => (case order when target + '_asc' then 'asc' when target + '_desc' then 'desc' else 'asc' end)}) do
      h(title) +
        content_tag(:span, '',
                    :class => 'glyphicon glyphicon-sort' + (case order when target + '_asc' then '-by-' + type when target + '_desc' then '-by-' + type + '-alt' else '' end),
                    :aria => { :hidden => 'true' })
    end
  end


  # 数値を配列の添字と見なして文字列に変換する。
  # zero=true にすると、添字＝パラメータv
  # zero=false(default) にすると、添字-1＝パラメータv
  def number_to_string(ary, v, zero = false)
    vi = v.to_i
    (vi >= (zero ? 0 : 1) and vi <= (zero ? ary.size - 1 : ary.size)) ? ary[vi - (zero ? 0 : 1)] : '---'
  end

  SRIS = [ '無', '低', '中', '高', '重大' ].map(&:freeze).freeze
  # COLORSは削除予定
  SRI_COLORS = [ 'danger', 'warning', 'notice', 'info' ].map(&:freeze).freeze

  def display_percent(v)
    v ? "#{sprintf('%.1f', v)}%" : '---'
  end

  def display_sri(v)
    return unless v
    content_tag(:span, :class => "bgc_risk_d#{v} d-block mx-auto sri-btn text-white b_radius") do
      number_to_string(SRIS, v + 1)
    end
  end

  # --- の時に、condの条件でn/aに値を差し替える。
  def na_check(v, cond, str = '---', l = 'n/a')
    if v then
      if str == v and cond then
        l
      else
        v
      end
    else
      v
    end
  end

  # CVSS値の出力。小数点第一位まで必ず出力。
  def cvss(v)
    sprintf('%.1f', (v ? v : 0).to_f)
  end

  RISK_COLORS = [ 'info', 'notice', 'warning', 'danger' ].map(&:freeze).freeze

  def display_risk(v)
    if v then
      content_tag(:span, :class => 'text-' + number_to_string(RISK_COLORS, v.risk_id)) do
        v.name_en
      end
    end
  end

  AREA_COLORS = [ 'notice', 'info', 'warning' ].map(&:freeze).freeze

  def display_area(v)
    if v then
      content_tag(:span, :class => 'text-' + number_to_string(AREA_COLORS, v.area_id)) do
        v.name
      end
    end
  end

  # カラーマップをJavaScriptから参照できる形で出力する。
  def color_map
    COLOR.keys.each do |k|
      concat hidden_field_tag "color[#{k}]", symbol_color(k), :id => "color_#{k}", :disabled => 'disabled'
    end
  end

  # 指標のパラメータ値を表示用の文字列に変換
  def idx2str(idx)
    selector = @hosts ? @hosts : @vuls
    case selector[:idx]
    when :sri then 'SRI'
    when :cvss then 'CVSS'
    when :softek then 'SofTek'
    end
  end

  # 各ページに出力する本日日付
  def show_today
    Time.current.localtime.strftime('%Y/%m/%d')
  end

  # 「対応が必要なホスト数」等で使用する円グラフの描画
  ### FIXME: percentによる制御はまだ未対応
  def circle_graph(percent, opts = {})
    cx = 0.0
    cy = 0.0
    r = 85.0
    content_tag(:svg, opts.update(:width => '150px', :height => '150px',
                :viewBox => '-105 -105 210 210',
                :xmlns => 'http://www.w3.org/2000/svg',
                :version => '1.1')) do
      if percent != 0 and percent != 100 then
        d = 360.0 * percent.to_f / 100.0
        start = -90.0
        
        s = -90.0
        e = -90.0 + d

        x1 = cx + r * Math.cos(s * Math::PI / 180.0)
        y1 = cy + r * Math.sin(s * Math::PI / 180.0)
        x2 = cx + r * Math.cos(e * Math::PI / 180.0)
        y2 = cy + r * Math.sin(e * Math::PI / 180.0)
        dx = x2 - x1
        dy = y2 - y1

        f1 = d > 180.0 ? 1 : 0
        f2 = 1

        concat(content_tag(:path, '',
                           :d => "M #{x1.to_i},#{y1.to_i} a #{r.to_i} #{r.to_i} #{start.to_i} #{f1} #{f2} #{dx.to_i},#{dy.to_i}",
                           :'fill-opacity' => '0',
                           :stroke => '#c12e2a',
                           :'stroke-width' => '30'))

        start = -90.0 + d
        s = -90.0 + d
        e = 270.0

        x1 = cx + r * Math.cos(s * Math::PI / 180.0)
        y1 = cy + r * Math.sin(s * Math::PI / 180.0)
        x2 = cx + r * Math.cos(e * Math::PI / 180.0)
        y2 = cy + r * Math.sin(e * Math::PI / 180.0)
        dx = x2 - x1
        dy = y2 - y1

        f1 = d > 180.0 ? 0 : 1
        f2 = 1

        concat(content_tag(:path, '',
                           :d => "M #{x1.to_i},#{y1.to_i} a #{r.to_i} #{r.to_i} #{start.to_i} #{f1} #{f2} #{dx.to_i},#{dy.to_i}",
                           :'fill-opacity' => '0',
                           :stroke => '#e6e6e6',
                           :'stroke-width' => '30'))
      else
        # 100%と0%は円弧では描画できないので、circleで描画するよう
        # 特別扱いする。
        if percent == 100 then
          concat(content_tag(:circle, '',
                             :cx => cx.to_i.to_s,
                             :cy => cy.to_i.to_s,
                             :r => r.to_i.to_s,
                             :'fill-opacity' => '0',
                             :stroke => '#c12e2a',
                             :'stroke-width' => '30'))
        elsif percent == 0 then
          concat(content_tag(:circle, '',
                             :cx => cx.to_i.to_s,
                             :cy => cy.to_i.to_s,
                             :r => r.to_i.to_s,
                             :'fill-opacity' => '0',
                             :stroke => '#e6e6e6',
                             :'stroke-width' => '30'))
        end
      end
      concat(content_tag(:text, percent.to_s, :y => '20', :x => '-10',
                         :'text-anchor' => 'middle',
                         :'font-weight' => 'bold',
                         :'font-size' => '56'))
      concat(content_tag(:text, '%', :y => '20', :x => '50',
                         :'text-anchor' => 'middle',
                         :'font-size' => '30'))
    end
  end

  # bがtrueの場合' selected'、falseの場合''を返す。
  def selected(b)
    b ? ' selected' : ''
  end

  # 対策予定日設定済表示用
  def display_due_dates_percent(due_dates)
    return '---' unless due_dates
    f_cnt = due_dates.size
    return display_percent(0.0) if f_cnt == 0
    return display_percent(0.0) if due_dates.compact.size == 0
    cnt = due_dates.inject(0) {|r, x| r += 1 if (x and x.size > 0); r }
    display_percent(cnt.to_f / f_cnt.to_f * 100.0)
  end

  def display_due_dates_percent_value(due_dates)
    return nil unless due_dates
    f_cnt = due_dates.size
    return nil if f_cnt == 0
    return 0.0 if due_dates.compact.size == 0
    cnt = due_dates.inject(0) {|r, x| r += 1 if (x and x.size > 0); r }
    cnt.to_f / f_cnt.to_f * 100.0
  end

  # 対応状況表示用
  def display_finished_percent(status_ids)
    return '---' unless status_ids
    f_cnt = status_ids.size
    cnt = status_ids.inject(0) {|r, x|
      r += 1 if SidfmdIssue::STATUS_FINISHED.include?(x)
      r
    }
    display_percent(cnt.to_f / f_cnt.to_f * 100.0)
  end

  def display_finished_percent_value(status_ids)
    return nil unless status_ids
    f_cnt = status_ids.size
    cnt = status_ids.inject(0) {|r, x|
      r += 1 if SidfmdIssue::STATUS_FINISHED.include?(x)
      r
    }
    cnt.to_f / f_cnt.to_f * 100.0
  end

  # ソートアイコンの表示
  def sort_icon(name, value)
    case value
    when name + '_asc' then ' sort-up'
    when name + '_desc' then ' sort-down'
    else ''
    end
  end

  # ユーザの名前表示
  def display_user_name(user)
    user ? "#{user.lastname} #{user.firstname}" : '（存在しないユーザ）'
  end

end
