#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class SidfmdIssue < ActiveRecord::Base
  belongs_to :issue
  belongs_to :sidfmd_host

  has_many :sidfmd_issue_cves, :dependent => :destroy
  has_many :sidfmd_issue_sris, :dependent => :destroy
  has_many :sidfmd_issue_affprods, :dependent => :destroy
  has_many :sidfmd_issue_affvers, :dependent => :destroy
  has_many :sidfmd_issue_affpkgs, :dependent => :destroy

  scope :enable_hosts, -> { where(:sidfmd_host_id =>  SidfmdHost.enable_hosts.select(:id)) }
  scope :categorized, -> { where(:sidfmd_host_id => SidfmdCategoriesHost.select(:sidfmd_host_id).distinct(:sidfmd_host_id)) }

  scope :full_content, -> { except(:offset, :limit) }
  scope :total_size, -> { except(:order, :offset, :limit).length }

  scope :order_param, ->(p) {
    case p
    #when 'id_asc' then order(:id => :asc)
    #when 'id_desc' then order(:id => :desc)
    when 'title_asc' then order(:title => :asc)
    when 'title_desc' then order(:title => :desc)
    #when 'created_on_asc' then order('issues.created_on ASC').order(:id => :asc)
    #when 'created_on_desc' then order('issues.created_on DESC').order(:id => :asc)
    else order(:title => :asc)
    end
  }

  # 管理ユーザ
  ADMIN_USER_ID = begin
                 ret = User.find_by_login('sidfmadm')
                 ret = User.find_by_login('admin') unless ret
                 ret ? ret[:id] : nil
               end

  # 未確認
  STATUS_NOTCONFIRMED = 1
  # 対応中
  STATUS_ONGOING = 2
  # 対策済
  STATUS_FIXED = 3
  # 影響無
  STATUS_NOTAFFECT = 4

  STATUS_ALL = [ STATUS_NOTCONFIRMED,
                 STATUS_ONGOING,
                 STATUS_FIXED,
                 STATUS_NOTAFFECT ].freeze
  STATUS_UNFINISHED = [ STATUS_NOTCONFIRMED, STATUS_ONGOING ].freeze
  STATUS_FINISHED = [ STATUS_FIXED, STATUS_NOTAFFECT ].freeze
  SORT_PRIORITIES = {
    SidfmdIssue::STATUS_NOTCONFIRMED => 200,
    SidfmdIssue::STATUS_ONGOING => 100,
    SidfmdIssue::STATUS_FIXED => 300,
    SidfmdIssue::STATUS_NOTAFFECT => 400,
  }.freeze

  # Status Name Table
  STATUS_NAMES =
    [ '(不正)',
      IssueStatus.find(1).name.to_s,
      IssueStatus.find(2).name.to_s,
      IssueStatus.find(3).name.to_s,
      IssueStatus.find(4).name.to_s ].map(&:freeze).freeze
  STATUS_LABELS =
    [ nil, :notconfirmed, :ongoing, :fixed, :notaffect ].freeze

  def self.label2value(label)
    idx = STATUS_LABELS.index(label)
    (idx and idx > 0) ? idx : 1
  end

  # status_idの配列から、{ <status_id> => { :id => <status_id>, :name => <name>, :count => <count> }, ... }
  # のようにstatus_id毎に集計する。
  def self.count_status_ids(status_ids)
    ret = {}
    status_ids.sort.each do |status_id|
      if ret.has_key?(status_id) then
        ret[status_id][:value] += 1
      else
        ret[status_id] = {
          :id => status_id,
          :name => STATUS_NAMES[status_id],
          :value => 1,
          :color => ('status_' + STATUS_LABELS[status_id].to_s).to_sym,
        }
      end
    end
    ret.values
  end

  # status_idから対応状況名称に変換
  def self.status_id_to_s(i_status_id)
    ret = '(不正)'
    if STATUS_ALL.include?(i_status_id) then
      ret = IssueStatus.find(i_status_id).name
    end
    ret
  end

  # due_dateから対応日付に変換
  def self.due_date_to_s(i_due_date)
    if i_due_date.respond_to?(:strftime) then
      i_due_date = i_due_date.strftime('%Y/%m/%d')
    end
    i_due_date ? i_due_date : '未設定'
  end

  def get_sri
    sc = sidfmd_host.system_category
    sidfmd_issue_sris.where(:category_id => sc.id).first
  end

  def self.risk_host_count_sri
    ### FIXME ###
  end

  def self.risk_host_count_cvss
    ### FIXME ###
  end

  def self.risk_host_count_softek
    cnt = enable_hosts.group(:sidfmd_host_id).maximum(:risk_id)
    ret = {}
    cnt.values.each do |k|
      if ret.has_key?(k) then
        ret[k] += 1
      else
        ret[k] = 1
      end
    end
    ret
  end

  # issueの状態をカウントして返す。
  # { status_id => issueの件数, ... }の形となる。
  def self.status_count_htmlid(htmlid)
    status2color = [ '', :danger, :warning, :info, :success ]

    cnt = SidfmdIssue.enable_hosts.joins(:issue).where(:html_id => htmlid).group(:status_id).order('issues.status_id').count
    ret = []
    cnt.each_pair do |k, v|
      ret << {
        :id => htmlid,
        :value => v,
        :name => STATUS_NAMES[k],
        :color => ('status_' + STATUS_LABELS[k].to_s).to_sym,
        :placement => 'top' }
    end
    ret
  end

  def self.status_count
    hs_count = enable_hosts.joins(:issue).group(:sidfmd_host_id, :status_id).count
    hs = {}
    hs_count.keys.each do |k|
      hostid = k[0]
      statusid = k[1]
      if hs.has_key?(k[0]) then
        hs[k[0]] = k[1] if hs[k[0]] > k[1]
      else
        hs[k[0]] = k[1]
      end
    end
    sh_cnt = {}
    hs.each do |k, v|
      if sh_cnt.has_key?(v) then
        sh_cnt[v] += 1
      else
        sh_cnt[v] = 1
      end
    end
    sh_cnt
  end

  # 登録済で、かつ、未確認・確認中の対策予定日を返す。
  # 対策予定日=nilは未設定。
  def self.get_due_dates
    self.enable_hosts.joins(:issue).where(:issues => { :status_id => STATUS_UNFINISHED }).select('issues.due_date').distinct('issues.due_date').order('due_date DESC')
  end

  def self.get_max_sri(htmlid)
    ### FIXME ###
    1
  end

  def self.get_max_cvss(htmlid)
    ### FIXME ###
    10.0
  end

  # 対応が必要な脆弱性かどうかを返す
  # sidfmd_issue自体をチェックするのではなく、htmlidに対して検証する
  # ため、クラスメソッドになっていることに注意。
  def self.corresp_required_htmlid?(htmlid, index)
    case index
    when :sri then
      corresp_required_htmlid_sri?(htmlid)
    when :cvss then
      corresp_required_htmlid_cvss?(htmlid)
    when :softek then
      corresp_required_htmlid_softek?(htmlid)
    else
      false
    end
  end


  # 対応が必要な脆弱性かどうかを返す(基準:SRI)。
  def self.corresp_required_htmlid_sri?(htmlid)
    ret = false

    cr_sri = Sidfm.corresp_required_sri

    issues = SidfmdIssue.enable_hosts.joins(:sidfmd_issue_sris).where(:html_id => htmlid).where('sidfmd_issue_sris.risk_id <= ?', cr_sri)
    if issues.size > 0 then
      issues.each do |issue|
        if issue.get_sri.risk_id <= cr_sri then
          ret = true
          break
        end
      end
    end

    ret
  end

  # 対応が必要な脆弱性かどうかを返す(基準:CVSS)。
  def self.corresp_required_htmlid_cvss?(htmlid)
    cr_cvss = Sidfm.corresp_required_cvss

    issues = SidfmdIssue.enable_hosts.joins(:sidfmd_issue_cves).where(:html_id => htmlid).where('sidfmd_issue_cves.c3_base_score > ? OR sidfmd_issue_cves.c2_base_score > ?', cr_cvss, cr_cvss)
    issues ? issues.count > 0 : false
  end

  # 対応が必要な脆弱性かどうかを返す(基準:Softek)。
  def self.corresp_required_htmlid_softek?(htmlid)
    cr_risk = Sidfm.corresp_required_risk

    issues = SidfmdIssue.enable_hosts.where('risk_id >= ?', cr_risk)
    issues ? issues.count > 0 : false
  end

  def has_exploitable?
    SidfmdIssueCve.joins(:sidfmd_issue).where(:sidfmd_issues => { :sidfmd_host_id => sidfmd_host_id }).where('c2_exploitable IS TRUE OR c3_exploitable IS TRUE').count > 0
  end

  # SidfmdHost#due_date_scheduled_percentのhtmlid版
  # 対策予定日(due_date)設定済の脆弱性の件数の割合
  # STATUS_FINISHEDはカウントから除外。
  # STATUS_UNFINISHEDかつ issues.due_dateがnil以外を設定済として扱う。
  def self.due_date_scheduled_percent_htmlid(htmlid)
    s = SidfmdIssue.joins(:issue).where(:html_id => htmlid).where(:issues => { :status_id => STATUS_UNFINISHED })
    cnt_all = s.count
    cnt_due_date = s.where.not(:issues => { :due_date => nil }).count
    cnt_all > 0 ? cnt_due_date.to_f / cnt_all.to_f : nil
  end

  # SidfmdHost#finished_percentのhtmlid版
  # 対応状況(%)を返す。
  # STATUS_FINISHEDを対応済としてカウント。
  def self.finished_percent_htmlid(htmlid)
    s = SidfmdIssue.joins(:issue).where(:html_id => htmlid)
    cnt_all = s.count
    cnt_finished = s.where(:issues => { :status_id => STATUS_FINISHED }).count
    cnt_all > 0 ? cnt_finished.to_f / cnt_all.to_f : nil
  end

  # 対応状況のデータを返す。
  # 基本的にはIssueStatusの情報にlabelというフィールドを追加したものになる。
  def self.statuses
    ret = JSON.parse(IssueStatus.all.order(:id).to_json)
    ret.each do |i|
      case i['id']
      when STATUS_NOTCONFIRMED then i['label'] = :notconfirmed
      when STATUS_ONGOING then i['label'] = :ongoing
      when STATUS_FIXED then i['label'] = :fixed
      when STATUS_NOTAFFECT then i['label'] = :notaffect
      end
    end
    ret
  end

  # 対応状況毎の脆弱性数分布
  def self.distribution_status(category_id = 0)
    dists = SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      dists = dists.categorized
    end
    dists = dists.joins(:issue).group(:html_id, :status_id).count
    dists_htmlid = {}
    dists.each_pair do |dist, v|
      if dists_htmlid.has_key?(dist[0]) then
        if dists_htmlid[dist[0]] > dist[1] then
          dists_htmlid[dist[0]] = dist[1]
        end
      else
        dists_htmlid[dist[0]] = dist[1]
      end
    end
    ret = {
      :notconfirmed => 0,
      :ongoing => 0,
      :fixed => 0,
      :notaffect => 0,
    }
    dists_htmlid.each_pair do |html_id, status_id|
      case status_id
      when SidfmdIssue::STATUS_NOTCONFIRMED then
        ret[:notconfirmed] += 1
      when SidfmdIssue::STATUS_ONGOING then
        ret[:ongoing] += 1
      when SidfmdIssue::STATUS_FIXED then
        ret[:fixed] += 1
      when SidfmdIssue::STATUS_NOTAFFECT then
        ret[:notaffect] += 1
      end
    end
    ret
  end

  # 危険度毎の脆弱性数分布
  def self.distribution_risk(idx = :sri, category_id = 0)
    case idx
    when :sri then self.distribution_risk_sri(category_id)
    when :cvss then self.distribution_risk_cvss(category_id)
    when :softek then self.distribution_risk_softek(category_id)
    end
  end

  # 危険度毎の脆弱性数分布(SRI)
  def self.distribution_risk_sri(category_id = 0)
    hosts = SidfmdHost.enable_hosts
    if category_id > 0 then
      hosts = hosts.where(:id =>
                          SidfmdCategoriesHost
                            .where(:sidfmd_category_id => category_id)
                            .select(:sidfmd_host_id).distinct)
    else
      hosts = hosts.categorized
    end
    dists = {}
    hosts.each do |host|
      scat = host.system_category.id
      issues = host.sidfmd_issues
      issues.each do |issue|
        sri = issue.sidfmd_issue_sris.where(:category_id => scat).first
        unless dists.has_key?(issue.html_id) then
          dists[issue.html_id] = 0
        end
        if dists[issue.html_id] < sri.risk_id then
          dists[issue.html_id] = sri.risk_id
        end
      end
    end
    ret = {
      :critical => 0,
      :high => 0,
      :medium => 0,
      :low => 0,
      :none => 0,
    }
    risks = [ :none, :low, :medium, :high, :critical ]
    dists.each_pair do |html_id, risk_id|
      ret[risks[risk_id]] += 1
    end
    ret
  end

  # 危険度毎の脆弱性数分布(CVSS)
  def self.distribution_risk_cvss(category_id = 0)
    htmlids = SidfmdIssue.enable_hosts
    if category_id > 0 then
      htmlids = htmlids
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      htmlids = htmlids.categorized
    end
    htmlids = htmlids.select(:html_id).distinct
    ret = {
      :d4 => 0,
      :d3 => 0,
      :d2 => 0,
      :d1 => 0,
      :notreg => 0,
    }
    htmlids.each do |htmlid|
      issue = SidfmdIssue
        .where(:html_id => htmlid.html_id)
        .order(:id => :desc)
        .limit(1).first
      max_score = (issue.sidfmd_issue_cves.select('MAX(c2_base_score) AS c2_max_score').select('MAX(c3_base_score) AS c3_max_score'))[0]
      if max_score then
        c2_max_score = max_score.c2_max_score
        c3_max_score = max_score.c3_max_score
        if c3_max_score.nil? and c2_max_score.nil? then
          ret[:notreg] += 1
        elsif c3_max_score and c2_max_score.nil? then
          # CVSS 3
          if c3_max_score >= 9.0 then
            ret[:d4] += 1
          elsif c3_max_score < 9.0 and c3_max_score >= 7.0 then
            ret[:d3] += 1
          elsif c3_max_score < 7.0 and c3_max_score >= 4.0 then
            ret[:d2] += 1
          else
            ret[:d1] += 1
          end
        else
          # CVSS 2
          if c2_max_score >= 9.0 then
            ret[:d4] += 1
          elsif c2_max_score < 9.0 and c2_max_score >= 7.0 then
            ret[:d3] += 1
          elsif c2_max_score < 7.0 and c2_max_score >= 4.0 then
            ret[:d2] += 1
          else
            ret[:d1] += 1
          end
        end
      else
        ret[:notreg] += 1
      end
    end
    ret
  end
    
  # 危険度毎の脆弱性数分布(SofTek)
  def self.distribution_risk_softek(category_id = 0)
    dists =
      SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id)
                 .distinct)
    else
      dists = dists.categorized
    end
    dists = dists
      .group(:html_id, :risk_id).count
    ret = {
      :critical => 0,
      :high => 0,
      :medium => 0,
      :low => 0,
      :illegal => 0,
    }
    risks = [ :illegal, :low, :medium, :high, :critical ]
    dists.each_pair do |k, v|
      ret[risks[k[1]]] += 1
    end
    ret
  end

  # ヒートマップ用のデータを返す
  # :html_id, :title, :risk_id :risk_name
  def self.enable_all_vuls(idx = :sri, category_id = 0)
    case idx
    when :sri then self.enable_all_vuls_sri(category_id)
    when :cvss then self.enable_all_vuls_cvss(category_id)
    when :softek then self.enable_all_vuls_softek(category_id)
    end
  end

  # ヒートマップ用のデータを返す(SRI)
  def self.enable_all_vuls_sri(category_id = 0)
    hosts = SidfmdHost.enable_hosts
    if category_id > 0 then
      hosts = hosts.where(:id =>
                          SidfmdCategoriesHost
                            .where(:sidfmd_category_id => category_id)
                            .select(:sidfmd_host_id).distinct)
    else
      hosts = hosts.categorized
    end
    dists = {}
    hosts.each do |host|
      scat = host.system_category.id
      issues = host.sidfmd_issues
      issues.each do |issue|
        sri = issue.sidfmd_issue_sris.where(:category_id => scat).first
        unless dists.has_key?(issue.html_id) then
          dists[issue.html_id] = { :title => issue.title, :risk_id => 0 }
        end
        if dists[issue.html_id][:risk_id] < sri.risk_id then
          dists[issue.html_id][:risk_id] = sri.risk_id
        end
      end
    end
    risks = [ '無', '低', '中', '高', '重大' ]
    ret = []
    dists.each_pair do |html_id, v|
      ret << { :html_id => html_id,
        :title => v[:title],
        :risk_id => v[:risk_id],
        :risk_name => risks[v[:risk_id]],
      }
    end
    ret.sort! {|x, y| x[:html_id] <=> y[:html_id] }
  end

  # ヒートマップ用のデータを返す(CVSS)
  def self.enable_all_vuls_cvss(category_id = 0)
    htmlids = SidfmdIssue.enable_hosts
    if category_id > 0 then
      htmlids = htmlids
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id).distinct)
    else
      htmlids = htmlids.categorized
    end
    htmlids = htmlids.select(:html_id).distinct
    ret = []
    htmlids.each do |htmlid|
      issue = SidfmdIssue
        .where(:html_id => htmlid.html_id)
        .order(:id => :desc)
        .limit(1).first
      max_score = (issue.sidfmd_issue_cves.select('MAX(c2_base_score) AS c2_max_score').select('MAX(c3_base_score) AS c3_max_score'))[0]
      if max_score then
        c2_max_score = max_score.c2_max_score
        c3_max_score = max_score.c3_max_score
        if c3_max_score.nil? and c2_max_score.nil? then
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_id => 'notreg',
            :risk_name => '未登録',
          }
        elsif c3_max_score and c2_max_score.nil? then
          # CVSS 3
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_name => sprintf('%2.1f', c3_max_score),
            :risk_id => if c3_max_score >= 9.0 then
                          'd4'
                        elsif c3_max_score < 9.0 and c3_max_score >= 7.0 then
                          'd3'
                        elsif c3_max_score < 7.0 and c3_max_score >= 4.0 then
                          'd2'
                        else
                          'd1'
                        end,
          }
        else
          # CVSS 2
          ret << {
            :html_id => htmlid.html_id,
            :title => issue.title,
            :risk_name => sprintf('%2.1f', c2_max_score),
            :risk_id => if c2_max_score >= 9.0 then
                          'd4'
                        elsif c2_max_score < 9.0 and c2_max_score >= 7.0 then
                          'd3'
                        elsif c2_max_score < 7.0 and c2_max_score >= 4.0 then
                          'd2'
                        else
                          'd1'
                        end,
          }
        end
      else
        ret << {
          :html_id => htmlid.html_id,
          :title => issue.title,
          :risk_id => 'notreg',
          :risk_name => '未登録',
        }
      end
    end
    ret.sort! {|x, y| x[:html_id] <=> y[:html_id] }
  end

  # ヒートマップ用のデータを返す(SofTek)
  def self.enable_all_vuls_softek(category_id = 0)
    dists =
      SidfmdIssue.enable_hosts
    if category_id > 0 then
      dists = dists
        .where(:sidfmd_host_id =>
               SidfmdCategoriesHost
                 .where(:sidfmd_category_id => category_id)
                 .select(:sidfmd_host_id)
                 .distinct)
    else
      dists = dists.categorized
    end
    dists = dists
      .group(:html_id, :title, :risk_id).count
    risks = [ 'Illeg', 'Low', 'Medium', 'High', 'Critical' ]
    ret = []
    dists.each_pair do |k, v|
      ret << {
        :html_id => k[0],
        :title => k[1],
        :risk_id => k[2],
        :risk_name => risks[k[2]],
      }
    end
    ret.sort!{|x, y| x[:html_id] <=> y[:html_id] }
  end

  # status_idのが配列からstatus_bar用のデータを作成して返す。
  def self.statusids2count(statusids)
    status2color = {
      STATUS_NOTAFFECT => :status_notaffect,
      STATUS_FIXED => :status_fixed,
      STATUS_ONGOING => :status_ongoing,
      STATUS_NOTCONFIRMED => :status_notconfirmed
    }
    ret = []
    STATUS_ALL.each do |status_id|
      ret << {
        :id => status_id,
        :value => statusids.count(status_id),
        :name => IssueStatus.find(status_id).name,
        :color => status2color[status_id],
        :placement => 'top',
      }
    end
    ret
  end

  # journal付で対応状況の変更を行う
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  def change_status_with_journal(user_id, new_status_id, notes)
    journal_hash = {
      :journalized_id => self.issue.id,
      :journalized_type => 'Issue',
      :user_id => user_id,
      :notes => notes,
      :private_notes => false,
    }
    journal = Journal.create(journal_hash)
    old_value = self.issue.status_id
    detail_hash = {
      :journal_id => journal.id,
      :property => 'attr',
      :prop_key => 'status_id',
      :old_value => old_value ? "#{old_value}" : nil,
      :value => "#{new_status_id}",
    }
    detail = JournalDetail.create(detail_hash)
    self.issue.update_attribute :status_id, new_status_id
  end

  # journal付で対応状況の変更を行う
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  # change_status_with_journalの複数版だが、機能を向上してこちらに移行する予定。
  # うまくいけば、duedatesも同様に変更する。
  def self.change_statuses_with_journal(issue_ids, new_status_id, notes, user_id = ADMIN_USER_ID)
=begin
    journal_hash = {
      :journalized_id => self.issue.id,
      :journalized_type => 'Issue',
      :user_id => user_id,
      :notes => notes,
      :private_notes => false,
    }
    journal = Journal.create(journal_hash)
    old_value = self.issue.status_id
    detail_hash = {
      :journal_id => journal.id,
      :property => 'attr',
      :prop_key => 'status_id',
      :old_value => old_value ? "#{old_value}" : nil,
      :value => "#{new_status_id}",
    }
    detail = JournalDetail.create(detail_hash)
    self.issue.update_attribute :status_id, new_status_id

journal登録は、COPYコマンド使用
    conn = HundredColumn.connection.raw_connection
    enco = PG::TextEncoder::CopyRow.new
    conn.copy_data "COPY hundred_columns (#{header.join(',')}) FROM STDIN", enco do
      CSV.foreach(tmp.path) do |row|
        conn.put_copy_data row
      end
    end

require 'csv'
class User < ActiveRecord::Base
  def self.to_csv
    CSV.generate do |csv|
      csv << column_names
      all.each do |model|
        csv << model.attributes.values_at(*column_names)
      end
    end
  end
end

# Update all customers with the given attributes
Customer.update_all :wants_email => true

# Update all books with 'Rails' in their title
Book.update_all "author = 'David'", "title LIKE '%Rails%'"

# Update all avatars migrated more than a week ago
Avatar.update_all ['migrated_at = ?', Time.now.utc], ['migrated_at > ?', 1.week.ago]

# Update all books that match conditions, but limit it to 5 ordered by date
Book.update_all "author = 'David'", "title LIKE '%Rails%'", :order => 'created_at', :limit => 5

# Conditions from the current relation also works
Book.where('title LIKE ?', '%Rails%').update_all(:author => 'David')

# The same idea applies to limit and order
Book.where('title LIKE ?', '%Rails%').order(:created_at).limit(5).update_all(:author => 'David')

=end
  end

  # journal付で対応日付の変更を行う 
  # 必ずこれで変更すること。
  # transaction, lockは呼び元で考慮する。 
  def change_due_date_with_journal(user_id, new_due_date, notes)
    journal_hash = {
      :journalized_id => self.issue.id,
      :journalized_type => 'Issue',
      :user_id => user_id,
      :notes => notes,
      :private_notes => false,
    }
    journal = Journal.create(journal_hash)
    old_value = self.issue.due_date
    detail_hash = {
      :journal_id => journal.id,
      :property => 'attr',
      :prop_key => 'due_date',
      :old_value => old_value ? old_value.strftime('%Y/%m/%d') : nil,
      :value => new_due_date ? new_due_date.strftime('%Y/%m/%d') : nil,
    }
    detail = JournalDetail.create(detail_hash)
    self.issue.update_attribute :due_date, new_due_date
  end

  # 影響しているパッケージ一覧
  def affect_packages(suffix = '.rpm')
    ret = []
    deli = '-'
    dist = self.sidfmd_host.get_dist_data
    if dist and 'Ubuntu' == dist.name then
      deli = '_'
    end
    pkgs = sidfmd_issue_affpkgs
    pkgs.each do |pkg|
      temp = pkg.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp[:fullname] = "#{pkg['name']}#{deli}#{pkg['version']}"
      temp[:fullname] << "-#{pkg['release']}" if pkg['release']
      temp[:fullname] << "#{pkg['arch']}" if pkg['arch']
      temp[:fullname] << "#{suffix}" if (suffix and suffix.size > 0)
      ret << temp
    end
    ret.sort! {|x, y| x['name'] <=> y['name'] }
    ret.uniq!
    ret
  end

  # 影響しているプロダクト一覧
  def affect_products
    ret = []
    prds = sidfmd_issue_affprods
    prds.each do |prd|
      temp = prd.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp.delete('create_datetime')
      temp.delete('update_datetime')
      temp[:display_name] = temp['name']
      ret << temp
    end
    vers = sidfmd_issue_affvers
    vers.each do |ver|
      temp = ver.attributes
      temp.delete('id')
      temp.delete('sidfmd_issue_id')
      temp.delete('create_datetime')
      temp.delete('update_datetime')
      temp[:display_name] = temp['fullname']
      ret << temp
    end
    ret.sort! {|x, y| x[:display_name] <=> y[:display_name] }
    ret.uniq!
    ret
  end

  # 脆弱性一覧のデータを生成する。
  def self.create_all_content(**opts)
    [ :category_id, :role_admin, :corresp_required ].each do |k|
      raise ArgumentError, "must be {k.to_s}" unless opts.has_key?(k)
    end
    unless opts[:role_admin] then
      [ :user_host_ids, :user_category_ids ].each do |k|
        raise ArgumentError, "must be {k.to_s}" unless opts.has_key?(k)
      end
    end
    cache_key = '//vuls'
    if opts[:role_admin] then
      cache_key += '/admin/#{opts[:category_id]}'
    else
      cache_key += "/user/#{opts[:user_category_ids].sort.join(',')}/#{opts[:category_id]}"
    end
    opts[:no_cache] ? create_all_content_base(opts) : Rails.cache.fetch(cache_key, { :expires_in => 1.hour, :race_condition_ttl => 1.minute }) { create_all_content_base(opts) }
  end

  # sub method for create_all_content
  def self.create_all_content_base(opts)
    i_htmlids =
      SidfmdIssue.enable_hosts
    if opts[:html_ids] then
      i_htmlids = i_htmlids
        .where(:html_id => opts[:html_ids])
    end
    if opts[:role_admin] then
      if opts[:category_id] > 0 then
        i_htmlids = i_htmlids
          .where(:sidfmd_host_id =>
                 SidfmdCategoriesHost
                   .joins(:sidfmd_host)
                   .where(:sidfmd_hosts => { :flg_delete => false })
                   .where(:sidfmd_category_id => opts[:category_id])
                   .select(:sidfmd_host_id)
                   .distinct)
      else
        i_htmlids = i_htmlids.categorized
      end
    else
      if opts[:category_id] > 0 then
        cat = SidfmdCategory.find(opts[:category_id])
        if cat then
          i_htmlids = i_htmlids
            .categorized
            .where(:sidfmd_host_id => opts[:user_host_ids])
            .where(:sidfmd_host_id =>
                   SidfmdCategoriesHost
                     .joins(:sidfmd_host)
                     .where(:sidfmd_hosts => { :flg_delete => false })
                     .where(:sidfmd_category_id => opts[:category_id])
                     .select(:sidfmd_host_id)
                     .distinct)
        else
          i_htmlids = i_htmlids
            .categorized
            .where(:sidfmd_host_id => opts[:user_host_ids])
        end
      else
        i_htmlids = i_htmlids
          .categorized
          .where(:sidfmd_host_id => opts[:user_host_ids])
      end
    end
    i_htmlids = i_htmlids
      .select(:html_id).group(:html_id)
      .select(:title).group(:title)
      .select(:risk_id).group(:risk_id)
      .select(:area_id).group(:area_id)
    flg_short_description = SidfmdQueueVul.column_names.include?('short_description')
    if flg_short_description then
      i_htmlids = i_htmlids
        .select(:short_description).group(:short_description)
    end
    i_htmlids = i_htmlids
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT (CASE WHEN due_date IS NULL THEN '' ELSE TO_CHAR(due_date, 'YYYY/MM/DD') END)
 FROM issues
 WHERE id IN
       (
        SELECT issue_id
        FROM sidfmd_issues AS dd_sidfmd_issues
        INNER JOIN sidfmd_hosts AS dd_sidfmd_hosts
         ON dd_sidfmd_hosts.id=dd_sidfmd_issues.sidfmd_host_id
        WHERE dd_sidfmd_hosts.flg_delete IS FALSE AND
              dd_sidfmd_issues.html_id=sidfmd_issues.html_id
       )
), ',') AS due_dates")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT (CASE WHEN due_date IS NULL THEN '' ELSE TO_CHAR(due_date, 'YYYY/MM/DD') END)
 FROM issues
 WHERE status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')}) AND
       id IN
       (
        SELECT issue_id
        FROM sidfmd_issues AS dd_sidfmd_issues
        INNER JOIN sidfmd_hosts AS dd_sidfmd_hosts
         ON dd_sidfmd_hosts.id=dd_sidfmd_issues.sidfmd_host_id
        WHERE dd_sidfmd_hosts.flg_delete IS FALSE AND
              dd_sidfmd_issues.html_id=sidfmd_issues.html_id
       )
), ',') AS due_dates_unfinished")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT DISTINCT(TO_CHAR(created_on, 'YYYY/mm/dd HH24:MI:SS OF'))
 FROM issues
 WHERE id IN
       (
        SELECT issue_id
        FROM sidfmd_issues AS co_sidfmd_issues
        INNER JOIN sidfmd_hosts AS co_sidfmd_hosts
         ON co_sidfmd_hosts.id=co_sidfmd_issues.sidfmd_host_id
        WHERE co_sidfmd_hosts.flg_delete IS FALSE AND
              co_sidfmd_issues.html_id=sidfmd_issues.html_id
       )
),',') AS created_ons")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT DISTINCT(TO_CHAR(created_on, 'YYYY/mm/dd HH24:MI:SS OF'))
 FROM issues
 WHERE id IN
       (
        SELECT issue_id
        FROM sidfmd_issues AS eco_sidfmd_issues
        INNER JOIN sidfmd_hosts AS eco_sidfmd_hosts
         ON eco_sidfmd_hosts.id=eco_sidfmd_issues.sidfmd_host_id
        INNER JOIN issues
         ON issues.id=eco_sidfmd_issues.issue_id
        WHERE eco_sidfmd_hosts.flg_delete IS FALSE AND
              eco_sidfmd_issues.html_id=sidfmd_issues.html_id AND
              issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
       )
),',') AS elapsed_created_ons")
      .select("
(
 SELECT DISTINCT(BOOL_OR(COALESCE(c3_exploitable,c2_exploitable,FALSE)))
 FROM sidfmd_issues AS e_sidfmd_issues
 INNER JOIN issues
  ON e_sidfmd_issues.issue_id=issues.id
 INNER JOIN sidfmd_hosts AS e_sidfmd_hosts
  ON e_sidfmd_hosts.id=e_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_issue_cves
  ON e_sidfmd_issues.id=sidfmd_issue_cves.sidfmd_issue_id
 WHERE e_sidfmd_hosts.flg_delete IS FALSE AND
       e_sidfmd_issues.html_id=sidfmd_issues.html_id AND
       issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
 LIMIT 1
) AS exploitable")
      .select("
(
 SELECT DISTINCT(MAX(c3_base_score))
 FROM sidfmd_issues AS c3_sidfmd_issues
 INNER JOIN issues
  ON c3_sidfmd_issues.issue_id=issues.id
 INNER JOIN sidfmd_hosts AS c3_sidfmd_hosts
  ON c3_sidfmd_hosts.id=c3_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_issue_cves
  ON c3_sidfmd_issues.id=sidfmd_issue_cves.sidfmd_issue_id
 WHERE c3_sidfmd_hosts.flg_delete IS FALSE AND
       c3_sidfmd_issues.html_id=sidfmd_issues.html_id AND
       issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
 LIMIT 1
) AS c3_base_score")
      .select("
(
 SELECT DISTINCT(MAX(c2_base_score))
 FROM sidfmd_issues AS c2_sidfmd_issues
 INNER JOIN sidfmd_hosts AS c2_sidfmd_hosts
  ON c2_sidfmd_hosts.id=c2_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_issue_cves
  ON c2_sidfmd_issues.id=sidfmd_issue_cves.sidfmd_issue_id
 WHERE c2_sidfmd_hosts.flg_delete IS FALSE AND
       c2_sidfmd_issues.html_id=sidfmd_issues.html_id
 LIMIT 1
) AS c2_base_score")
      .select("
(
 SELECT ms_sris.sri_risk_id
 FROM
 (
  SELECT sidfmd_issue_sris.risk_id AS sri_risk_id
  FROM sidfmd_issues AS ms_sidfmd_issues
  INNER JOIN issues
   ON issues.id=ms_sidfmd_issues.issue_id
  INNER JOIN sidfmd_issue_sris
   ON sidfmd_issue_sris.sidfmd_issue_id=ms_sidfmd_issues.id
  INNER JOIN sidfmd_hosts
   ON sidfmd_hosts.id=ms_sidfmd_issues.sidfmd_host_id
  INNER JOIN sidfmd_categories_hosts
   ON sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id
  WHERE sidfmd_hosts.flg_delete IS FALSE AND
        sidfmd_issues.html_id=ms_sidfmd_issues.html_id AND
        sidfmd_issue_sris.category_id=sidfmd_categories_hosts.sidfmd_category_id AND
        issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
 ) AS ms_sris
 ORDER BY ms_sris.sri_risk_id DESC
 LIMIT 1
) AS max_sri")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT DISTINCT(
         CAST(sidfmd_hosts.id AS text) || ':' ||
         ARRAY_TO_STRING(ARRAY(
          SELECT DISTINCT(CAST(sidfmd_categories_hosts.sidfmd_category_id AS text))
          FROM sidfmd_categories_hosts
          WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id
         ),'!') || ':' ||
         sidfmd_hosts.hostname
        )
 FROM sidfmd_issues AS h_sidfmd_issues
 INNER JOIN sidfmd_hosts
  ON sidfmd_hosts.id=h_sidfmd_issues.sidfmd_host_id
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       sidfmd_hosts.id IN (
         SELECT DISTINCT(sidfmd_host_id)
         FROM sidfmd_categories_hosts
         INNER JOIN sidfmd_categories
          ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
         WHERE sidfmd_categories.is_system IS TRUE
       ) AND
       h_sidfmd_issues.html_id=sidfmd_issues.html_id
),',') AS hosts")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT DISTINCT(
         CAST(sidfmd_categories_hosts.sidfmd_category_id AS text) || ':' ||
         CAST(sidfmd_categories.is_system AS text) || ':' ||
         sidfmd_categories.name
        )
 FROM sidfmd_issues AS h_sidfmd_issues
 INNER JOIN sidfmd_hosts
  ON sidfmd_hosts.id=h_sidfmd_issues.sidfmd_host_id
 INNER JOIN sidfmd_categories_hosts
  ON sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id
 INNER JOIN sidfmd_categories
  ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       html_id=sidfmd_issues.html_id
), ',') AS categories")
      .select("
(
 SELECT COUNT(id)
 FROM
 (
  SELECT sidfmd_hosts.id,MAX(COALESCE(c3_base_score,c2_base_score,0.0)) AS max_cvss_score
  FROM sidfmd_hosts
  INNER JOIN sidfmd_issues AS cs_sidfmd_issues
   ON cs_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
  INNER JOIN issues
   ON cs_sidfmd_issues.issue_id=issues.id
  LEFT OUTER JOIN sidfmd_issue_cves
   ON sidfmd_issue_cves.sidfmd_issue_id=cs_sidfmd_issues.id
  WHERE sidfmd_hosts.flg_delete IS FALSE AND
        cs_sidfmd_issues.html_id=sidfmd_issues.html_id AND
        issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
  GROUP BY sidfmd_hosts.id
  ORDER BY sidfmd_hosts.id
 ) AS max_cvss_scores
 WHERE max_cvss_scores.max_cvss_score >= #{opts[:corresp_required][:cvss]}
 LIMIT 1
) AS not_fixed_cvss")
      .select("
(
 SELECT COUNT(id)
 FROM
 (
  SELECT sidfmd_hosts.id,MAX(sidfmd_issue_sris.risk_id) AS max_sri
  FROM sidfmd_hosts
  INNER JOIN sidfmd_issues AS sr_sidfmd_issues
   ON sr_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
  INNER JOIN issues
   ON sr_sidfmd_issues.issue_id=issues.id
  LEFT OUTER JOIN sidfmd_issue_sris
   ON sidfmd_issue_sris.sidfmd_issue_id=sr_sidfmd_issues.id
  WHERE sidfmd_hosts.flg_delete IS FALSE AND
        sr_sidfmd_issues.html_id=sidfmd_issues.html_id AND
        issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')}) AND
        sidfmd_issue_sris.category_id=
        (
         SELECT sidfmd_category_id
         FROM sidfmd_categories_hosts
         INNER JOIN sidfmd_categories
          ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
         WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id AND
               sidfmd_categories.is_system IS TRUE
         ORDER BY sidfmd_categories.id
         LIMIT 1
        )
  GROUP BY sidfmd_hosts.id
  ORDER BY sidfmd_hosts.id
 ) AS max_sris
 WHERE max_sris.max_sri >= #{opts[:corresp_required][:sri]}
 LIMIT 1
) AS not_fixed_sri")
      .select("
(
 SELECT COUNT(issues.status_id)
 FROM sidfmd_hosts
 INNER JOIN sidfmd_issues AS si_sidfmd_issues
  ON si_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
 INNER JOIN issues
  ON issues.id=si_sidfmd_issues.issue_id
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       sidfmd_hosts.id IN (
        SELECT DISTINCT(sidfmd_host_id)
        FROM sidfmd_categories_hosts
        INNER JOIN sidfmd_categories
         ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
        WHERE sidfmd_categories.is_system IS TRUE
       ) AND
       si_sidfmd_issues.html_id=sidfmd_issues.html_id AND
       issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})
 LIMIT 1
) AS not_fixed_hosts")
      .select("
ARRAY_TO_STRING(ARRAY(
 SELECT
  CAST((SELECT issues.status_id
        FROM sidfmd_hosts
        INNER JOIN sidfmd_issues AS si_sidfmd_issues
         ON si_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
        INNER JOIN issues
         ON issues.id=si_sidfmd_issues.issue_id
        WHERE sidfmd_hosts.flg_delete IS FALSE AND
              si_sidfmd_issues.html_id=sidfmd_issues.html_id AND
              base_sidfmd_hosts.id=sidfmd_hosts.id
        ORDER BY sidfmd_hosts.id) AS text)
  || ':' ||
  CAST(COALESCE((SELECT MAX(COALESCE(c3_base_score,c2_base_score,0.0))
       FROM sidfmd_hosts
       INNER JOIN sidfmd_issues AS cs_sidfmd_issues
        ON cs_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
       LEFT OUTER JOIN sidfmd_issue_cves
        ON sidfmd_issue_cves.sidfmd_issue_id=cs_sidfmd_issues.id
       WHERE sidfmd_hosts.flg_delete IS FALSE AND
             cs_sidfmd_issues.html_id=sidfmd_issues.html_id AND
             base_sidfmd_hosts.id=sidfmd_hosts.id
       GROUP BY sidfmd_hosts.id
       ORDER BY sidfmd_hosts.id),null) AS text)
  || ':' ||
  CAST(COALESCE((SELECT MAX(sidfmd_issue_sris.risk_id)
       FROM sidfmd_hosts
       INNER JOIN sidfmd_issues AS sr_sidfmd_issues
        ON sr_sidfmd_issues.sidfmd_host_id=sidfmd_hosts.id
       LEFT OUTER JOIN sidfmd_issue_sris
        ON sidfmd_issue_sris.sidfmd_issue_id=sr_sidfmd_issues.id
       WHERE sidfmd_hosts.flg_delete IS FALSE AND
             sr_sidfmd_issues.html_id=sidfmd_issues.html_id AND
             sidfmd_issue_sris.category_id=
              (SELECT sidfmd_category_id
               FROM sidfmd_categories_hosts
               INNER JOIN sidfmd_categories
                ON sidfmd_categories.id=sidfmd_categories_hosts.sidfmd_category_id
               WHERE sidfmd_categories_hosts.sidfmd_host_id=sidfmd_hosts.id AND
                     sidfmd_categories.is_system IS TRUE
               ORDER BY sidfmd_categories_hosts.id
               LIMIT 1) AND
              base_sidfmd_hosts.id=sidfmd_hosts.id
       GROUP BY sidfmd_hosts.id
       ORDER BY sidfmd_hosts.id),sidfmd_issues.risk_id) AS text)
 FROM sidfmd_hosts AS base_sidfmd_hosts
 WHERE base_sidfmd_hosts.flg_delete IS FALSE AND
       base_sidfmd_hosts.id IN
        (SELECT DISTINCT sidfmd_host_id
         FROM sidfmd_issues AS scsio_sidfmd_issues
         WHERE scsio_sidfmd_issues.html_id=sidfmd_issues.html_id)
),',') AS sri_cvss_status_ids_orig")
      .select("
(
 SELECT COUNT(sidfmd_hosts.id) > 0
 FROM sidfmd_hosts
 WHERE sidfmd_hosts.flg_delete IS FALSE AND
       sidfmd_hosts.id IN
       (
        SELECT nfw_sidfmd_issues.sidfmd_host_id
        FROM sidfmd_issues AS nfw_sidfmd_issues
        INNER JOIN issues ON issues.id=nfw_sidfmd_issues.issue_id
        WHERE nfw_sidfmd_issues.html_id=sidfmd_issues.html_id AND
              nfw_sidfmd_issues.html_id < 0 AND
              nfw_sidfmd_issues.risk_id >= #{opts[:corresp_required][:sri]} AND
              issues.status_id IN (#{SidfmdIssue::STATUS_UNFINISHED.join(',')})              
       )
 LIMIT 1
) AS not_fixed_win")
    ret = []
    # 依存関係に注意
    i_htmlids.each do |i|
      sri_cvss_status_ids =
        i[:sri_cvss_status_ids_orig].split(/,/, -1).collect {|x|
          y = x.split(/:/, -1)
          { :status_id => y[0].to_i,
            :cvss => y[1] ? y[1].to_f : nil,
            :sri => y[2] ? y[2].to_i : 0,
          }
      }
      item = {
        :sri_cvss_status_ids => sri_cvss_status_ids,
        :html_id => i[:html_id],
        :title => i[:title],
        :softek => {
          :risk_id => i[:risk_id],
          :area_id => i[:area_id],
        },
        :short_description => flg_short_description ? (i[:short_description] || '') : '',
        :due_dates => i[:due_dates].split(/,/, -1).collect {|x| x.size > 0 ? x : nil },
        :due_dates_unfinished => i[:due_dates_unfinished].split(/,/, -1).collect {|x| x.size > 0 ? x : nil },
        :created_ons => i[:created_ons].split(/,/, -1).collect {|x| DateTime.parse(x) },
        :elapsed_created_ons => i[:elapsed_created_ons].split(/,/, -1).collect {|x| DateTime.parse(x) },
        :exploitable => i[:exploitable],
        :c3_base_score => i[:c3_base_score],
        :c2_base_score => i[:c2_base_score],
        :status_ids => sri_cvss_status_ids.collect {|x| x[:status_id] },
        :max_sri => i[:max_sri] ? i[:max_sri].to_i : 0,
        :hosts => i[:hosts].split(/,/).collect {|x|
          if /\A(\d+):([0-9!]*):(.*)\z/ =~ x then
            id = Regexp.last_match(1).to_i
            hostname = Regexp.last_match(3)
            categories = Regexp.last_match(2).split(/!/).collect {|x| x.to_i }
            if opts[:category_id] <= 0 then
              if opts[:role_admin] then
                { :id => id, :hostname => hostname }
              elsif opts[:user_host_ids].include?(id) then
                { :id => id, :hostname => hostname }
              else
                nil
              end
            elsif opts[:category_id] > 0 and categories.include?(opts[:category_id]) then
              { :id => id, :hostname => hostname }
            else
              nil
            end
          else
            # invalid data
            nil
          end
        }.compact,
      }
      temp_categories = i[:categories].split(/,/).collect {|x|
        if /\A(\d+):(true|false):(.*)\z/ =~ x then
          id = Regexp.last_match(1).to_i
          is_system = ('true' == Regexp.last_match(2))
          cat_name = Regexp.last_match(3)
          { :id => id, :is_system => is_system, :name => cat_name }
        else
          # invalid data
          nil
        end
      }.compact.sort {|x, y| x[:id] <=> y[:id] }
      item[:categories] = if opts[:role_admin] then
                            temp_categories
                          else
                            temp_categories.collect {|x|
                              if x[:is_system] then
                                x
                              elsif opts[:user_category_ids].include?(x[:id]) then
                                x
                              else
                                nil
                              end
                            }.compact
                          end
      if item[:html_id] < 0 then
        item[:status_ids] = [] unless item[:status_ids]
        item[:max_sri] = i[:risk_id]
      end
      item[:min_status_id] = item[:status_ids].min
      temp_cu = item[:created_ons].compact.uniq
      item[:min_created_on] = { :value => temp_cu.min }
      item[:min_created_on][:display] = item[:min_created_on][:value] ? item[:min_created_on][:value].localtime.strftime('%Y/%m/%d') : '---'
      item[:min_created_on][:elapsed_days] = item[:min_created_on][:value] ? ((Time.current - item[:min_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
      item[:min_elapsed_created_on] = { :value => item[:elapsed_created_ons].compact.uniq.min }
      item[:min_elapsed_created_on][:display] = item[:min_elapsed_created_on][:value] ? ((Time.current - item[:min_elapsed_created_on][:value]) / 24.0 / 60.0 / 60.0).to_i : -1
      item[:cvss] = {
        :score => [ item[:c3_base_score], item[:c2_base_score] ].compact.max,
      }
      item[:cvss][:label] = SidfmdIssueCve.labelling(item[:cvss][:score])
      item[:hosts].sort! {|x, y| x[:hostname] <=> y[:hostname] }
      item[:warning_orig] = {
        :sri => item[:max_sri] ? (item[:max_sri] >= opts[:corresp_required][:sri]) : false,
        :cvss => item[:cvss][:score] ? (item[:cvss][:score] >= opts[:corresp_required][:cvss]) : true,
      }
      item[:warning] = {
        :sri => SidfmdIssue::STATUS_UNFINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:sri] : false,
        :cvss => SidfmdIssue::STATUS_UNFINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:cvss] : false,
      }
      item[:fixed_warning] = {
        :sri => SidfmdIssue::STATUS_FINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:sri] : false,
        :cvss => SidfmdIssue::STATUS_FINISHED.include?(item[:min_status_id]) ? item[:warning_orig][:cvss] : false,
      }
      item[:not_fixed_warning] = {
        :sri => item[:warning][:sri],
        :cvss => item[:warning][:cvss],
      }
      if item[:categories] then
        item[:categories].sort! {|x, y|
          r = (x[:is_system] ? 100 : 500) <=> (y[:is_system] ? 100 : 500)
          if r == 0 then
            r = x[:name] <=> y[:name]
          end
          r
        }
      end
      item[:not_fixed_hosts] = i[:not_fixed_hosts] ? i[:not_fixed_hosts] : 0
      ret << item
    end
    ret
  end

end
