var list_box = (function(){
  "use strict";
  var list_box = {};
  var _this = list_box;

  _this.show = function(opts){
    _this.init(opts);
    var loading = $("#content-block").modalLoading(100, "<div class=\"_loading\"></div>");
    var vulns_req = $.post($('#sidfm_internal_api_info_find_vulns_path').val(), _this.query_params, function(res){
                      _this.vulns = res.vulns;
                      _this.vulns.total_count = res.total_count;
                      _this.vulns.limit = res.limit;
                      _this.vulns.offset = res.offset;
                    });
    var areas_req = $.post($('#sidfm_internal_api_info_areas_path').val(), function(res){_this.areas = res.areas;});
    var risks_req = $.post($('#sidfm_internal_api_info_risks_path').val(), function(res){_this.risks = res.risks;});
    $.when(vulns_req, areas_req, risks_req).done(function(){
      _this.make_data();
      var htmlids = [];
      _this.vulns.forEach(function(vuln){htmlids.push(vuln.id)});
      var htmlids2issue_req = $.post($('#sidfm_informations_api_htmlids2issue_path').val(), { html_ids:htmlids }, function(res) {_this.htmlids2issue = res;});
      $.when(htmlids2issue_req).done(function() {
        _this.render();
	_this.bind_event();
	loading.remove();
      });
    });
  };

  _this.init = function(opts){
    var d2 = new Date();
    var d1 = new Date();
    var y = d1.getFullYear();
    d1.setYear(y-3); // デフォルトの検索期間は3年
    _this.vuln_content = {};
    _this.vuln_tab = "description";
    _this.query_params = {
      limit:25,
      offset:0,
      search_text:null,
      begin_date:d1.toISOString().substr(0,10),
      end_date:d2.toISOString().substr(0,10),
      cvss_base_score:null
    }
    $("input.j_begin_date").val(_this.query_params.begin_date);
    $("input.j_end_date").val(_this.query_params.end_date);
  };

  _this.make_data = function(){
    _this.query_params.limit = _this.vulns.limit;
    _this.query_params.offset = _this.vulns.offset;

    _this.vulns.forEach(function(vuln){
      vuln.area = _.find(_this.areas, function(area){return area.id === vuln.area_id});
      vuln.risk = _.find(_this.risks, function(risk){return risk.id === vuln.risk_id});
      vuln.create_date = vuln.create_datetime.substr(0, 10).replace(/-/g, "/");
      vuln.update_date = vuln.update_datetime.substr(0, 10).replace(/-/g, "/");
      if(vuln.create_date === vuln.update_date){
        vuln.update_date = '-'
      }
      vuln.cvss_count = 0;
      vuln.cves.forEach(function(cve){
        if(cve.cvss2 == null){
          cve.cvss2 = {base_score:-1}
        }
        if(cve.cvss3 == null){
          cve.cvss3 = {base_score:-1}
        }
        //最大値・ソート用 //CVSS3優先
        if(cve.cvss3.base_score >= 0){
          cve.cve_cvss = SidCvss3Map.parse_vector_string(cve.cvss3);
          cve.cve_cvss.ver = 3;
          cve.cve_cvss.color = {};
          ["av", "ac", "pr", "ui", "s", "c", "i", "a"].forEach(function(key){
            cve.cve_cvss.color[key] = SidCvss3Map[key].colors[ SidCvss3Map[key][cve.cve_cvss["base_" + key].toUpperCase()] ];
          });
          vuln.cvss_count++;
        }else if(cve.cvss2.base_score >= 0){
          cve.cve_cvss = SidCvss2Map.parse_vector_string(cve.cvss2);
          cve.cve_cvss.ver = 2;
          cve.cve_cvss.color = {};
          ["av", "ac", "au", "c", "i", "a"].forEach(function(key){
            cve.cve_cvss.color[key] = SidCvss2Map[key].colors[ SidCvss2Map[key][cve.cve_cvss["base_" + key].toUpperCase()] ];
          });
          vuln.cvss_count++;
        }else {
          cve.cve_cvss = {base_score:-1}
        }
      });
      if(vuln.cves.length > 0){
        var max_cvss_cve = _.max(vuln.cves, function(cve){return cve.cve_cvss.base_score});
        if(max_cvss_cve.cve_cvss.base_score > -1){
          if(max_cvss_cve.cvss3.base_score >= 0){
            vuln.max_cvss = max_cvss_cve.cvss3;
          }else{
            vuln.max_cvss = max_cvss_cve.cvss2;
          }
        }else{
          vuln.max_cvss = max_cvss_cve.cve_cvss
        }
        vuln.cvss_ordered_cves = _.sortBy(vuln.cves, function(cve){return -cve.cve_cvss.base_score});
        vuln.ordered_cves = _.sortBy(vuln.cves, function(cve){return (cve.cve_id.split("-")[2] - 0)});
      }else{
        vuln.max_cvss = null;
        vuln.cvss_ordered_cves = [];
        vuln.ordered_cves = [];
      }
    });
  };

  _this.render = function(){
    var vulns = _this.vulns;
    if(_this.compiled_template == null){
      var tmpl = $("#template").html();
      _this.compiled_template = _.template(tmpl);
    }
    $("#vuln_list").html(_this.compiled_template({vulns:vulns, query_params:_this.query_params, htmlids2issue:_this.htmlids2issue}));
    $("input.j_begin_date").val(_this.query_params.begin_date);
    $("input.j_end_date").val(_this.query_params.end_date);
  };

  _this.update_list = function(opts){
    var params = $.extend(_this.query_params, opts);
    var loading = $("#content-block").modalLoading(100, "<div class=\"_loading\"></div>");
    var vulns_req = $.post($('#sidfm_internal_api_info_find_vulns_path').val(), params, function(res){
                      _this.vulns = res.vulns;
                      _this.vulns.total_count = res.total_count;
                      _this.vulns.limit = res.limit;
                      _this.vulns.offset = res.offset;
                      _this.query_params.begin_date = res.begin_date;
                      _this.query_params.end_date = res.end_date;
                    });
    $.when(vulns_req).done(function(){
      _this.make_data();
      var htmlids = [];
      _this.vulns.forEach(function(vuln){htmlids.push(vuln.id)});
      var htmlids2issue_req = $.post($('#sidfm_informations_api_htmlids2issue_path').val(), { html_ids:htmlids }, function(res) {_this.htmlids2issue = res;});
      $.when(htmlids2issue_req).done(function() {
        _this.render();
        loading.remove();
      });
    });
  };

  _this.bind_event = function(){
    $("main").on("change", ".j_paginator .j_limit", function(){list_box.change_limit(this)});
    $("main").on("click", ".j_paginator .j_next", function(){list_box.click_next(this)});
    $("main").on("click", ".j_paginator .j_prev", function(){list_box.click_prev(this)});
    $("main").on("keypress", ".j_paginator .j_specified_record", function(ev){list_box.keypress_specified_record(this,ev)});
    $("main").on("click", ".j_cvss_more", function(){list_box.click_cvss_more(this)});
    $("main").on("submit", "#finder", function(){list_box.submit_finder();return false;});
  };

  _this.submit_finder = function(){
    var j_form = $("#finder");
    var params = {};
    var allow_keys = {search_text:true, cvss_base_score:true, begin_date:true, end_date:true}
    j_form.serializeArray().forEach(function(e){
      if(allow_keys[e.name]){
        params[e.name] = e.value;
      }
    });
    _this.query_params = $.extend(_this.query_params, params);
    _this.update_list(_this.query_params);
  };

  _this.click_cvss_more = function(elm){
    var j_elm = $(elm);
    var j_caption = j_elm.find(".j_cvss_more_caption");
    if(j_caption.text() === "すべて表示"){
      j_elm.closest(".j_cvsss").find(".j_cvss_other").show();
      j_caption.text("元に戻す")
    }else{
      j_elm.closest(".j_cvsss").find(".j_cvss_other").hide();
      j_caption.text("すべて表示")
    }
  };

  _this.keypress_specified_record = function(elm, ev){
    var j_elm = $(elm);
    if(ev.keyCode === 13){// Return
      var offset = parseInt(j_elm.val(),10) - 1;
      if(offset >= 0){
        j_elm.val(offset+1);
        _this.update_list({offset:(offset)});
      }else{
        j_elm.val(1);
      }
    }
  };

  _this.click_prev = function(elm){
    var offset = _this.vulns.offset - _this.vulns.limit;
    _this.update_list({offset:offset});
  };

  _this.click_next = function(elm){
    var offset = _this.vulns.offset + _this.vulns.limit;
    _this.update_list({offset:offset});
  };

  _this.change_limit = function(elm){
    var j_elm = $(elm);
    var limit = j_elm.val() - 0;
    _this.update_list({limit:limit});
  };

  return _this;
})();

$(function(){
  "use strict";
  list_box.show();
});
