# -*- coding: utf-8 -*-

require 'date'
require 'sidfm_controller_support'

class SidfmRuleSettingController < ApplicationController
  unloadable

  layout 'sidfm'

  self.main_menu = false

  protect_from_forgery with: :exception

  helper :sidfm

  private
  
  include SidfmControllerSupport
  before_action :get_sidfm_roles
  before_action :create_common_data

  before_action :check_license
  before_action :check_roles, except: [:index]
  before_action :define_user_custom_category_ids


  def check_roles 
    if !@sidfm_roles.include?(:admin) && !@sidfm_roles.include?(:use) then
      redirect_to :action => :index, :notice => '許可されていない操作です。'
    end
  end

  def define_user_custom_category_ids 
    @user_custom_category_ids = if @sidfm_roles.include?(:admin) then
                                  SidfmdCategory.where(:is_system => false)
                                                  .map{|c| c.id}
                                else
                                  User.current
                                      .sidfmd_categories
                                      .where(:is_system => false)
                                      .map{|c| c.id} 
                                end
  end

  def create_rule_param(rule_name, rule_category, is_enable, rule_trigger, rule_conditions, rule_actions)
    trigger_name = rule_trigger.class.table_name
    send_mail_user_ids = []
    send_mail_user_emails = []
    send_mail_user_record = []
    param = {}
    param["rule_name"] = rule_name
    param["is_enable"] = is_enable
    param["category"] = rule_category ? rule_category : "all"
    param["trigger"] = if trigger_name == "sidfmd_rule_tp_hours" then
                         {
                           "hostname" => "",
                           "period" => "periodic-hour",
                           "value" => rule_trigger.hour,
                           "unit" => "",
                           "category" => "",
                         }
                       elsif trigger_name == "sidfmd_rule_tp_intervals" then
                         {
                           "hostname" => "",
                           "period" => "periodic-interval",
                           "value" => rule_trigger.value,
                           "unit" => rule_trigger.unit,
                           "category" => "",
                         }
                       elsif trigger_name == "sidfmd_rule_t_vulsyncs" then
                         {
                           "hostname" => rule_trigger.sidfmd_host_id ? SidfmdHost.find(rule_trigger.sidfmd_host_id).hostname : "",
                           "period" => "vulsync",
                           "value" => "vulsync",
                           "unit" => "",
                           "category" => rule_trigger.sidfmd_category_id ? rule_trigger.sidfmd_category_id : "all",
                         }
                       end
    cond_param = {}
    rule_conditions.each do |cond|
      cond = cond.detail
      cond_name = cond.class.table_name
      if cond_name == "sidfmd_rule_c_issue_updates" then
        if cond.cond_or && cond.cnt_create > 0 then
          cond_param["issue-update-regi"] = cond.cnt_create
        elsif cond.cond_or && cond.cnt_remove > 0 then
          cond_param["issue-update-del"] = cond.cnt_remove
        elsif !cond.cond_or then
          cond_param["issue-update-regi-and-del"] = {"regi" => cond.cnt_create, "del" => cond.cnt_remove}
        end
      elsif cond_name == "sidfmd_rule_c_sri_aboves" then
        cond_param["sri-above"] = cond.risk_id
      elsif cond_name == "sidfmd_rule_c_sri_belows" then
        cond_param["sri-below"] = cond.risk_id
      elsif cond_name == "sidfmd_rule_c_cvss_aboves" then
        cond_param["cvss-above"] = {
          "val" => cond.cvss_value,
          "include_notreg" => cond.is_notreg.to_s,
          "flg_exploitable" => if cond.exploitable.nil? then
                                 "all"
                               elsif cond.exploitable == false then
                                 "not_exist"
                               else
                                 "exist"
                               end
        }
      elsif cond_name == "sidfmd_rule_c_cvss_belows" then
        cond_param["cvss-below"] = {
          "val" => cond.cvss_value,
          "include_notreg" => cond.is_notreg.to_s,
          "flg_exploitable" => if cond.exploitable.nil? then
                                 "all"
                               elsif cond.exploitable == false then
                                 "not_exist"
                               else
                                 "exist"
                               end
      }
      elsif cond_name == "sidfmd_rule_c_cvss_notregs" then
        cond_param["cvss-notreg"] = cond.is_notreg.to_s
      elsif cond_name == "sidfmd_rule_c_warnings" then
        if cond.is_cvss then
          cond_param["cvss-warning"] = cond.is_warning.to_s
        else
          cond_param["sri-warning"] = cond.is_warning.to_s
        end
      elsif cond_name == "sidfmd_rule_c_history_posts" then
        cond_param["history-create"] = { "val" => cond.cnt }
      elsif cond_name == "sidfmd_rule_c_duedate_expireds" then
        cond_param["due-date-expired"] = "due-date-expired"
      elsif cond_name == "sidfmd_rule_c_duedate_approachings" then
        cond_param["due-date-approach"] = { "val" => cond.days }
      elsif cond_name == "sidfmd_rule_c_match_categories" then
        sub_categories = cond.sidfmd_rule_cm_category_subs.map{|c| c.sidfmd_category_id}
        if cond.inv then
          cond_param["category-unmatch"] = sub_categories
        else
          cond_param["category-match"] = sub_categories
       end
      elsif cond_name == "sidfmd_rule_c_match_hosts" then
        host_ids = cond.sidfmd_rule_cm_host_subs.map{|h| h.sidfmd_host_id}
        hosts = SidfmdHost.where(:id => host_ids)
        hostnames = hosts.map{|h| h.hostname}
        if cond.inv then
          cond_param["host-unmatch"] = hostnames
        else
          cond_param["host-match"] = hostnames
        end
      elsif cond_name == "sidfmd_rule_c_statuses" then
        status_ids = [ cond.status_id1 ]
        status_ids.push(cond.status_id2) if cond.status_id2
        cond_param["status-match"] = status_ids
      elsif cond_name == "sidfmd_rule_c_match_apps" then
        sub_apps = cond.sidfmd_rule_cm_app_subs
        prod_ids = []
        ver_ids = []
        sub_apps.each do |app|
          if !app.version_id then
            prod_ids.push(app.product_id)
          else
            ver_ids.push(app.version_id)
          end
        end
        match_app_param = {
          "product-ids" => prod_ids,
          "version-ids" => ver_ids,
        }
        if cond.inv then
          cond_param["product-unmatch"] = match_app_param
        else
          cond_param["product-match"] = match_app_param
        end
      elsif cond_name == "sidfmd_rule_c_cvss_exploitables" then
        cond_param["cvss-exploitable"] = {
          "value" => cond.value.to_s,
          "is_change" => cond.is_change.to_s
        }
      end
    end
    param["condition"] = cond_param

    action_param = {}
    rule_actions.each do |action|
      action_target = action.target.to_s
      action_flg = action.proc_empty.to_s
      action = action.detail
      action_name = action.class.table_name
      if action_name == "sidfmd_rule_a_mail_categories" then
        if action.from_user_id then
          sender_info  = User.find(action.from_user_id)
          sender = { "userid" => action.from_user_id, "username" => sender_info.lastname + sender_info.firstname }
        else
          sender = { "address" => action.from_email }
        end
        action_param["send-mail-category"] = {
          "sendby" => { "ids" => [(action.sidfmd_category_id ? action.sidfmd_category_id : 'all')] },
          "title" => action.subject,
          "content" => action.body,
          "sender" => sender,
          "target" => action_target,
          "proc_empty" => action_flg
        }
      elsif action_name == "sidfmd_rule_a_mail_users" then
        if action.user_id then
          send_mail_user_ids.push(action.user_id)
        else
          send_mail_user_emails.push(action.email)
        end
          send_mail_user_record.push(action) if send_mail_user_record.size == 0
      elsif action_name == "sidfmd_rule_a_status_changes" then
        status_ids_from = [action.prev_status1]
        status_ids_from.push(action.prev_status2) if action.prev_status2
        action_param["status-change"] = {
          "status_ids_from" => status_ids_from,
          "status_ids_to" => [action.status_id],
          "target" => action_target,
          "proc_empty" => action_flg
        }
      elsif action_name == "sidfmd_rule_a_duedate_changes" then
        if action.fixed_date then
          action_param["due-date-change"] = {
            "date" => action.fixed_date.strftime("%Y/%m/%d"),
            "permit-update" => action.permit_update.to_s,
            "target" => action_target,
            "proc_empty" => action_flg
          }
        else
          action_param["due-date-change"] = {
            "days" => action.refdt_days,
            "permit-update" => action.permit_update.to_s,
            "target" => action_target,
            "proc_empty" => action_flg
          }
        end
      end
    end
    if send_mail_user_record.size == 1 then
      record = send_mail_user_record[0]
      record_action = record.sidfmd_rule_action
      if send_mail_user_ids.size > 0 then
        sendby = { "ids" => send_mail_user_ids }
      else 
        sendby ={ "address" => send_mail_user_emails }
      end
      if record.from_user_id then
        sender_info = User.find(record.from_user_id)
        sender = { "userid" => record.from_user_id, "username" => sender_info.lastname + sender_info.firstname }
      else
        sender = { "address" => record.from_email }
      end
      action_param["send-mail-user"] = {
        "sendby" => sendby,
        "title" => record.subject,
        "content" => record.body,
        "sender" => sender,
        "target" => record_action.target.to_s,
        "proc_empty" => record_action.proc_empty.to_s
      }
    end
    param["action"]= action_param
    return param
  end

  def set_rule(rulename, category, flg, trigger, condition, operation, dump, method_name, rule_id)
    begin
      SidfmdRule.transaction do
        if method_name == 'create' then
          new_rule = SidfmdRule.new(
                                user_id: User.current.id,
                                sidfmd_category_id: category,
                                position: 0,
                                flg_enable: flg == "enable" ? true : false,
                                name: rulename
                               )
          if new_rule.save then
            rule_id = new_rule.id
          end
        elsif method_name == 'modify' then
          target_rule = SidfmdRule.find(rule_id)
          target_rule.sidfmd_rule_trigger.destroy
          target_rule.sidfmd_rule_conds.destroy_all
          target_rule.sidfmd_rule_actions.destroy_all
          target_rule.update_attributes(
                      sidfmd_category_id: category,
                      position: 0,
                      flg_enable: flg == "enable" ? true : false,
                      name: rulename
                     )
        end
        rule_trigger = SidfmdRuleTrigger.new(sidfmd_rule_id: rule_id)
        if rule_trigger.save then
        #トリガー作成
          period = trigger["period"]
          num_of_unit = trigger["value"]
          unit = trigger["unit"]
          category = trigger["category"]
          if period == "periodic-hour" then
            trigger_setting = SidfmdRuleTpHour.new(
                                                sidfmd_rule_trigger_id: rule_trigger.id,
                                                hour: num_of_unit.to_i
                                              )
          elsif period == "periodic-interval" then
            trigger_setting = SidfmdRuleTpInterval.new(
                                                    sidfmd_rule_trigger_id: rule_trigger.id,
                                                    value: num_of_unit.to_i,
                                                    unit: unit
                                                   )
          elsif period == "vulsync" then
            host = SidfmdHost.find_by(:hostname => trigger["hostname"])
            host_id = host ? host.id : nil
            trigger_setting = SidfmdRuleTVulsync.new(
                                                  sidfmd_rule_trigger_id: rule_trigger.id,
                                                  sidfmd_category_id: category == "all" ? nil : category.to_i,
                                                  sidfmd_host_id: host_id
                                                 )
          end
          trigger_setting.save
        end
        #タイミング作成
        cond_vul_regi = condition["issue-update-regi"]
        cond_vul_del = condition["issue-update-del"]
        cond_vul_regi_and_del = condition["issue-update-regi-and-del"]
        cond_sri_above = condition["sri-above"]
        cond_sri_below = condition["sri-below"]
        cond_sri_warning = condition["sri-warning"]
        cond_cvss_above = condition["cvss-above"]
        cond_cvss_below = condition["cvss-below"]
        cond_cvss_warning = condition["cvss-warning"]
        cond_cvss_notreg = condition["cvss-notreg"]
        cond_due_date_expired = condition["due-date-expired"]
        cond_due_date_approach = condition["due-date-approach"]
        cond_history_create = condition["history-create"]
        cond_category_match = condition["category-match"]
        cond_category_unmatch = condition["category-unmatch"]
        cond_host_match = condition["host-match"]
        cond_host_unmatch = condition["host-unmatch"]
        cond_status_match = condition["status-match"]
        cond_product_match = condition["product-match"]
        cond_product_unmatch = condition["product-unmatch"]
        cond_cvss_exploitable = condition["cvss-exploitable"]
        if cond_vul_regi then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                    sidfmd_rule_cond_id: rule_cond.id,
                                    cond_or: true,
                                    cnt_create: cond_vul_regi.to_i,
                                    cnt_remove: 0
                                   )
          end
        end
        if cond_vul_del then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                    sidfmd_rule_cond_id: rule_cond.id,
                                    cond_or: true,
                                    cnt_create: 0,
                                    cnt_remove: cond_vul_del.to_i    
                                   )
          end
        end
        if cond_vul_regi_and_del then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCIssueUpdate.create(
                                   sidfmd_rule_cond_id: rule_cond.id,
                                   cond_or: false,
                                   cnt_create: cond_vul_regi_and_del["regi"].to_i,
                                   cnt_remove: cond_vul_regi_and_del["del"].to_i
                                   )
          end
        end
        if cond_sri_above then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCSriAbove.create(
                                 sidfmd_rule_cond_id: rule_cond.id,
                                 risk_id: cond_sri_above.to_i
                                )
          end
        end
        if cond_sri_below then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCSriBelow.create(
                                sidfmd_rule_cond_id: rule_cond.id,
                                risk_id: cond_sri_below.to_i
                                )
          end
        end
        if cond_sri_warning then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCWarning.create(
                               sidfmd_rule_cond_id: rule_cond.id,
                               is_warning: cond_sri_warning == "true" ? true : false,
                               is_cvss: false
                               )
          end
        end
        if cond_cvss_above then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            flg_exploitable = cond_cvss_above["flg_exploitable"]
            SidfmdRuleCCvssAbove.create(
                                 sidfmd_rule_cond_id: rule_cond.id,
                                 is_notreg: cond_cvss_above["include_notreg"] == "true" ?  true : false,
                                 cvss_value: cond_cvss_above["val"].to_f,
                                 exploitable: if flg_exploitable == "all" then
                                                nil
                                              elsif flg_exploitable == "exist" then
                                                true
                                              else
                                                false
                                              end
                                 )
          end
        end
        if cond_cvss_below then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
              flg_exploitable = cond_cvss_below["flg_exploitable"]
              SidfmdRuleCCvssBelow.create(
                                   sidfmd_rule_cond_id: rule_cond.id,
                                   is_notreg: cond_cvss_below["include_notreg"] == "true" ?  true : false,
                                   cvss_value: cond_cvss_below["val"].to_f,
                                   exploitable: if flg_exploitable == "all" then
                                                  nil
                                                elsif flg_exploitable == "exist" then
                                                  true
                                                else
                                                  false
                                                end
                                 )
          end
        end
        if cond_cvss_warning then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCWarning.create(
                                sidfmd_rule_cond_id: rule_cond.id,
                                is_warning: cond_cvss_warning == "true" ? true : false,
                                is_cvss: true
                               )
          end
        end
        if cond_cvss_notreg then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCCvssNotreg.create(
                                  sidfmd_rule_cond_id: rule_cond.id,
                                  is_notreg: cond_cvss_notreg == "true" ? true : false
                                  )
          end
        end
        if cond_due_date_expired then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCDuedateExpired.create(
                                      sidfmd_rule_cond_id: rule_cond.id,
                                     )
          end
        end
        if cond_due_date_approach then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            days = cond_due_date_approach["val"].to_i
            SidfmdRuleCDuedateApproaching.create(
                                          sidfmd_rule_cond_id: rule_cond.id,
                                          days: days.to_i
                                          )
          end
        end
        if cond_history_create then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            cnt = cond_history_create["val"]
            SidfmdRuleCHistoryPost.create(
                                   sidfmd_rule_cond_id: rule_cond.id,
                                   cnt: cnt.to_i
                                  )
          end
        end
        if cond_category_match then
          if !@sidfm_roles.include?(:admin) then
            pass = true
            cond_category_match.each do |c_id|
              if !@user_custom_category_ids.include?(c_id.to_i) then
                pass = false
                break
              end
            end
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            if !cond_category_match || (!@sidfm_roles.include?(:admin) && !pass) then
              dump[:notice] = "不正な入力です"
              redirect_to dump and return
            end
            c_match = SidfmdRuleCMatchCategory.new(
                                               sidfmd_rule_cond_id: rule_cond.id,
                                               inv: false
                                               )
            if c_match.save then
              cond_category_match.each do |cat_id|
                SidfmdRuleCmCategorySub.create(
                                        sidfmd_rule_c_match_category_id: c_match.id,
                                        sidfmd_category_id: cat_id.to_i
                                       )
              end
            end
          end
        end
        if cond_category_unmatch then
          if !@sidfm_roles.include?(:admin) then
            pass = true
            cond_category_unmatch.each do |c_id|
              if !@user_custom_category_ids.include?(c_id.to_i) then
                pass = false
                break
              end
            end
          end
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            if !cond_category_unmatch || (!@sidfm_roles.include?(:admin) && !pass) then
              dump[:notice] = "不正な入力です"
              redirect_to dump and return
            end
            c_unmatch = SidfmdRuleCMatchCategory.new(
                                                 sidfmd_rule_cond_id: rule_cond.id,
                                                 inv: true
                                                )
            if c_unmatch.save then
              cond_category_unmatch.each do |cat_id|
                SidfmdRuleCmCategorySub.create(
                                        sidfmd_rule_c_match_category_id: c_unmatch.id,
                                        sidfmd_category_id: cat_id.to_i
                                       )
              end
            end
          end
        end
        if cond_host_match then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            selected_hosts = SidfmdHost.where(:hostname => cond_host_match)
            if !@sidfm_roles.include?(:admin) then
              hosts_cat_ids = SidfmdCategoriesHost.where(:sidfmd_host_id => selected_hosts.map{|h| h.id})
                                                  .map{|h| h.sidfmd_category_id}
              @user_custom_category_ids.each do |id|
                if !hosts_cat_ids.include?(id) then
                  dump[:notice] = "不正な入力です"
                  redirect_to dump and return
                end
              end
            end
            h_match = SidfmdRuleCMatchHost.new(
                                           sidfmd_rule_cond_id: rule_cond.id,
                                           inv: false
                                           )
            if h_match.save then
              selected_hosts.each do |host|
                SidfmdRuleCmHostSub.create(
                                    sidfmd_rule_c_match_host_id: h_match.id,
                                    sidfmd_host_id: host.id
                                   )
              end
            end
          end
        end
        if cond_host_unmatch then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            selected_hosts = SidfmdHost.where(:hostname => cond_host_unmatch)
            if !@sidfm_roles.include?(:admin) then
              hosts_cat_ids = SidfmdCategoriesHost.where(:sidfmd_host_id => selected_hosts.map{|h| h.id})
                                                  .map{|h| h.sidfmd_category_id}
              @user_custom_category_ids.each do |id|
                if !hosts_cat_ids.include?(id) then
                  dump[:notice] = "不正な入力です"
                  redirect_to dump and return
                end
              end
            end
            h_unmatch = SidfmdRuleCMatchHost.new(
                                             sidfmd_rule_cond_id: rule_cond.id,
                                             inv: true
                                             )
            if h_unmatch.save then
              selected_hosts.each do |host|
                SidfmdRuleCmHostSub.create(
                                    sidfmd_rule_c_match_host_id: h_unmatch.id,
                                    sidfmd_host_id: host.id
                                    )
              end
            end
          end
        end
       if cond_status_match then
         rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCStatus.create(
                              sidfmd_rule_cond_id: rule_cond.id,
                              status_id1: cond_status_match[0].to_i,
                              status_id2: cond_status_match.length == 2 ? cond_status_match[1] : nil,
                              )
          end
        end
        if cond_product_match then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            app_match = SidfmdRuleCMatchApp.new(
                                            sidfmd_rule_cond_id: rule_cond.id,
                                            inv: false
                                            )
            if app_match.save then
              product_ids = cond_product_match["product-ids"]
              version_ids = cond_product_match["version-ids"]
              if product_ids.length > 0 then
                product_ids.each do |p_id|
                  SidfmdRuleCmAppSub.create(
                                     sidfmd_rule_c_match_app_id: app_match.id,
                                     product_id: p_id
                                     )
                end
              end
              if version_ids.length > 0 then
                version_ids.each do |v_id|
                  SidfmdRuleCmAppSub.create(
                                     sidfmd_rule_c_match_app_id: app_match.id,
                                     product_id: SidfmdVersion.find(v_id).sidfmd_product_id,
                                     version_id: v_id
                                    )
                end
              end
            end
          end
        end
        if cond_product_unmatch then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            app_unmatch = SidfmdRuleCMatchApp.new(
                                              sidfmd_rule_cond_id: rule_cond.id,
                                              inv: true
                                              )
            if app_unmatch.save then
              product_ids = cond_product_unmatch["product-ids"]
              version_ids = cond_product_unmatch["version-ids"]
              if product_ids.length > 0 then
                product_ids.each do |p_id|
                  SidfmdRuleCmAppSub.create(
                                     sidfmd_rule_c_match_app_id: app_unmatch.id,
                                     product_id: p_id
                                     )
                end
              end
              if version_ids.length > 0 then
                version_ids.each do |v_id|
                  SidfmdRuleCmAppSub.create(
                                     sidfmd_rule_c_match_app_id: app_unmatch.id,
                                     product_id: SidfmdVersion.find(v_id).sidfmd_product_id,
                                     version_id: v_id
                                    )
                end
              end
            end
          end
        end
        if cond_cvss_exploitable then
          rule_cond = SidfmdRuleCond.new(sidfmd_rule_id: rule_id)
          if rule_cond.save then
            SidfmdRuleCCvssExploitable.create(
                                       sidfmd_rule_cond_id: rule_cond.id,
                                       value: cond_cvss_exploitable["value"] == "true" ? true : false,
                                       is_change: cond_cvss_exploitable["is_change"] == "true" ? true : false
                                       )
          end
        end
        #操作作成
        current_user_id = User.current.id
        action_send_mail_category = operation["send-mail-category"]
        action_send_mail_user = operation["send-mail-user"]
        action_status_change = operation["status-change"]
        action_due_date_change = operation["due-date-change"]
        if action_send_mail_category then
          sender = action_send_mail_category["sender"]
          action_target = action_send_mail_category["target"]
          action_flg = action_send_mail_category["proc_empty"]
          action_send_mail_category["sendby"]["ids"].each do |id|
            rule_action = SidfmdRuleAction.new(
                                           sidfmd_rule_id: rule_id,
                                           target: action_target.to_i,
                                           proc_empty: action_flg == "true" ? true : false
                                          )
            if rule_action.save then
              SidfmdRuleAMailCategory.create(
                                      sidfmd_rule_action_id: rule_action.id,
                                      sidfmd_category_id: id == 'all' ? nil : id.to_i,
                                      from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                                      from_email: sender["address"] ? sender["address"] : nil,
                                      subject: action_send_mail_category["title"],
                                      body: action_send_mail_category["content"]
                                     )
            end
          end
        end
        if action_send_mail_user then
          sender = action_send_mail_user["sender"]
          action_target = action_send_mail_user["target"]
          action_flg = action_send_mail_user["proc_empty"]
          if action_send_mail_user["sendby"]["ids"] then
            action_send_mail_user["sendby"]["ids"].each do |id|
              rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
              if rule_action.save then
                SidfmdRuleAMailUser.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    user_id: id.to_i,
                                    from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                                    from_email: sender["address"] ? sender["address"] : nil,
                                    subject: action_send_mail_user["title"],
                                    body: action_send_mail_user["content"]
                                   )
              end
            end
          else
            action_send_mail_user["sendby"]["address"].each do |address|
              rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
              if rule_action.save then
                SidfmdRuleAMailUser.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    email: address,
                                    from_user_id: sender["userid"] ? sender["userid"].to_i : nil,
                                    from_email: sender["address"] ? sender["address"] : nil,
                                    subject: action_send_mail_user["title"],
                                    body: action_send_mail_user["content"]
                                   )
              end
            end
          end
        end
        if action_status_change then
          action_target = action_status_change["target"]
          action_flg = action_status_change["proc_empty"]
          rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
          if rule_action.save then
            SidfmdRuleAStatusChange.create(
                                    sidfmd_rule_action_id: rule_action.id,
                                    prev_status1: action_status_change["status_ids_from"][0].to_i,
                                    prev_status2: action_status_change["status_ids_from"][1] ? action_status_change["status_ids_from"][1].to_i : nil,
                                    status_id: action_status_change["status_ids_to"][0].to_i
                                   )
          end
        end
        if action_due_date_change then
          action_target = action_due_date_change["target"]
          action_flg = action_due_date_change["proc_empty"]
          rule_action = SidfmdRuleAction.new(
                                             sidfmd_rule_id: rule_id,
                                             target: action_target.to_i,
                                             proc_empty: action_flg == "true" ? true : false
                                            )
          if rule_action.save then
            SidfmdRuleADuedateChange.create(
                                     sidfmd_rule_action_id: rule_action.id,
                                     permit_update: action_due_date_change["permit-update"] == "true" ? true : false,
                                     fixed_date: action_due_date_change["date"] ? action_due_date_change["date"].to_date : nil,
                                     refdt_days: action_due_date_change["days"] ? action_due_date_change["days"].to_i : nil
                                     )
          end
        end
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'ルールの登録に失敗しました。' and return
    end
    return true
  end

  def get_users_custom_categories
    if @sidfm_roles.include?(:admin) then
      c_categories = @all_sidfm_categories.map{|c| [c.id, c.name]}
    else
      c_categories = User.current
                         .sidfmd_categories
                         .where(:is_system => false)
                         .map{|c| [c.id, c.name]}
    end
    return c_categories.sort{|a, b| a[1] <=> b[1]}.to_h
  end

  #ソート
  def sort_rules
    order_param = @rules[:order].split("_")
    col = order_param[0]
    order = order_param[1]
    case col
    when "rule-name" then
      col = "name"
    when "rule-category" then
      col = "sidfmd_category_id"
    when "rule-active" then
      col = "flg_enable"
    end
    case order
    when "asc"
      @all_rules = @all_rules.order(col => :asc).order(:name => :asc)
    when "desc"
      @all_rules = @all_rules.order(col => :desc).order(:name => :desc)
    end
  end

  #検索処理
  def search_items
    rule_name = @rules[:filters][:search_rulename]
    rule_category = @rules[:filters][:search_category]
    rule_trigger = @rules[:filters][:search_trigger]
    rule_cond = @rules[:filters][:search_cond]
    rule_action = @rules[:filters][:search_ope]
    rule_flg  = @rules[:filters][:search_flg]

    if rule_name && rule_name.length > 0 then
      @all_rules = @all_rules.where('name LIKE ?', "%#{rule_name}%")
    end

    if rule_category && rule_category.length > 0 then
      if rule_category == "-1" then
        @all_rules = @all_rules.where(:sidfmd_category_id => nil)
      else
        @all_rules = @all_rules.where(:sidfmd_category_id => rule_category)
      end
    end

    if rule_trigger && rule_trigger.length > 0 then
      case rule_trigger 
        when "periodic_hour" then trigger = SidfmdRuleTpHour
        when "periodic_interval" then trigger = SidfmdRuleTpInterval
        when "vulsync" then trigger = SidfmdRuleTVulsync 
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_trigger).where("sidfmd_rule_triggers.id IN (?)", trigger.all.map{|t| t.sidfmd_rule_trigger_id})
    end
    if rule_cond && rule_cond.length > 0 then
      case rule_cond 
        when "issue-update" then cond = SidfmdRuleCIssueUpdate.all
        when "cvss-exploitable" then cond = SidfmdRuleCCvssExploitable.all
        when "sri-above" then cond = SidfmdRuleCSriAbove.all
        when "sri-below" then cond = SidfmdRuleCSriBelow.all
        when "sri-warning" then cond = SidfmdRuleCWarning.where(:is_cvss => false)
        when "cvss-above" then cond = SidfmdRuleCCvssAbove.all
        when "cvss-below" then cond = SidfmdRuleCSriBelow.all
        when "cvss-warning" then cond = SidfmdRuleCWarning.where(:is_cvss => true)
        when "cvss-notreg" then cond = SidfmdRuleCCvssNotreg.all
        when "status-match" then cond = SidfmdRuleCStatus.all
        when "product-match" then cond = SidfmdRuleCMatchApp.where(:inv => false)
        when "product-unmatch" then cond = SidfmdRuleCMatchApp.where(:inv => true) 
        when "due-date-expired" then cond = SidfmdRuleCDuedateExpired.all
        when "due-date-approach" then cond = SidfmdRuleCDuedateApproaching.all
        when "history-create" then cond = SidfmdRuleCHistoryPost.all
        when "category-match" then cond = SidfmdRuleCMatchCategory.where(:inv => false)
        when "category-unmatch" then cond = SidfmdRuleCMatchCategory.where(:inv => true)
        when "host-match" then cond = SidfmdRuleCMatchHost.where(:inv => false)
        when "host-unmatch" then cond = SidfmdRuleCMatchHost.where(:inv => true)
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_conds).where("sidfmd_rule_conds.id IN (?)", cond.map{|c| c.sidfmd_rule_cond_id}).uniq
    end

    if rule_action && rule_action.length > 0 then
      case rule_action
        when "send-mail-user" then action = SidfmdRuleAMailUser
        when "send-mail-category" then action = SidfmdRuleAMailCategory
        when "status-change" then action = SidfmdRuleAStatusChange
        when "due-date-change" then action = SidfmdRuleADuedateChange
      end
      @all_rules = @all_rules.joins(:sidfmd_rule_actions).where("sidfmd_rule_actions.id IN (?)", action.all.map{|a| a.sidfmd_rule_action_id}).uniq
    end
    if rule_flg && rule_flg.length > 0 then
      if rule_flg == "enabled" then
        @all_rules = @all_rules.where(:flg_enable => true)
      else 
        @all_rules = @all_rules.where(:flg_enable => false)
      end
    end
  end

  #データ作成
  def create_content
    rules = []
    @all_rules.each do |rule|
      rule_name = rule.name
      rule_category = rule.sidfmd_category_id
      is_enable = rule.flg_enable
      trigger = rule.sidfmd_rule_trigger.detail
      trigger_name = trigger.class.table_name
      conditions = rule.sidfmd_rule_conds
      actions = rule.sidfmd_rule_actions
      rule_info  = {
        :rule_id => rule.id,
        :name => rule_name,
        :category_name => rule_category.nil? ? '全体' : @all_sidfm_categories.find(rule_category.to_i).name,
        :trigger => if trigger_name == "sidfmd_rule_tp_hours" then
                      ["毎日実行-時間指定", "実行時間: #{trigger.hour.to_s}時"]
                    elsif trigger_name == "sidfmd_rule_tp_intervals" then
                       unit = trigger.unit
                       case unit
                       when "hour" then
                         unit = "時間毎"
                       when "day" then
                         unit = "日毎"
                       when "week" then
                         unit = "週毎"
                       when "month" then
                         unit = "ヵ月毎"
                       end
                       ["定期実行-間隔指定", "実行間隔: #{trigger.value.to_s + unit}"]
                    else
                      category_id = trigger.sidfmd_category_id
                      host_id = trigger.sidfmd_host_id
                      trigger_text = if !category_id && !host_id then
                                       "対象カテゴリ: 全体"
                                     elsif host_id then
                                       "対象ホスト: #{SidfmdHost.find(host_id).hostname}"
                                     else 
                                       "対象カテゴリ: #{@all_sidfm_categories.find(category_id).name}"
                                     end
                      ["脆弱性同期時 実行", trigger_text]
                    end,
        :cond_num => conditions.length,
        :ope_num => actions.map{|a| a.detail.class.table_name}.uniq.size,
        :logs => rule.logs,
        :enabled => is_enable,
        :param => create_rule_param(rule_name, rule_category, is_enable, trigger, conditions, actions)
      }
      rules << rule_info
    end
    return rules
  end

  def get_preset_rules_data
    status_not_conf = SidfmdIssue::STATUS_NOTCONFIRMED
    status_on_going = SidfmdIssue::STATUS_ONGOING
    status_not_affect = SidfmdIssue::STATUS_NOTAFFECT
    sri_high = SidfmdIssueSri::VALUE_HIGH
    sri_low = SidfmdIssueSri::VALUE_LOW
    target_change = SidfmdRuleAction::TARGET_CHANGE
    target_vuls = SidfmdRuleAction::TARGET_VULS
    preset_mail_sender = Setting.mail_from
    user_category_ids = @c_categories.keys
    #報告者権限の場合は、ユーザのカテゴリのカテゴリ名でsortした初めのカテゴリをセット
    rule_category_id = @sidfm_roles.include?(:admin) ? 'all' : (user_category_ids.size > 0 ? user_category_ids[0] : '')
    rule_params = [
      {
        "rule_name" => "SRI値が「高」以上の脆弱性をメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" =>{"hostname"=>"", "period"=>"vulsync","value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
        "condition" =>{
            "sri-above" => sri_high,
            "status-match" => [status_not_conf, status_on_going]
        },
        "action" =>{
          "send-mail-category" =>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "SRI値が「高」以上の脆弱性",
            "content" => "新たに検出された脆弱性は以下の通りです。\n\n登録された脆弱性\n%VULSYNC_REGI_MATCH:T3%\n他、%VULSYNC_REGI_MATCH:C-3%件\n\n更新された脆弱性\n%VULSYNC_RENEW_MATCH:T3%\n他、%VULSYNC_RENEW_MATCH:C-3%件\n\n削除された脆弱性\n%VULSYNC_DEL_MATCH:T3%\n他、%VULSYNC_DEL_MATCH:C-3%件",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_change.to_s,
            "proc_empty" => "false"
          }
        },
        "description" => "脆弱性同期時に、SRI値が「高」または「重大」の脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "CVSS値が 7 以上の脆弱性をメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"vulsync","value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
        "condition" => {
          "cvss-above" => {"val"=>7.0, "include_notreg"=>"false", "flg_exploitable"=>"all"},
          "status-match" => [status_not_conf, status_on_going]
        },
        "action" =>{
          "send-mail-category" =>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "CVSS値が7以上の脆弱性",
            "content" => "新たに検出された脆弱性は以下の通りです。\n\n登録された脆弱性\n%VULSYNC_REGI_MATCH:T3%\n他、%VULSYNC_REGI_MATCH:C-3%件\n\n更新された脆弱性\n%VULSYNC_RENEW_MATCH:T3%\n他、%VULSYNC_RENEW_MATCH:C-3%件\n\n削除された脆弱性\n%VULSYNC_DEL_MATCH:T3%\n他、%VULSYNC_DEL_MATCH:C-3%件",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_change.to_s,
            "proc_empty" => "false"
          }
        },
        "description" => "脆弱性同期時に、CVSS値が 7.0 以上の脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "7 日毎に未対策の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger"=>{"hostname"=>"", "period"=>"periodic-interval", "value"=>7, "unit"=>"day", "category"=>""},
        "condition"=>{"status-match"=>[status_not_conf]},
        "action"=>{
          "send-mail-category"=>{
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "未対策の脆弱性",
            "content" => "未対策の脆弱性は以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "7 日毎に、ステータスが「未対策」となっている脆弱性が記載されたメールを送信します。"
      },
      {
        "rule_name" => "7 日毎に更新された脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-interval", "value"=>7, "unit"=>"day", "category"=>""},
        "condition" => {"history-create" => {"val" => 1}},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "更新された脆弱性チケット",
            "content" => "更新された脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "7 日毎に、脆弱性チケットの更新が１件以上ある脆弱性を記載したメールを送信します。"
      },
      {
        "rule_name" => "対策予定日を過ぎた未対策の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-hour", "value"=>3, "unit"=>"", "category"=>""},
        "condition" => {"due-date-expired" => "due-date-expired"},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "対策予定日を過ぎた未対策の脆弱性チケット",
            "content" => "対策予定日を過ぎた未対策の脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address" => preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "毎日午前 3 時に、対策予定日を過ぎた未対策の脆弱性チケットが記載されたメールを送信します。"
      },
      {
        "rule_name" => "対策予定日が 5 日以内の脆弱性チケットをメールで通知する",
        "is_enable" => true,
        "category" => rule_category_id,
        "trigger" => {"hostname"=>"", "period"=>"periodic-hour", "value"=>3, "unit"=>"", "category"=>""},
        "condition" => {"due-date-approach"=>{"val"=>5}},
        "action" => {
          "send-mail-category" => {
            "sendby" => {"ids"=>[rule_category_id]},
            "title" => "対策予定日が 5 日以内の脆弱性チケット",
            "content" => "対策予定日が 5 日以内の脆弱性チケットは以下の通りです。\n\n%VUL_LINKS_MATCH:T3%\n\nその他、%VUL_LINKS_MATCH:C-3%件検出されています。",
            "sender" => {"address"=> preset_mail_sender},
            "target" => target_vuls.to_s,
            "proc_empty" => "true"
          }
        },
        "description" => "毎日午前 3 時に、対策予定日が 5 日以内に設定されている脆弱性チケットが記載されたメールを送信します。"
      },
     {
       "rule_name" => "SRI値が「低」の脆弱性チケットを「影響無」にする", 
       "is_enable" => true, 
       "category" => rule_category_id,
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "sri-below" => sri_low,
         "status-match" => [status_not_conf, status_on_going]
       }, 
       "action" => {
         "status-change" => {
           "status_ids_from" => [status_not_conf, status_on_going], 
           "status_ids_to" => [status_not_affect],
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、SRI値が「低」となっている脆弱性チケットのステータスを「影響無」に変更します。"
     },
     {
       "rule_name" => "CVSS値が 3.9 以下の脆弱性チケットを「影響無」に変更する",
       "is_enable" => true,
       "category" => rule_category_id,
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "cvss-below" => {"val"=>3.9, "include_notreg"=>"false", "flg_exploitable"=>"all"},
         "status-match" => [status_not_conf, status_on_going]
       },
       "action" => {
         "status-change" => {
           "status_ids_from" => [status_not_conf, status_on_going], 
           "status_ids_to" => [status_not_affect],
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、CVSS値が 3.9 以下の脆弱性チケットのステータスを「影響無」に変更します。"
     },
     {
       "rule_name" => "SRI値が「高」以上の脆弱性チケットに対策予定日として 20 日後を設定する",
       "is_enable"=> true,
       "category" => rule_category_id,
       "trigger"=> {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=> rule_category_id},
       "condition" => {
         "sri-above" => sri_high,
         "status-match" => [status_not_conf, status_on_going]
       },
       "action" => {
         "due-date-change" => {
           "days" => 20, 
           "permit-update" => "false",
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、SRI値が「高」または「重大」となっている脆弱性チケットの対策予定日を 20 日後に設定します。"
     },
     {
       "rule_name" => "CVSS値が 7.0 以上の脆弱性チケットに対策予定日として 20 日後を設定する",
       "is_enable" => true, 
       "category" => rule_category_id, 
       "trigger" => {"hostname"=>"", "period"=>"vulsync", "value"=>"vulsync", "unit"=>"", "category"=>rule_category_id}, 
       "condition" => {
         "cvss-above" => {"val"=>7.0, "include_notreg"=>"false", "flg_exploitable"=>"all"},
         "status-match" => [status_not_conf, status_on_going]
       }, 
       "action" => {
         "due-date-change"=>{
           "days"=>20,
           "permit-update"=>"false",
           "target" => target_change.to_s,
           "proc_empty" => "true"
         }
       },
       "description" => "脆弱性同期時に、CVSS値が 7.0 以上の脆弱性の対策予定日を 20 日後に設定します。"
     }
    ]
  end

  def get_mail_preset
    list_values = SidfmdRuleAction::TARGETS
    preset_list = [
     {
       :text => ["条件に適合したホストの一覧（最大10件）を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[1],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合するホストの一覧",
       :message => "ルール %RULE_NAME% の条件に適合するホストは以下の通りです。\n\n" +
                   "対象ホスト数: %HOST_MATCH:C% 件\n\n" +
                   "ホスト一覧(最大10件):\n" +
                   "%HOST_MATCH:T10%",
     },
     {
       :text => ["条件に適合したホストの一覧（最大10件）を送信する","(データが存在しない場合でも、メールを送信する)"],
       :target => list_values[1],
       :proc_empty => true,
       :subject => "%RULE_NAME%の条件に適合するホストの一覧",
       :message => "ルール %RULE_NAME% の条件に適合するホストは以下の通りです。\n\n" +
                   "対象ホスト数: %HOST_MATCH:C% 件\n\n" +
                   "ホスト一覧(最大10件):\n" +
                   "%HOST_MATCH:T10%"
     },
     {
       :text => ["条件に適合した脆弱性の一覧（最大10件）を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[0],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULINFO_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大10件):\n" +
                   "%VULINFO_MATCH:T10%"
     },
     {
       :text => ["条件に適合した脆弱性の一覧（最大10件）を送信する", "(データが存在しない場合でも、メールを送信する)"],
       :target => list_values[0],
       :proc_empty => true,       
       :subject => "%RULE_NAME%の条件に適合する脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULINFO_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大10件):\n" +
                   "%VULINFO_MATCH:T10%"
     },
     {
       :text => ["条件に適合した新規登録脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[2],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する新規登録脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する新規登録の脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_REGI_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_REGI_MATCH:T100%",
       :trigger => "vulsync" 
     },
     {
       :text => ["条件に適合した更新脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],    
       :target => list_values[3],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する更新脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する更新脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_RENEW_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_RENEW_MATCH:T100%",
       :trigger => "vulsync"

     },
     {
       :text => ["条件に適合した削除脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[4],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する削除脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する削除脆弱性は以下の通りです。\n\n" +
                   "対象脆弱性数: %VULSYNC_DEL_MATCH:C% 件\n\n" +
                   "脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_DEL_MATCH:T100%",
       :trigger => "vulsync"
     },
     {
       :text => ["条件に適合した登録・更新・削除脆弱性を送信する", "(データが存在しない場合、メールを送信しない)"],
       :target => list_values[5],
       :proc_empty => false,
       :subject => "%RULE_NAME%の条件に適合する登録・更新・削除脆弱性の一覧",
       :message => "ルール %RULE_NAME% の条件に適合する登録・更新・削除登録の脆弱性は以下の通りです。\n\n" +    
                   "対象登録脆弱性数: %VULSYNC_REGI_MATCH:C% 件\n" +
                   "対象更新脆弱性数: %VULSYNC_RENEW_MATCH:C% 件\n" +
                   "対象削除脆弱性数: %VULSYNC_DEL_MATCH:C% 件\n" +
                   "登録脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_REGI_MATCH:T100%\n\n" +
                   "更新脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_RENEW_MATCH:T100%\n\n" +
                   "削除脆弱性一覧(最大100件):\n" +
                   "%VULSYNC_DEL_MATCH:T100%",
       :trigger => "vulsync"
     }
    ]

  end

  public

  def index 
    filtered_params = params.fetch(:pages, {}).permit(:offset, :per_page, :order, :filters, :checked, :all_checked)
    @rules = {}
    @rules[:per_page] = (filtered_params[:per_page] || 10).to_i
    @rules[:offset] = (filtered_params[:offset] || 0).to_i
    @rules[:filters] = JSON.parse(filtered_params[:filters] || '{}').symbolize_keys
    @rules[:checked] = JSON.parse(filtered_params[:checked] || '[]')
    @rules[:all_checked] = ((filtered_params[:all_checked] || '') == 'true')
    @rules[:order] = (filtered_params[:order] || 'rule-name_asc')
    @rules[:search_rulename] = @rules[:filters][:search_rulename] || ''
    @rules[:search_category] = @rules[:filters][:search_category] || ''
    @rules[:search_trigger] = @rules[:filters][:search_trigger] || ''
    @rules[:search_cond] = @rules[:filters][:search_cond] || ''
    @rules[:search_ope] = @rules[:filters][:search_ope] || ''
    @rules[:search_flg] = @rules[:filters][:search_flg] || ''
    if @sidfm_roles.include?(:admin) then
      @all_rules = SidfmdRule.all
    else
      @all_rules = SidfmdRule.where(:sidfmd_category_id => @user_custom_category_ids)
    end
    @all_sidfm_categories = SidfmdCategory.all.where(:is_system => false)
    #一括処理やデータ複製機能などのためにfilter前のレコードでデータ取得
    @rules_data = []
    @all_rules.each do |rule|
      rule_name = rule.name
      rule_category = rule.sidfmd_category_id
      is_enable = rule.flg_enable
      trigger = rule.sidfmd_rule_trigger.detail
      conditions = rule.sidfmd_rule_conds
      actions = rule.sidfmd_rule_actions
      @rules_data << {
        :rule_id => rule.id,
        :param => create_rule_param(rule_name, rule_category, is_enable, trigger, conditions, actions)
      }
    end
    #フィルター
    sort_rules
    @all_rule_ids = @all_rules.map{|r| r.id} #全ルールのidをlimit前に取得
    @all_rules = @all_rules.offset(@rules[:offset]).limit(@rules[:per_page])
    search_items
    @rules[:total_size] = @all_rules.except(:order, :offset, :limit).count
    @rules[:content] = create_content #フィルター後のデータ
    @c_categories = get_users_custom_categories()
    #@accessible_users = get_accessible_users()
    @all_triggers = SidfmdRule::RULE_TRIGGER_NAMES
    @all_conds = SidfmdRule::RULE_COND_NAMES
    @all_opes = SidfmdRule::RULE_ACTION_NAMES
    #最大登録件数
    @maximum_limit = SidfmdRule::MAXIMUM_LIMIT
    #プリセット一覧
    @data_for_preset = get_preset_rules_data
    #メールのプリセット一覧
    @data_for_mail_preset = get_mail_preset
    #ブラウザバックかどうか判断するための処理 チェックボックスの描画のバグを防ぐために使用
    if !session['chkbok_ids'] then
      session['chkbok_ids'] = {}
    end
    if !session['chkbok_ids']['rule'] then
      session['chkbok_ids']['rule'] = {}
    end
    session['chkbok_ids']['rule']['is_browser_back'] = false
  end

  def create
    if SidfmdRule.count == 100 then
      redirect_to :action => :index, :notice => "ルールの登録は#{SidfmdRule::MAXIMUM_LIMIT}件までとなっています" and return
    end
    f_params = params.fetch(:form, {}).permit(:rulename, :category, :flg, :trigger, :condition, :operation)
    dump = {} 
    dump[:action] = :index
    begin 
      rulename = f_params[:rulename]
      category = f_params[:category]
      flg = f_params[:flg]
      trigger = JSON.parse(f_params[:trigger])
      condition = JSON.parse(f_params[:condition]) 
      operation = JSON.parse(f_params[:operation]) 
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:notice] = "入力が不正です。"
      redirect_to dump and return
    end
    if !rulename || !category || !flg || !trigger || !condition || !operation then
      dump[:notice] = "ルールの登録に失敗しました。"
      redirect_to dump and return
    end
    if category == "all" then
      category = nil
    else
      c_id = category.to_i
      if @user_custom_category_ids.include?(c_id) then
        category = c_id
      else
        dump[:notice] = "入力が不正です。"
        redirect_to dump and return
      end
    end
    if SidfmdRule.where(:name => rulename).length > 0 then
      dump[:notice] = "既に同名のルールが存在します"
      redirect_to dump and return
    end
    create_rule = set_rule(rulename, category, flg, trigger, condition, operation, dump, 'create', nil)
    if create_rule == true then
      Rails.cache.clear
      redirect_to :action => :index and return
    end
  end

  def modify
    f_params = params.fetch(:form, {}).permit(:rule_id, :rulename, :category, :flg, :trigger, :condition, :operation, :offset, :per_page, :filters, :order)
    dump = {}
    dump[:action] = :index
    begin
      rule_id = f_params[:rule_id]
      rulename = f_params[:rulename]
      category = f_params[:category]
      flg = f_params[:flg]
      trigger = JSON.parse(f_params[:trigger])
      condition = JSON.parse(f_params[:condition])
      operation = JSON.parse(f_params[:operation])
    rescue JSON::ParserError => e
      log_exception(e)
      dump[:notice] = "入力が不正です。"
      redirect_to dump and return
    end
    if !rule_id || !rulename || !category || !flg || !trigger || !condition || !operation then
      dump[:notice] = "ルールの編集に失敗しました。"
      redirect_to dump and return
    end
    if category == "all" then
      category = nil
    else
      c_id = category.to_i
      if @user_custom_category_ids.include?(c_id) then
        category = c_id
      else
        dump[:notice] = "入力が不正です。"
        redirect_to dump and return
      end
    end
    edit_rule = set_rule(rulename, category, flg, trigger, condition, operation, dump, 'modify', rule_id)
    if edit_rule then
      Rails.cache.clear
      dump[:'pages[offset]'] = f_params[:offset].to_i
      dump[:'pages[per_page]'] = f_params[:per_page].to_i
      dump[:'pages[filters]'] = f_params[:filters] || '{}'
      dump[:'pages[order]'] = f_params[:order]
      redirect_to dump and return
    end
  end

  def delete
    begin 
      f_params = params.fetch(:form, {}).permit(:delete_id)
      delete_id = f_params[:delete_id].to_i
      SidfmdRule.find(delete_id).destroy
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => '削除に失敗しました' and return 
    end
    Rails.cache.clear
    redirect_to :action => :index and return
  end

  def bulk_delete
    begin 
      f_params = params.fetch(:form, {}).permit(:delete_ids)
      delete_ids = JSON.parse(f_params[:delete_ids])
    rescue JSON::ParserError => e
      log_exception(e)
      redirect_to :action => :index, :notice => '入力が不正です' and return
    end
    begin 
      SidfmdRule.transaction do 
        SidfmdRule.where(:id => delete_ids).destroy_all
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => '削除に失敗しました' and return
    end 
    Rails.cache.clear
    redirect_to :action => :index and return
  end

end  


